#include "StdVarTyp.h"
#include "CErrEStd.h"
#include "CFamilyEnum.h"
#include "Wacp.h"
#include "CSerialCommSvr.h"
#include "CEthCommSvr.h"
#include "CDeviceDStd.h"
#include "CErrEStd.h"
#include "CModulePSelector.h"
#include "CModuleDStd.h"


#if defined STACK_SERIAL
extern CSerialCommSvr CommSvr;
#else if STACK_ETHERNET
extern CEthCommSvr CommSvr;
#endif
extern boolean bConnected;
uint8 OutBuffer[MAX_PACKET_SIZE];

/*================================================================================
SendMsg: Encapuslates the optional serialized object into the a message
envelope and sends out of the transport.

Parameters:
    MsgClassID:    32 Bit value that contains the Family, Genus, and Species of 
                   the outgoing message. 
    pBuf:          Pointer to the binary MOIB object stream. This may be NULL.
    BufSize:       Size of the buffer pointed to by pBuf. This size may be 0.
    ErrClassID:    ClassID of the message that is being ACK'd or NAK'd. This
                   may be 0 for non-trap messages
Returns: 

    TRUE - If the message was successfully sent, otherwise FALSE.
================================================================================*/
uint32 SendMsg(uint32 MsgClassID, uint8* pBuf, uint32 BufSize, uint32 ErrClassID)
{
    uint32 Result = 0;
    uint32 OutSize = 0;

    Result = GeneratePacket(OutBuffer, &OutSize, pBuf, BufSize, MsgClassID, ErrClassID);
    if (Result == WACP_SUCCESS)
    {
#if defined STACK_SERIAL
        CSerialCommSvr_SendMessage(&CommSvr, OutBuffer, OutSize);
#else if STACK_ETHERNET
        CEthCommSvr_SendMessage(&CommSvr, OutBuffer, OutSize);
#endif
    }

    return Result;
}

/*================================================================================
RecvMessage: Main callback method for receiving a WACP Data or connection message.
    
Parameters:
    pSourceID:     SourceID of the incoming message.
    ClassID:       32 Bit value that contains the Family, Genus, and Species of 
                   the incoming message. 
    pBuffer:       Pointer to the binary MOIB object stream. This may be NULL.
    BufSize        Size of the buffer pointed to by pBuffer. This size may be 0.

Returns: 

    TRUE - If the message was successfully processed, otherwise FALSE.

================================================================================*/
boolean RecvMessage(const char *pSourceID, uint32 ClassID, uint8 *pBuffer, uint32 BufSize)
{
	boolean bResult = FALSE;
	uint16  ErrorCode  = CERR_PROC_UNIMPL;
	uint16  Family  = (uint16)(ClassID >> 16);
	uint8   Genus = (uint8)(ClassID >> 8);
	uint8   Species = (uint8)(ClassID);
    TOUCH(pSourceID);
    TOUCH(BufSize);

    switch(Family)
    {
		case FmMODULE:
        {
			switch(Genus)
			{
				case GnCOMMAND:
				{
					switch(Species)
					{
						case MODULE_CCMD_SpWRITE_CONFIG:
							break;
						case MODULE_CCMD_SpWRITE_MODULE_DESC:
							break;
						default:
							ErrorCode = CERR_PROC_UNSUPPORTED;
							break;
					}
					break;
				}
				case GnREQUEST:
				{
					switch(Species)
					{
						case MODULE_CRST_SpGET_MODULE_DESC:
                        {
                            /*Received a Module Description Request and the expected payload is: 
                            Object Structure: [CModulePSelector]
                            1. CModulePSelector - Indicates the module type requested */

                            SYS_PRINTF("Received a Module Description Request\n");

							if(pBuffer != NULL)
							{
								CModuleDStd ModuleDStd;
								CModulePSelector ModulePSelector;
								uint8    *ObjBuffer = NULL;
								uint32   ObjSize = 0;
					
								/*Reset object fields*/
								CModuleDStd_Init(&ModuleDStd);
								CModulePSelector_Init(&ModulePSelector);

                                /*Deserialize the serialized object into CModulePSelector*/
								if(CModulePSelector_Deserialize(&ModulePSelector, pBuffer))
								{
                                    /*Switch on the SpeciesSelector that indicates the module
                                    type requested*/
									switch (ModulePSelector.m_SpeciesSelector)
									{
										char* verStr; 
										char* desStr; 
										
										case CMODDSTD_SpHARDWARE:
										{
                                            verStr = "1.33.7";       //Sample hardware version
                                            desStr = "CO2 Hardware"; //Sample hardware description

											/*Setting sample values for Hardware version*/
											if(sizeof(ModuleDStd.m_Version) >= strlen(verStr)+1)
											{
												memcpy(ModuleDStd.m_Version, verStr, strlen(verStr)+1);
											}												

											/*Setting sample values for Hardware description*/
											if(sizeof(ModuleDStd.m_Description) >= strlen(desStr)+1)
											{
												memcpy(ModuleDStd.m_Description, desStr, strlen(desStr)+1 );
											}

											/*Calculate the size of the object*/
											ObjSize = CModuleDStd_SizeAll(&ModuleDStd);

											/*Allocate the buffer to the size of the object*/
											ObjBuffer = (uint8*)SYS_MALLOC(ObjSize);

											if(ObjBuffer != NULL)
											{
												/*Serialize the object into the buffer*/
												CModuleDStd_Serialize(&ModuleDStd ,ObjBuffer);	

												/*Sending request message*/
												SendMsg(MakeClassID(FmMODULE,						/*Family: FmMODULE*/
																	GnRESPONSE,						/*Genus:  GnREQUEST*/
																	MODULE_CRSP_SpPUT_MODULE_DESC),	/*Species:MODULE_CRSP_SpPUT_MODULE_DESC*/																															
														ObjBuffer,									/*Serialized CModuleDStd object*/
														ObjSize,									/*Size of Serialized CModuleDStd object*/
														0);									     	/*ErrorClsID: 0*/
												

												/*Clear up memory*/
												SYS_FREE(ObjBuffer);
												ObjBuffer = NULL;
											}
								
											break;
										}
										case CMODDSTD_SpBOOTLOADER:
										{
                                            verStr = "1.33.7";            //Sample bootloader version
                                            desStr = "CO2 Bootloader";    //Sample bootloader description

											/*Setting sample values for Bootloader Version*/												
											if(sizeof(ModuleDStd.m_Version) >= strlen(verStr)+1)
											{
												memcpy( ModuleDStd.m_Version, verStr, strlen(verStr)+1 );
											}

											/*Setting sample values for Bootloader description*/												
											if(sizeof(ModuleDStd.m_Description) >= strlen(desStr)+1)
											{
												memcpy(ModuleDStd.m_Description, desStr, strlen(desStr)+1 );
											}

											/*Calculate the size of the object*/
											ObjSize = CModuleDStd_SizeAll(&ModuleDStd);

											/*Allocate the buffer to the size of the object*/
											ObjBuffer = (uint8*)SYS_MALLOC(ObjSize);

											if(ObjBuffer != NULL)
											{
												/*Serialize the object into the buffer*/
												CModuleDStd_Serialize(&ModuleDStd ,ObjBuffer);	

												/*Sending request message*/
												SendMsg(MakeClassID(FmMODULE,						/*Family: FmMODULE*/
																	GnRESPONSE,						/*Genus:  GnREQUEST*/
																	MODULE_CRSP_SpPUT_MODULE_DESC),	/*Species:MODULE_CRSP_SpPUT_MODULE_DESC*/																														
														ObjBuffer,									/*Serialized CModuleDStd object*/
														ObjSize,									/*Size of Serialized CModuleDStd object*/
														0);									     	/*ErrorClsID: 0*/
												
												/*Clear up memory*/
												SYS_FREE(ObjBuffer);
												ObjBuffer = NULL;
											}
											break;
										}
                                        default:
							                ErrorCode = CERR_PROC_UNSUPPORTED;
                                            SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRMSGPROC), NULL, 0, ClassID);
							                break;
									}
											
								}
							}
                        }  
						default:
							ErrorCode = CERR_PROC_UNSUPPORTED;
							break;                           
					}
					break;
				}
				case GnRESPONSE:
				{
					switch(Species)
					{
						case MODULE_CRSP_SpPUT_MODULE_DESC:
							break;
						default:
							ErrorCode = CERR_PROC_UNSUPPORTED;
							break;
					}
					break;
				}
				case GnDATA:
				{
					switch(Species)
					{
						case CMODDSTD_SpSTANDARD:
							break;
						case CMODDSTD_SpHOST:
							break;
						case CMODDSTD_SpHARDWARE:
							break;
						case CMODDSTD_SpBOOTLOADER:
							break;
						case CMODDSTD_SpBOOTRECOVERY:
							break;
						case CMODDSTD_SpNIBPMODULE:
							break;
						case CMODDSTD_SpNIBPSAFETYALG:
							break;
						case CMODDSTD_SpNIBPSTEPALG:
							break;
						case CMODDSTD_SpNIBPFASTALG:
							break;
						case CMODDSTD_SpNIBPNEONATSTEPALG:
							break;
						case CMODDSTD_SpNIBPPOEMALG:
							break;
						case CMODDSTD_SpNIBPSTEPAOALG:
							break;
						case CMODDSTD_SpTEMPMODULE:
							break;
						case CMODDSTD_SpSURETEMPALG:
							break;
						case CMODDSTD_SpSPO2MODULE:
							break;
						case CMODDSTD_SpECGMODULE:
							break;
						case CMODDSTD_SpMEANS:
							break;
						case CMODDSTD_SpSPIROMETRY:
							break;
						case CMODDSTD_SpFRONTEND:
							break;
						case CMODDSTD_SpPPP:
							break;
						case CMODDSTD_SpSECUREDIGITALCARD:
							break;
						case CMODDSTD_SpALL:
							break;
						default:
							ErrorCode = CERR_PROC_UNSUPPORTED;
							break;
					}
					break;
				}
				default:
					ErrorCode = CERR_PROC_UNSUPPORTED;
					break;
            }
		}
        case FmDEVICE:
        {
            switch(Genus)
            {
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION:
                        {
                            /*Received a Device Description Response and the expected payload is: 
                            Object Structure: [CDeviceDStd]
                            1. CDeviceDStd - Contains basic device description information */

                            SYS_PRINTF("Received a Device Description Response\n");
                            
                            if(pBuffer != NULL)
                            {
                                CDeviceDStd NewObj;
                                ansichar GUIDBuf[GUID_STR_SIZE];

                                /*Initialize the object*/
                                CDeviceDStd_Init(&NewObj);
                
                                /*Deserialize the buffer into the device object*/
                                if (CDeviceDStd_Deserialize(&NewObj, pBuffer))
                                {                                
                                    SYS_PRINTF("ModelName: %s\n", NewObj.m_ModelName);
                                    SYS_PRINTF("SerialNumber: %s\n", NewObj.m_SerialNumber);
                                    SYS_PRINTF("Device GUID: %s\n", GUIDToStr(NewObj.m_DeviceGUID, GUIDBuf, sizeof(GUIDBuf)));
                                }
                            }
                        }
                    }
                }
            }
        }
        case FmTEMP:
        {
            break;
        }
        case FmRENDEZVOUS:
        {
#if defined(WACP_DEBUG_EXT)
            SYS_PRINTF("RecvMessage: FmRENDEZVOUS\n");
#endif
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    /* RNDZ_CCMD_SpSTART_HOST_PROC is sent from the client once RNDZ initialization is
                    * completed. 
                    */
                    if (Species == RNDZ_CCMD_SpSTART_HOST_PROC)
                    {
                        /* Send the HOST READY command and serialize the CRNDZDStd object to send */
                        SendMsg(MakeClassID(FmRENDEZVOUS, GnCOMMAND, RNDZ_CCMD_SpSEND_HOST_READY), NULL, 0, 0);
#if defined(WACP_DEBUG)
                        SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CCMD_SpSTART_HOST_PROC. RNDZ Complete\n");
#endif
                        bConnected = TRUE;
                    }
                    /* RNDZ_CCMD_SpSEND_HOST_READY is sent from the host indicating a successful
                     * RNDZ authentication. This signals the client to terminate the RNDZ client
                     * and to start up the WACP session.
                     */
                    else if (Species == RNDZ_CCMD_SpSEND_HOST_READY)
                    {
                        /* RNDZ authentication complete */
#if defined(WACP_DEBUG)
                        SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CCMD_SpSEND_HOST_READY. RNDZ Complete\n");
#endif
                        bConnected = TRUE;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    if (Species == RNDZ_CRST_SpSTART_RNDZ)
                    {
                        if(strstr((const char*)pBuffer, "RNDZCONNECT"))
                        {   
#if defined(WACP_DEBUG)
                            SYS_PRINTF("RecvMessage: FmRENDEZVOUS - Recv'd RNDZ_CRST_SpSTART_RNDZ.\n");
#endif

                            /* Send the RNDZ_CRSP_SpRNDZ_ACCEPT Response */
                            SendMsg(MakeClassID(FmRENDEZVOUS, GnRESPONSE, RNDZ_CRSP_SpRNDZ_ACCEPT), NULL, 0, 0);
                        }
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    /* RNDZ_CRSP_SpRNDZ_ACCEPT is sent from the client indicating that the request
                     * to start RNDZ has been accepted.
                     */
                    if (Species == RNDZ_CRSP_SpRNDZ_ACCEPT)
                    {
                        /* If CRNDZDStd is supported, then serialize the object that contains
                         * the GUID and send it out */
                        /* CRNDZDStd_Serialize(&GUIDStd, pObjBuffer); */

                        /* Send the START HOST PROC command */
                        SendMsg(MakeClassID(FmRENDEZVOUS, GnCOMMAND, RNDZ_CCMD_SpSTART_HOST_PROC), NULL, 0, 0);
                    }
                    break;
                }
            }
            break;
        }
        case FmCONNECTION:
        {
#if defined(WACP_DEBUG_EXT)
            SYS_PRINTF("RecvMessage: FmCONNECTION\n");
#endif
            if (Species == CONN_CMD_SpSHUTDOWN)
            {
#if defined(WACP_DEBUG)
                SYS_PRINTF("RecvMessage: FmCONNECTION - CONN_CMD_SpSHUTDOWN\n");
#endif
                /* Handle Shutdown sequence */
            }
            break;
        }
        default:
        {
            ErrorCode = CERR_PROC_UNIMPL;
            SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRMSGPROC), NULL, 0, ClassID);
            break;
        }
    }

    return bResult;
}

/*================================================================================
RecvError: Main callback method for receiving an Error message.
    
Parameters:
    pSourceID:     SourceID of the incoming message.
    ErrorType:     ACK or NAK identifier. If this is an ACK, the identifier will 
                   be CERR_SpERRNONE (0).
    ClassID:       32 Bit value that contains the Family, Genus, and Species of 
                   the incoming message. 
    pBuffer:       Pointer to the binary CErrEStd object stream. This may be NULL.
    BufSize        Size of the buffer pointed to by pBuffer. This size may be 0.

Returns: 

    TRUE - If the message was successfully processed, otherwise FALSE.

================================================================================*/
boolean RecvError(const char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer, uint32 BufSize)
{
    boolean Result  = TRUE;
    uint16  Family  = (uint16)(ClassID >> 16);
    uint8   Genus   = (uint8)(ClassID >> 8);
    uint8   Species = (uint8)ClassID;

    TOUCH(pSourceID);
    TOUCH(BufSize);
    TOUCH(pBuffer);

    if(ErrorType == CERR_SpERRNONE)
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("RxError: ACK F:%d G:%d S:%d\n", Family, Genus, Species);
#endif
    }
    else 
    {
#if defined(WACP_DEBUG)
        /* Handle simple NAK notification */
        SYS_PRINTF("RxError: NAK F:%d G:%d S:%d\n", Family, Genus, Species);
#endif
    }

    return Result;
}