#include <windows.h>
#include <process.h>
#include <stdio.h>
#include "CSerialCommSvr.h"
#include "CEthCommSvr.h"
#include "CFamilyEnum.h"
#include "SysUtil.h"
#include "DtData.h"
#include "CErrEStd.h"
#include "CNumDStd.h"
#include "CRespDData.h"
#include "CRespDIPI.h"
#include "CRespDCO2.h"
extern uint32 SendMsg(uint32 MsgClassID, uint8* pBuf, uint32 BufSize, uint32 ErrClassID);

/*'****************************************************************************
'** Disconnect 
'*****************************************************************************/
const uint8 DISCONNECT_MSG[] =
{   0x17, 0x01, 0x0c, //envelope preamble
	0x00, 0x00, 0x00, 0x1a, //envelope size
	0x01, 0x1a, //envelope port
		0x00, 0x1e, 0x01, 0x00,//message class ID 
		0x00, 0x00, 0x00, 0x07,//message package size
		0x00, //message encComp
		0x00, 0x00, 0x00, 0x00, //Message payload data size
		0x61, 0xad, //Message CRC
	0xb2, 0x78 //Envelope CRC
};

/*Thread proccess is using a C++ method. Need to type cast to WINAPI function*/
typedef unsigned (WINAPI* WACP_THREAD_PROC)
(    LPVOID lpThreadParameter);

/* Globals */

#if defined STACK_SERIAL
CSerialCommSvr  CommSvr;
#else if STACK_ETHERNET
CEthCommSvr     CommSvr;
#endif

boolean         bConnected;
CRespDData		RespDData;
CRespDIPI		RespDIPI;
CRespDCO2		RespDCO2;
uint32			TestID = 1;
uint32			CmdRqst = 0;
uint8			*ObjBuffer = NULL;
uint32			ObjSize = 0;



#if defined STACK_SERIAL
void StartSerial()
{
    boolean Result;

    CSerialCommSvr_Init(&CommSvr);
    Result = CSerialCommSvr_Connect(&CommSvr, (uint8*)"COM1", RNDZ_PORT_DEFAULT);

    /* Open up the communications channel */
    if (Result)
    {
		/* Launch the reader thread in the CommSvr */
        HANDLE hHandle = (HANDLE)_beginthreadex(NULL, 0,(WACP_THREAD_PROC)CSerialCommSvr_StartClient, &CommSvr, 0, NULL);
        if (hHandle)
        {
            Result = (boolean)SetThreadPriority(hHandle, THREAD_PRIORITY_HIGHEST);
            CloseHandle(hHandle);
        }
    }
}
#else if STACK_ETHERNET
/*'****************************************************************************
'** StartEthernet: Initializes a CommSvr and connects to Host
'*****************************************************************************/
void StartEthernet()
{
    boolean Result = FALSE;

    CEthCommSvr_Init(&CommSvr);
    Result = CEthCommSvr_Connect(&CommSvr,				/*Pointer to the CommSvr*/
								 (uint8*)"127.1",		/*Source ID of connection (Source ID is loopback for demo purposes*/
								 RNDZ_PORT_DEFAULT);	/*Port the RNDZ connection is listening on*/

    /* Open up the communications channel */
    if (Result)
    {
		/* Launch the reader thread in the CommSvr */
        HANDLE hHandle = (HANDLE)_beginthreadex(NULL, 0,(WACP_THREAD_PROC)CEthCommSvr_StartClient, &CommSvr, 0, NULL);
        if (hHandle)
        {
            Result = (boolean)SetThreadPriority(hHandle, THREAD_PRIORITY_HIGHEST);
            CloseHandle(hHandle);
        }
    }
}
#endif
void Menu()
{
    int     Data = 0;
    uint32  Result = 0;

	do
	{
		Data = 0;
		SYS_SLEEP(500);
		SYS_PRINTF("\n\n0. Disconnect\n");
		SYS_PRINTF("1. Get Device Description\n");
		SYS_PRINTF("2. Get Temperature\n");
		SYS_PRINTF("3. Send Respiration Status Message\n");
        SYS_PRINTF("4. Send ACK\n");
		SYS_PRINTF("5. Send NAK\n");
		SYS_PRINTF("6. Send NAK With Description\n");
                
		scanf("%d", &Data);
		
		switch(Data)
		{
			case 0: /* Disconnect */
			{
#if defined STACK_SERIAL
				CSerialCommSvr_SendMessage(&CommSvr, DISCONNECT_MSG, sizeof(DISCONNECT_MSG));
#else if STACK_ETHERNET
				CEthCommSvr_SendMessage(&CommSvr, DISCONNECT_MSG, sizeof(DISCONNECT_MSG));
#endif
             

				SYS_SLEEP(1000);

#if defined STACK_SERIAL
				CSerialCommSvr_Destroy(&CommSvr);
#else if STACK_ETHERNET
				CEthCommSvr_Destroy(&CommSvr);
#endif
				Data =0;
				break;
			}
			case 1: /* Get Device Description */
			{	
                /*Send out a request to get the device description*/
				Result = SendMsg(MakeClassID(FmDEVICE,								/*Family: FmDEVICE*/
											 GnREQUEST,								/*Genus:  GnREQUEST*/
											 DEVICE_CRST_SpGET_DEVICEDESCRIPTION),	/*Species:DEVICE_CRST_SpGET_DEVICEDESCRIPTION*/
											 NULL,									/*No Object*/
							     0,          										/*No size for object*/
								 0);		             							/*Encrypt Message: False*/
				break;
			}
			case 2: /* Get Temp */
			{	
                /*Send out a request to get the temperature*/
				Result = SendMsg(MakeClassID(FmTEMP,                                /*Family: FmTEMP*/
                                             GnREQUEST,                             /*Genus:  GnREQUEST*/
                                             TEMP_CRST_SpGET_TEMP),                 /*Species:TEMP_CRST_SpGET_TEMP*/
                                 NULL,                                              /*No Object*/
                                 0,                                                 /*No size for object*/
                                 0);                                                /*Encrypt Message: False*/
				break;                                                              
			}
			case 3: /*Send Resptration Status Message*/
			{
				SYS_PRINTF("\t0. Exit\n");
                SYS_PRINTF("\t1. REPORT_BREATH_DETECT\n");
                SYS_PRINTF("\t2. REPORT_BREATH\n");
                SYS_PRINTF("\t3. REPORT_PI\n");
                SYS_PRINTF("\nEnter Status ID: ");
                Result = (boolean)scanf("%d", &CmdRqst);
				CRespDData_Init(&RespDData);
				CRespDIPI_Init(&RespDIPI);
				CRespDCO2_Init(&RespDCO2);
			    
                switch(CmdRqst)
                {					
                    case 0:
                    {
                        SYS_PRINTF("\n\nExit Status Menu.\n\n");
                        break;
                    }
                    case 1:
                    {

						/*Setting sample values for EtCO2 and FiCO2*/								
						RespDData.EtCO2.m_Value = 50;
						RespDData.FiCO2.m_Value = 55;
						RespDData.RR.m_Value = 60;

						/*Calculate the size of the object*/
						ObjSize = CRespDData_SizeAll(&RespDData);

						/*Allocate the buffer to the size of the object*/
						ObjBuffer = (uint8*)SYS_MALLOC(ObjSize);

						/*Serialize the object into the buffer*/
						Result = CRespDData_Serialize(&RespDData, ObjBuffer);
						
						if(ObjBuffer != NULL)
						{
                            if(Result)
							{
								/*Sending status breath detect message*/
								SendMsg(MakeClassID(FmRESPIRATION,						  	/*Family: FmRESPIRATION*/
													GnSTATUS,						        /*Genus:  GnSTATUS*/
													RESPIRATION_CSS_SpREPORT_BREATH_DETECT),/*Species: RESPIRATION_CSS_SpREPORT_BREATH_DETECT*/
																															
										ObjBuffer,	                                        /*Serialized CRespDIPI object*/
										ObjSize,	                                        /*Size of Serialized CRespDIPI object*/
										FALSE);		                                        /*Encrypt Message: False*/
								

								/*Free allocations*/
								SYS_FREE(ObjBuffer);
								ObjBuffer = NULL;
							}
						}
							
                        break;
                    }
                    case 2:
                    {

						/*Setting sample values for EtCO2 and FiCO2*/								
						RespDData.EtCO2.m_Value = 50;
						RespDData.FiCO2.m_Value = 55;
						RespDData.RR.m_Value = 60;

						/*Calculate the size of the object*/
						ObjSize = CRespDData_SizeAll(&RespDData);

						/*Allocate the buffer to the size of the object*/
						ObjBuffer = (uint8*)SYS_MALLOC(ObjSize);

						/*Serialize the object into the buffer*/
						Result = CRespDData_Serialize(&RespDData, ObjBuffer);
						if(ObjBuffer != NULL)
						{
                            if(Result)
							{
								/*Sending status report breath*/
								SendMsg(MakeClassID(FmRESPIRATION,					  /*Family: FmRESPIRATION*/
													GnSTATUS,						  /*Genus:  GnSTATUS*/
													RESPIRATION_CSS_SpREPORT_BREATH), /*Species: RESPIRATION_CSS_SpREPORT_BREATH*/
																															
										ObjBuffer,	                                  /*Serialized CRespDIPI object*/
										ObjSize,	                                  /*Size of Serialized CRespDIPI object*/
										FALSE);		                                  /*Encrypt Message: False*/
								

								/*Free allocations*/
								SYS_FREE(ObjBuffer);
								ObjBuffer = NULL;							
							}
						}
                        break;
                    }
                    case 3:
                    {
						/*Setting example value for PulmonaryIndex*/
						RespDIPI.PulmonaryIndex.m_Value = 5;

						/*Calculate the size of the object*/
						ObjSize = CRespDIPI_SizeAll(&RespDIPI);

						/*Allocate the buffer to the size of the object*/
						ObjBuffer = (uint8*)SYS_MALLOC(ObjSize);

						/*Serialize the object into the buffer*/
						Result = CRespDIPI_Serialize(&RespDIPI, ObjBuffer);

						if(ObjBuffer != NULL)
						{
                            if(Result)
							{
								/*Sending status report PI message*/
								SendMsg(MakeClassID(FmRESPIRATION,				 /*Family: FmRESPIRATION*/
													GnSTATUS,					 /*Genus:  GnSTATUS*/
													RESPIRATION_CSS_SpREPORT_PI),/*Species: RESPIRATION_CSS_SpREPORT_PI*/
																															
										ObjBuffer,	                             /*Serialized CRespDIPI object*/
										ObjSize,	                             /*Size of Serialized CRespDIPI object*/
										FALSE);		                             /*Encrypt Message: False*/
								

								/*Free allocations*/
								SYS_FREE(ObjBuffer);
								ObjBuffer = NULL;						
							}
						}
                        break;
                    }
                    /* -- Unrecognized StatusID -- */
                    default:
                    {
                        SYS_PRINTF("\n\nInvalid Selection.\n\n");
                        break;
                    }
                }
                break;
			}
			case 4: /* Test ACK */
			{	
                /*Example send an ACK message */
				Result = SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRNONE), /*Trap MsgClassID. Note: Species of CERR_SpERRNONE indicates ACK*/
                                 NULL,                                         /*NULL: No serialized object*/
                                 0,                                            /*0: No size of serialized object*/
                                 MakeClassID(FmNIBP, GnSTATUS, 1));            /*ClassID of the message that is acknowledged*/
				break;
			}
			case 5: /* Test NAK */
			{	
                /*Example send NAK message*/
				Result = SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRAPPINTERNAL), /*Trap MsgClassID. Note: Species of CERR_SpERRAPPINTERNAL indicates Application Internal NAK*/
                                 NULL,                                                /*NULL: No serialized object*/
                                 0,                                                   /*0: No size of serialized object*/
                                 MakeClassID(FmNIBP, GnSTATUS, 1));                   /*ClassID of the message that is NAK'd*/
				break;
			}
			case 6: /* Test NACK with error object */
			{	
                CErrEStd TestError;

                uint8 TestBuf[128];
                uint32 TestSize = 0;

                CErrEStd_Init(&TestError);

                /*Set the ClassID of the object that caused the error*/
                TestError.m_ErrClassID = MakeClassID(FmDEVICE, GnDATA, 3);

                /*Set the ErrorCode*/
                TestError.m_ErrorCode = 33;

                /*Set a textual description of the error*/
                sprintf(TestError.m_Description, "Testing ErrObj");

                /*Retrieve the size of the serialized object*/
                TestSize = CErrEStd_SizeAll(&TestError);

                /*Serialize the object into the buffer*/
                CErrEStd_Serialize(&TestError, TestBuf);

                /*Send the NAK message*/
				Result = SendMsg(MakeClassID(FmTRAP, GnERROR, CERR_SpERRAPPINTERNAL), /*Trap MsgClassID. Note: Species of CERR_SpERRAPPINTERNAL indicates Application Internal NAK*/
                                TestBuf,                                              /*Serialized Error object*/
                                TestSize,                                             /*Size of serialized  Error object*/
                                MakeClassID(FmNIBP, GnSTATUS, 1));                    /*ClassID of the message that is NAK'd*/
				break;
			}
			default:
			{
						Data = -1;
						SYS_PRINTF("CLS");
						SYS_PRINTF("\n\n---UNSUPPORTED---\n");
			}

		}//End Switch

	}while (Data != 0); //End Do
}


int main()
{

    /* Start up transport and send out Rendezvous Connection*/
#if defined STACK_SERIAL
    StartSerial();
#else if STACK_ETHERNET
    StartEthernet();
#endif


    while(bConnected == FALSE)
    {
        SYS_SLEEP(200);
    }

    Menu();

}
