/*-D*************************************************************************
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Common.h       :   List of common definitions                     //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(COMMON_H)
    #define  COMMON_H

#include "Target.h"
#include "StdVarTyp.h"

#ifdef __cplusplus
extern "C" {
#endif

#if !defined (WIN32)
    #if !defined(INFINITE)
        #define INFINITE 0xFFFFFFFF
    #endif

    #if !defined(INVALID_HANDLE_VALUE)
        #define INVALID_HANDLE_VALUE       ( 0 )
    #endif

    typedef void *HANDLE;
#endif

#define WAIT_SUCCESS       ( 0 )
#define WAIT_FAILURE       ( 1 )

#define ADDR_BUF_SIZE   (32)
#define EVENT_BUF_SIZE  (32)

#define RNDZ_PORT_DEFAULT ( 281 )
#define WACP_PORT_DEFAULT ( 282 )

#define TRACE_EXCG_LOG      "TraceEXCGLog.xml"
#define TRACE_WACP_LOG      "TraceWACPLog.xml"

typedef enum STACK_TYPE{
    STACK_UNDEFINED = 0, 
    SERIAL,
    ETHERNET,
    USB,
    BLUETOOTH,
    ZIGBEE
}STACK_TYPE;

typedef  enum CTStypOPTION{
    CTS_SpNONE = 0,
    CTS_SpACK,
    CTS_SpDESC,
    CTS_SpMAX
} CTS_OPTION;


/* Macros for extracting the Family, Genus, and Species from
 * a bytes stream 
 */
#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    #define GET_FAMILY(pBuf) ((uint16)(pBuf[0] << 8 | pBuf[1]))
    #define GET_GENUS(pBuf) (pBuf[2])
    #define GET_SPECIES(pBuf) (pBuf[3])

#else
    #define GET_FAMILY(pBuf) ((uint16)(pBuf[3] << 8 | pBuf[2]))
    #define GET_GENUS(pBuf) (pBuf[1])
    #define GET_SPECIES(pBuf) (pBuf[0])
#endif   

#define SET_FAMILY(ClassID, Family)   ((ClassID & 0x0000FFFF) | (Family << 16))
#define SET_GENUS(ClassID, Genus)    ((ClassID & 0xFFFF00FF) | (Genus << 8))
#define SET_SPECIES(ClassID, Species)  ((ClassID & 0xFFFFFF00) | Species)

typedef  enum WACP_ENCRYPTION_TYPES {
    WACP_ENCRYPTION_NONE = 0,
    WACP_ENCRYPTION_AES,
    WACP_ENCRYPTION_MAX
}WACP_ENCRYPTION_TYPES;

typedef enum THREAD_ID{
    EXCG_RECV_THREAD = 0,
    WACP_CCOMSVR_THREAD,
    CONNOBJSVR_THREAD
}THREAD_ID;

typedef struct ConnArgs
{
    /* User configurable */
    boolean     bSvrFlag;               /* TRUE - Host/Server, FALSE - Client */
    boolean     bRNDZDisabled;          /* Turns off RNDZ authentication if set to TRUE */
    uint8       EncryptionType;         /* From Common.h of type WACP_ENCRYPTION_TYPES enuermation */
    int32       Priority;               /* 0 for normal, Positive for higher, Negative for lower */
    uint32      Timeout;                /* RNDZ authentication timeout in millseconds */
    STACK_TYPE  StackType;              /* From Common.h of type STACK_TYPE enumeration */
    uint16      RNDZPort;               /* RNDZ Port identifier */
    uint16      WACPPort;               /* RNDZ Port identifier */
    void*       pConfig;                       /* pointer to a stack configuration structure */
    char        SourceID[ADDR_BUF_SIZE];        /* Name of connection string identifier */

    /* Stack modified values */
    uint8       ValidationState;
    boolean     bRNDZStatus;
    boolean     bConnected;
    uint8*      pKey;
    void*       pCommSvr;
    void*       pConnObj;
    void*       pGUIDStd;
    char*       pByPassSrc;
}ConnArgs;

#if defined( __cplusplus )
}
#endif

#endif /* #if !defined(COMMON_H) */
