/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Dt.c           :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

/*********************************************************************************************
Helper Class Dt
This class provides some static methods to help manipulate DataClass identifiers and size.
These are implemented as methods instead of macros because methods are type safe and do not
suffer from many of the side-effects of macros.
*********************************************************************************************/

#include "Dt.h"


/*********************************************************************************
Creates a DataClass identifier from the family, genus and species.
*********************************************************************************/
uint32 dt_make_id ( uint16 family, uint8 genus, uint8 species )
{
    return (((uint32)family << 16) | ((uint16)genus << 8) | species);
}
/*********************************************************************************
Returns the family sub-code from an input identifier.
*********************************************************************************/
uint16 dt_family ( const uint32 id )
{
    return (uint16)(id >> 16);
}
/*********************************************************************************
Returns the genus sub-code from an input DataClass identifier.
*********************************************************************************/
uint8 dt_genus ( const uint32 id )
{
    return (uint8)(id >> 8);
}
/*********************************************************************************
Returns the species sub-code from the input class identifier.
*********************************************************************************/
uint8 dt_species ( const uint32 id )
{
    return (uint8)id;
}

/*---------------------------End of DataClass Dt ----------------------------------------*/


