/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// DtData.h       :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(DTDATA_H)
    #define  DTDATA_H

#include "Target.h"
#include "StdVarTyp.h"
#include "SysUtil.h"
#include "Dt.h"
#include "DtSupport.h"
#include "MiscUtil.h"

#define DIRTY_FLAG_MASK 0x80          /*[1000 0000]*/
#define VALID_FLAG_MASK 0x40          /*[0100 0000]*/
#define DIRTY_VALID_FLAG_MASK 0xC0    /*[1100 0000]*/

/* Global references */
#if defined(TRACE_SERVER_ON)
extern const char* const FmStringTable[];
extern const char* const GnStringTable[];
#endif

typedef struct t_VersionEntry {    /* -- Version entry -- */
    uint16 min_version;            /* Minimum Version */
    uint16 max_version;            /* Maximum Version */
} VersionEntry;

/*********************************************************************************
Creates a DataClass identifier from the family, genus and species.
*********************************************************************************/
uint32 MakeClassID(uint16 Family, uint8 Genus, uint8 Species);

/*********************************************************************************
 Set function for data member species
*********************************************************************************/
void SetFamily(uint32* ClassID, uint16 new_Family);

/*********************************************************************************
Returns the family sub-code from a serialized byte stream.
*********************************************************************************/
uint16 GetFamily (const uint8* pBuf);

/*********************************************************************************
 Set function for data member species
*********************************************************************************/
void SetGenus(uint32* ClassID, uint8 new_Genus);

/*********************************************************************************
Returns the genus sub-code from a serialized byte stream.
*********************************************************************************/
uint8 GetGenus(const uint8* pBuf);

/*********************************************************************************
 Set function for data member species
*********************************************************************************/
void SetSpecies(uint32* ClassID, uint8 new_Species);

/*********************************************************************************
Returns the species sub-code from a serialized byte stream.
*********************************************************************************/
uint8 GetSpecies(const uint8* pBuf);

/*********************************************************************************
Sets the DataBitField
*********************************************************************************/
void SetDataBitField(byte* DataBitField, uint8 new_DataBitField);

/*********************************************************************************
Returns a boolean value representing the state of the dirtyFlag bit
*********************************************************************************/
boolean IsDirty(byte DataBitField);

/*********************************************************************************
Sets the dirtyFlag bit 
*********************************************************************************/
void SetDirty(byte* DataBitField, boolean new_dirtyFlag);

/*********************************************************************************
Returns a boolean value representing the state of the validFlag bit
*********************************************************************************/
boolean IsValid(byte DataBitField);

/*********************************************************************************
Sets the DataBitField
*********************************************************************************/
void SetValid(byte* DataBitField, boolean new_validFlag);

/*********************************************************************************
Returns the CRCsize
*********************************************************************************/
uint32 GetCRCSize(const uint8* Buffer);

/*********************************************************************************
Helper Function:
Peforms a comparison between the SrcVersion and the version span in the 
VersionEntry table. Returns TRUE if the SrcVersion falls within the version span
*********************************************************************************/
boolean GetVersionStatus(VersionEntry* pEntryList, uint16 EntryCount, uint16 MemberEnum,
                         uint16 SrcVersion);

/*********************************************************************************
Helper Function to calculate a serialized object's size
*********************************************************************************/
uint32 MOIBSerialSize(uint8* pBuffer, uint8* pSizeWidthOut);

/*********************************************************************************
Helper Function to return TRUE if a serialized object has a uint32 size identifier
*********************************************************************************/
boolean IsExtended_Bytes(uint8* pBuffer);

/*********************************************************************************
Helper Function to return TRUE if a serialized object has a uint32 size identifier
*********************************************************************************/
boolean IsExtended_ClassID(uint32 ClassID);

#endif /* #if !defined(DTDATA_H) */


