/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Mem.h          :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(MEM_H)
    #define  MEM_H

#include "StdVarTyp.h"
#include "Target.h"
#include <string.h> /* For memxxx function */

#define SIZEOF_FLOAT   (sizeof(float))
#if (TARGET_ENDIANNESS == MEDIA_ENDIANNES)
    #define F32_START_INDEX (0)
    #define F32_INC (1)
#else
    #define F32_START_INDEX (SIZEOF_FLOAT-1)
    #define F32_INC (-1)
#endif

#define SIZEOF_DOUBLE  (sizeof(double))
#if (TARGET_ENDIANNESS == MEDIA_ENDIANNES)
    #define F64_START_INDEX (0)
    #define F64_INC (1)
#else
    #define F64_START_INDEX (SIZEOF_DOUBLE-1)
    #define F64_INC (-1)
#endif

/* Macros for use with CopyBlock()/CopyString() */
#define COPY_BLOCK(X,Y)     CopyBlock((X), sizeof(X), (Y), sizeof(Y))
#define COPY_STRING(X,Y)    CopyString((X), sizeof(X), ((ansichar*)Y), (uint16)strlen(Y))

/*================================================================================
ReadInt8 Endian-proof function to read a 8 bit integer from a buffer.
================================================================================*/
uint8 ReadInt8(uint8** ppBuf);

/*================================================================================
ReadInt16 Endian-proof function to read a 16 bit integer from a buffer.
================================================================================*/
uint16 ReadInt16(uint8** ppBuf);
uint16 ReadInt16Ext(const uint8* pBuf);

/*================================================================================
ReadInt32 Endian-proof function to read an 32 bit integer from a buffer.
================================================================================*/
uint32 ReadInt32(uint8** ppBuf);
uint32 ReadInt32Ext(const uint8* pBuf);

/*================================================================================
Method to get a 4 byte IEEE format floating point value from the buffer. 
================================================================================*/
float ReadFloat(uint8** ppBuf);

/*================================================================================
Method to get a 8 byte IEEE format double value from the buffer. 
================================================================================*/
double ReadDouble(uint8** ppBuf);

/*================================================================================
WriteInt8 Endian-proof function to write a 16 bit integer into a buffer.
================================================================================*/
void WriteInt8(uint8** ppBuf, uint8 Val);

/*================================================================================
WriteInt16 Endian-proof function to write a 16 bit integer into a buffer.
================================================================================*/
void WriteInt16(uint8** ppBuf, uint16 Val);

/*================================================================================
WriteInt32 Endian-proof function to write a 32 bit integer into a buffer.
================================================================================*/
void WriteInt32(uint8** ppBuf, uint32 Val);

/*================================================================================
Method to write a 4 byte IEEE format floating point value to the buffer. 
================================================================================*/
void WriteFloat(uint8** ppBuf, float Val);

/*================================================================================
Method to write a 8 byte IEEE format double value to the buffer. 
================================================================================*/
void WriteDouble(uint8** ppBuf, double Val);

/*================================================================================
ReadInt18 Endian-proof function to write a 18 bit integer into a buffer.
================================================================================*/
uint32 ReadInt18(const uint8 *buffer, uint32 index);

/*================================================================================
WriteInt18 Endian-proof function to read a 18 bit integer from a buffer.
================================================================================*/
void WriteInt18(uint8 *buffer, uint32 index, uint32 value);

/*================================================================================
Helper function to copy data into a dynamic buffer
================================================================================*/
boolean SetData(uint8 **pDestBuf, const uint8 *pSrcBuf, uint32 Size);

/*================================================================================
Helper function to enqueue MOIB_TIME struct
================================================================================*/
void Put_MOIBTIME(uint8 **ppBuf, MOIB_TIME sDateTime);

/*================================================================================
Helper function to dequeue MOIB_TIME struct
================================================================================*/
MOIB_TIME Get_MOIBTIME(uint8 **ppBuf);

/*================================================================================
Helper function to copy data into a member
================================================================================*/
void CopyBlock(void* pDest, uint16 DestSize, const void* pSrc, uint16 SrcLength);

/*================================================================================
Helper function to copy a string into a member
================================================================================*/
void CopyString(ansichar * pDest, uint16 DestSize, const ansichar* pSrc, uint16 SrcLength);

#endif  /* #if !defined(MEM_H) */
