/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// SysUtil.c      :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include <stdlib.h>

#include "Target.h"
#include "StdVarTyp.h"
#include "SysUtil.h"

#include <stdio.h>
#include <stdarg.h>

/*
 * Abstractions for C memory allocations functions
 *
 * void *malloc( size_t size );
 * void *calloc( size_t num, size_t size );
 * void free( void *memblock );
 * void *realloc( void *memblock, size_t size );
 */

/****************************************************************/
/*                            WIN32                             */
/****************************************************************/
#include <process.h>
#include <Windows.h>

/* System setup */
void SysUtil_Setup(void)
{
}

#if defined(MEMORY_DEBUG)
#define MEMORY_TXT  "memory.txt"

static uint32   MallocCount;
FILE            *pMemFile;
void *sys_malloc( MEMORY_POOL Pool, size_t Size, char *pName, int Line )
{   
    void *pTemp;
    TOUCH(Pool);
    
    pTemp = malloc(Size);
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    if (pMemFile)
    {
        fprintf(pMemFile, "sys_malloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, ++MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }

    return pTemp;
}

void *sys_calloc( MEMORY_POOL Pool, size_t num, size_t Size, char *pName, int Line  )
{
    void *pTemp;
    TOUCH(Pool);
    
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    pTemp = calloc(num,Size);

    if (pMemFile)
    {
        fprintf(pMemFile, "sys_calloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, ++MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }

    return pTemp;
}

void sys_free( MEMORY_POOL Pool, void *pMemBlock , char *pName, int Line )
{
    TOUCH(Pool);
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    if ((pMemFile) && (pMemBlock))
    {
        fprintf(pMemFile, "sys_free\t %p\t %d\t \t %s\t Line %d\n", pMemBlock, --MallocCount, pName, Line);
        fflush(pMemFile);
        free(pMemBlock);
    }
}

void *sys_realloc( MEMORY_POOL Pool, void *pMemBlock, size_t Size, char *pName, int Line   )
{
    void *pTemp;

    TOUCH(Pool);

    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    if ((pMemBlock) && (pMemFile))
    {
        fprintf(pMemFile, "clr_free\t %p\t %d\t \t %s\t Line %d\n", pMemBlock, MallocCount, pName, Line);
    }
    else
    {
        MallocCount++;
    }

    pTemp = realloc(pMemBlock, Size);

    if (pMemFile) 
    {
        fprintf(pMemFile, "sys_realloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }
    return pTemp;
}
#else
void *sys_malloc( MEMORY_POOL Pool, size_t Size)
{   
    TOUCH(Pool);
    return malloc(Size);
}

void *sys_calloc( MEMORY_POOL Pool, size_t num, size_t Size)
{
    TOUCH(Pool);
    return calloc(num,Size);
}

void sys_free( MEMORY_POOL Pool, void *pMemBlock)
{
    TOUCH(Pool);
    if (pMemBlock)
    {
       free(pMemBlock);
    }
}

void *sys_realloc(MEMORY_POOL Pool, void *pMemBlock, size_t Size)
{
    TOUCH(Pool);
    return realloc(pMemBlock, Size);
}

#endif


/* Get ID of currently running process */
int  sys_getpid(void)
{
    return _getpid();
}


/* Suspend current process/thread/task for a specified time. */
void sys_sleep(unsigned long Msecs)
{
    Sleep(Msecs);
}


/* Get system time. */
time_t sys_time(time_t *pTimer)
{
    return (time_t)time(pTimer);
}


/*
 * Abstractions for C pseudorandom number functions.
 *
 * void srand( unsigned int seed );
 * int rand( void );
 */

/* Set a random starting point. */
void sys_srand(unsigned int Seed)
{
    srand(Seed);
}

/* Generate and return a pseudorandom number. */
int sys_rand(void)
{
    return(rand());
}


/* Abstraction for printf function*/
int sys_printf(const char *pFormat, ...)
{
    va_list(vargs);
    va_start(vargs, pFormat);

    return (int)vprintf(pFormat, vargs);
}

/* Abstraction for initialization function*/
boolean sys_init(void)
{
    return TRUE;
}

/* Abstraction for destroy function*/
boolean sys_destroy(void)
{
    return TRUE;
}
