/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Wacp .h        :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :							                       //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CWACP_H)
    #define  CWACP_H

#include "StdVarTyp.h"

/*WACP Parser errors */
#define WACP_SUCCESS            0
#define WACP_CRC_ERROR          1
#define WACP_PREAMBLE_ERROR     2
#define WACP_PORT_ERROR         3
#define WACP_PARAMETER_ERROR    4

/*Envelope Protocol Header*/
#define PREAMBLE_SIZE (3)
#define ENVELOPE_SIZE (4)
#define ENVELOPE_PORT (2)

#define PACKET_HEADER (sizeof(uint8) * PREAMBLE_SIZE + sizeof(uint32))
#define ENVELOPE_HEADER_SIZE (PREAMBLE_SIZE + ENVELOPE_SIZE + ENVELOPE_PORT)
#define MESSAGE_HEADER_SIZE (MSG_CLASSID + MSG_LGSIZE + MSG_ENCRYPTION + MSG_PAYLOAD_SIZE + CRCSIZE)
#define TRAP_HEADER_SIZE (MSG_CLASSID + MSG_SMSIZE + MSG_SMSIZE + MSG_CLASSID + MSG_PAYLOAD_SIZE + CRCSIZE)

#define CTL_W 0x17
#define CTL_A 0x01
#define CTL_C 0x03
#define CTL_L 0x0C

/*Message Protocol Header*/
#define MSG_CLASSID (4)
#define MSG_LGSIZE (4)
#define MSG_SMSIZE (2)
#define MSG_ENCRYPTION (1)
#define MSG_PAYLOAD_SIZE (4)


/*Common*/
#define CRCSIZE (2)

/*Protocol Ports*/
#define RNDZ_PORT_DEFAULT ( 281 )
#define WACP_PORT_DEFAULT ( 282 )

/* WACP Message creation functions */
uint32 GeneratePacket(uint8* pOutBuffer, uint32* pOutSize, uint8* pBuf, uint32 BufSize, uint32 MsgClassID, uint32 ErrClassID);

/* WACP Parsing functions */
uint32 DeserializeEnvelope(uint8** pBuf, uint32 BufSize, uint32* OutSize);
uint32 MsgHandler(char* pSourceID, uint8** pBuf, uint32 BufSize);
uint32 WACPParser(char *pSourceID, uint8* pBuf, uint32 BufSize, uint32 *pPacketLength);

#endif /* #if !defined(CWACP_H) */

/*---------------------------------------------------*/
/* End of File */





