/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Target.h       :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(TARGET_H)
#define TARGET_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Simple macro to make compiler happy when function arguments are not used.
 */
#define TOUCH(X)    (X=X)

/*
 * Target Processor Types
 */
#define CPU_UNKNOWN     0   /* Unknown or generic CPU */
#define CPU_X86         1   /* Intel CPU Family */
#define CPU_6833X       2   /* Motorola 6833X CPU Family */
#define CPU_Z180        3   /* Zilog CPU Family */
#define CPU_68K         4   /* 68000 CPU Family */
#define CPU_H8X         5   /* Hitachi H8/H8s Family */
#define CPU_MXL         6   /* Motorola MC9328MXL */
#define CPU_C5000       7   /* Texas Instrument C5000 DSP */

/* 
 * Stack Type
 */
#define STACK_ETHERNET 1
//#define STACK_SERIAL 1

#define DEFAULT_BAUD_RATE 115200

/*
 * Processor endian Types
 */
#define LITTLE_ENDIAN   0  /* Intel, et. al. */
#define BIG_ENDIAN      1

/*
 * Endianness of date stream on the media.
 */
#define MEDIA_ENDIANNES         BIG_ENDIAN

/*
 * XML support
 */
#define XML_NONE        0
#define XML_SERIALIZE   1
#define XML_DESERIALIZE 2
#define XML_ALL         3


/******************************************/
/* Target Processor Dependent Definitions */
/******************************************/

#if defined(_MSC_VER)
/************************************************************************
  Microsoft Visual C++
*************************************************************************/
#define TARGET_CPU              CPU_X86
#define TARGET_ENDIANNESS       LITTLE_ENDIAN
#define TARGET_XML_SUPPORT      XML_NONE

/* Calling convention */
#define WACPAPI                 __cdecl

/* Set the maximum number of connections required */
#define MAX_CONNECTIONS         256

/* Set the maximum number of messages to queue up for incoming messages */
#define MAX_QUEUE_SIZE          1024

/* Set this to the max message buffer size */
#define RCVBUFSIZE              (32768)

/* Turn this on to print out the dynamic memory usage log */
/* #define MEMORY_DEBUG       1 */

/* Turn this on to trace all errors in the link and presentation layers */
#define TRACE_SERVER_ON  1 

/* Turn this on to trace all messages received and sent from the link layer 
 * TRACE_SERVER_ON must be defined for this to be active
 */
 /*#define TRACE_SERVER_ON_STATUS 1*/  

/* Turn these on for more debug output */
#define DEBUG_PRINT_STATUS

/* Turn this on to view detailed debug information throughtout the system */
/*#define DEBUG_PRINT        1 */

/* Turn this on for supporting encryption */
//#define ENCRYPTION_ON   1   /* Exception: Not supported in the C Host baseline. */
    
/* Connection Object Server defaults */
#define DEFAULT_CONN_SETTING    CONN_CFG_SpKEEPALIVEOFF
#define DEFAULT_TRAP_SETTING    CTS_SpNONE

#define KEEPALIVE_SUPPORT 1

/* Default values for the Connection object server */
#define DEFAULT_TIMEOUT_INTERVAL 20     /* in seconds */
#define DEFAULT_KEEPALIVE_INTERVAL 2    /* in seconds */

/* Set the default interval in milliseconds between keep alive checks 0.. INFINITE */
#define DEFAULT_KEEPALIVE_CHECK         (DEFAULT_KEEPALIVE_INTERVAL * 1000)

/* Set the default timeout for RNDZ connections in milliseconds */
#define DEFAULT_RNDZ_TIMEOUT            10000

#define WAIT_COUNT                      200

/* Set the Non-Overlapped IO polling time */
#define SERIAL_POLLTIME             100  /* milliseconds */

#endif      /* #if defined(_MSC_VER) */

/* Target specific parameters */
#define MAX_PACKET_SIZE         256

#if defined( __cplusplus )
}
#endif

#endif  /* #if !defined (TARGET_H) */

/* End of File */


