/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFamilyStub.cpp                                                   //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.7.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CWACPApp.h"
#include "CFamilyStub.h"



CModulePSelector ModulePSelector;

CFamilyStub::CFamilyStub()
{
    pCDeviceDStd = new CDeviceDStd();
    pCDevDPOST = new CDevDPOST();
    pCModuleDStd = new CModuleDStd();
    pCRespDData = new CRespDData();
    pCRespDIPI = new CRespDIPI();
    pCRespDCO2 = new CRespDCO2();
    pCSpO2DData = new CSpO2DData();
    pCNumDStd = new CNumDStd();
	pCModulePSelector = new CModulePSelector();
    LoadObjects();

#if defined(DEBUG_PRINT_STATUS)
    ErrCount = 0;
    AckCount = 0;
#endif
}

CFamilyStub::~CFamilyStub()
{
    delete pCDeviceDStd;
    delete pCDevDPOST;
    delete pCModuleDStd;
    delete pCRespDData;
    delete pCRespDIPI;
    delete pCRespDCO2;
    delete pCSpO2DData;
    delete pCNumDStd;
	delete pCModulePSelector;
}

/*================================================================================
LoadObjects :
================================================================================*/
void CFamilyStub::LoadObjects()
{
}

/*================================================================================
AppInit : Virtual Function called by CMOMPBaseClass::AppInitInt
================================================================================*/
CMOMPBaseClass* CFamilyStub::AppInit(CWACPApp *pWACPAppRef, boolean bSvrFlag, char* pSourceID)
{
    TOUCH(pWACPAppRef);
    TOUCH(bSvrFlag);
    TOUCH(pSourceID);

    return (CMOMPBaseClass*)this;
}

/*================================================================================
AppEnd : Virtual Function called by CMOMPBaseClass::AppEndInt
================================================================================*/
uint32 CFamilyStub::AppEnd(CWACPApp *pWACPAppRef)
{
    uint32 Result = 0;

    TOUCH(pWACPAppRef);

	pWACPApp = NULL;

    return(Result);
}

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
uint32 CFamilyStub::RecvMessage(char *pSourceID, uint32 ClassID, uint8 *pBuffer)
{
    uint32  Result     = 0;
    uint16  ErrorCode  = CERR_PROC_UNIMPL;
    uint16  Family     = GetFamily(ClassID);
    uint8   Genus      = GetGenus(ClassID);
    uint8   Species    = GetSpecies(ClassID);
    uint32  MsgResult  = 0;
	uint8   *ObjBuffer = NULL;
    uint32  ObjSize = 0;
    TOUCH(pSourceID);
    TOUCH(pBuffer);

    switch(Family)
    {
        case FmDEVICE:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case DEVICE_CCMD_SpWRITE_CONFIG:
                            break;
                        case DEVICE_CCMD_SpWRITE_DEVICEDESCRIPTION:
                            break;
                        case DEVICE_CCMD_SpENABLE_DISPLAY:
                            break;
                        case DEVICE_CCMD_SpSTART_DISPLAY_TEST:
                            break;
                        case DEVICE_CCMD_SpRESET:
                            break;
                        case DEVICE_CCMD_SpPOWER:
                            break;
                        case DEVICE_CCMD_SpBEEP:
                            break;
                        case DEVICE_CCMD_SpSAVE_CONFIG:
                            break;
                        case DEVICE_CCMD_SpRESTORE_CONFIG:
                            break;
                        case DEVICE_CCMD_SpENABLE_AUTO_POWER_DOWN:
                            break;
                        case DEVICE_CCMD_SpSET_DISPLAY_CONTRAST:
                            break;
                        case DEVICE_CCMD_SpSET_BACKLIGHT:
                            break;
                        case DEVICE_CCMD_SpSTART_DIGITAL_TEST:
                            break;
                        case DEVICE_CCMD_SpSET_DEVICE_MODE:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case DEVICE_CRST_SpGET_DEVICEDESCRIPTION:
                        {
                            /*Received a device description request and contains no payload*/
                            SYS_PRINTF("Received Device Description Request\n");
                            
                            MsgResult = TRUE;
                            
                            /*Set sample data*/
                            pCDeviceDStd->SetModelName("PC Host", (uint16)strlen("PC Host"));
                            pCDeviceDStd->SetSerialNumber("123456", (uint16)strlen("1234656"));
                            
                            /*Retrieve the size of the serialized object*/
                            ObjSize = pCDeviceDStd->size_all();
                            
                            /*Allocate a buffer to serialize the object*/
                            ObjBuffer = (uint8*)SYS_MALLOC(ObjSize);
                            
                            if(ObjBuffer!= NULL)
                            {
                                /*Serialize the device object*/
                                pCDeviceDStd->serialize_all(ObjBuffer);
                                  
                                /*Sending response message containing the device description*/
							    SendMsg(pSourceID,						/*SourceID of the connection*/
									    FmDEVICE,						/*Family: FmDEVICE*/
										GnRESPONSE,						/*Genus:  GnRESPONSE*/
										DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION,	/*Species: DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION*/
										ObjBuffer,						/*Serialized CDeviceDStd object*/
										ObjSize,						/*Size of Serialized CDeviceDStd object*/
										FALSE);							/*Encrypt Message: False*/							

							    /*free allocation*/
							    SYS_FREE(ObjBuffer);
							    ObjBuffer = NULL;                                 
                                  
                            }  
                            break;
                        }
                        case DEVICE_CRST_SpGET_CONFIG:
                            break;
                        case DEVICE_CRST_SpGET_POST:
                            break;
                        case DEVICE_CRST_SpGET_DIGITAL_STATE:
                            break;
                        case DEVICE_CRST_SpGET_DEVICE_MODE:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case DEVICE_CRSP_SpPUT_DEVICEDESCRIPTION:
                            break;
                        case DEVICE_CRSP_SpPUT_CONFIG:
                            break;
                        case DEVICE_CRSP_SpPUT_POST:
                            break;
                        case DEVICE_CRSP_SpPUT_DIGITAL_STATE:
                            break;
                        case DEVICE_CRSP_SpPUT_DEVICE_MODE:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CDEVDSTD_SpSTANDARD:
                            break;
                        case CDEVDSTD_SpHOST:
                            break;
                        case CDEVDSTD_SpSENSOR:
                            break;
                        case CDEVDSTD_SpALL:
                            break;
                        case CDEVDPST_SpPOST:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmMODULE:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case MODULE_CCMD_SpWRITE_CONFIG:
                            break;
                        case MODULE_CCMD_SpWRITE_MODULE_DESC:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case MODULE_CRST_SpGET_MODULE_DESC:
                          
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case MODULE_CRSP_SpPUT_MODULE_DESC:
						{
							/*Received Module response message the expected payload is:
								Object structure: [CModuleDStd]
								1. CModuleDStd: Object containing the requested version and description information							
							*/

							SYS_PRINTF("Received Module response\n");
							MsgResult = TRUE;
				
							/*Make sure pBuffer is filled*/
							if(pBuffer != NULL)
							{
								/*Deserialize the CModuleDStd */
								if(pCModuleDStd->deserialize_all(pBuffer))
								{
									uint16 Size = 0;
									
									/*Print version and description of selected module*/
									SYS_PRINTF("Version: %s\n", pCModuleDStd->GetVersion(&Size));
									SYS_PRINTF("Description: %s\n", pCModuleDStd->GetDescription(&Size));
									SYS_PRINTF("\n");
								}
							}
                            break;
						}
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CMODDSTD_SpSTANDARD:
                            break;
                        case CMODDSTD_SpHOST:
                            break;
                        case CMODDSTD_SpHARDWARE:
                            break;
                        case CMODDSTD_SpBOOTLOADER:
                            break;
                        case CMODDSTD_SpBOOTRECOVERY:
                            break;
                        case CMODDSTD_SpNIBPMODULE:
                            break;
                        case CMODDSTD_SpNIBPSAFETYALG:
                            break;
                        case CMODDSTD_SpNIBPSTEPALG:
                            break;
                        case CMODDSTD_SpNIBPFASTALG:
                            break;
                        case CMODDSTD_SpNIBPNEONATSTEPALG:
                            break;
                        case CMODDSTD_SpNIBPPOEMALG:
                            break;
                        case CMODDSTD_SpNIBPSTEPAOALG:
                            break;
                        case CMODDSTD_SpTEMPMODULE:
                            break;
                        case CMODDSTD_SpSURETEMPALG:
                            break;
                        case CMODDSTD_SpSPO2MODULE:
                            break;
                        case CMODDSTD_SpECGMODULE:
                            break;
                        case CMODDSTD_SpMEANS:
                            break;
                        case CMODDSTD_SpSPIROMETRY:
                            break;
                        case CMODDSTD_SpFRONTEND:
                            break;
                        case CMODDSTD_SpPPP:
                            break;
                        case CMODDSTD_SpSECUREDIGITALCARD:
                            break;
                        case CMODDSTD_SpALL:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmERROR:
        {
            switch(Genus)
            {
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case ERROR_CRST_SpGET_ERROR:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case ERROR_CRSP_SpPUT_ERROR:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case ERROR_CSS_SpREPORT_ERROR:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmRESPIRATION:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case RESPIRATION_CCMD_SpRESTART_BREATH_ALG:
                            break;
                        case RESPIRATION_CCMD_SpSET_CALIBRATION_DATE:
                            break;
                        case RESPIRATION_CCMD_SpSTART_CALIBRATION:
                            break;
                        case RESPIRATION_CCMD_SpSTOP_CALIBRATION:
                            break;
                        case RESPIRATION_CCMD_SpFORCE_SFM:
                            break;
                        case RESPIRATION_CCMD_SpPUMP_CONTROL:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case RESPIRATION_CRST_SpGET_BREATH:
                            break;
                        case RESPIRATION_CRST_SpGET_AMBIENT_PRESSURE:
                            break;
                        case RESPIRATION_CRST_SpGET_MODULE_CONFIG:
                            break;
                        case RESPIRATION_CRST_SpGET_CALIBRATION_DATE:
                            break;
                        case RESPIRATION_CRST_SpGET_CALIBRATION:
                            break;
                        case RESPIRATION_CRST_SpGET_EXTENDED_STATUS:
                            break;
                        case RESPIRATION_CRST_SpGET_CAL_TIME:
                            break;
                        case RESPIRATION_CRST_SpGET_SERVICE_TIME:
                            break;
                        case RESPIRATION_CRST_SpGET_LAST_SERVICE_CODE:
                            break;
                        case RESPIRATION_CRST_SpGET_SERVICE_CODES:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case RESPIRATION_CRSP_SpPUT_BREATH:
                            break;
                        case RESPIRATION_CRSP_SpPUT_AMBIENT_PRESSURE:
                            break;
                        case RESPIRATION_CRSP_SpPUT_MODULE_CONFIG:
                            break;
                        case RESPIRATION_CRSP_SpPUT_CALIBRATION_DATE:
                            break;
                        case RESPIRATION_CRSP_SpPUT_CALIBRATION:
                            break;
                        case RESPIRATION_CRSP_SpPUT_EXTENDED_STATUS:
                            break;
                        case RESPIRATION_CRSP_SpPUT_CAL_TIME:
                            break;
                        case RESPIRATION_CRSP_SpPUT_SERVICE_TIME:
                            break;
                        case RESPIRATION_CRSP_SpPUT_LAST_SERVICE_CODE:
                            break;
                        case RESPIRATION_CRSP_SpPUT_SERVICE_CODES:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case RESPIRATION_CSS_SpREPORT_BREATH_DETECT:	
						{
						    /*Received Report Breath Detect Message and the expected payload is::
								Object Structure: [CRespDData[EtCO2][FiCO2][RR]]
								1. EtCO2:	A CNumDStd object that has EtCO2 value and Status information.
								2. FiCO2:	A CNumDStd object that has FiCO2 value and Status information. 
								3. Respiration Rate:	A CNumDStd object that has the respiration value and Status information. 
						    */
								
							SYS_PRINTF("Received Report Breath Detect Message\n");

							MsgResult = TRUE;							

							if(pBuffer != NULL)
							{								
								if(pCRespDData->deserialize_all(pBuffer))
								{
									int16 EtCO2Val = 0;
									int16 FiCO2Val = 0;
									int16 RRVal    = 0;

									/*Retrieve and print value*/
									EtCO2Val = pCRespDData->EtCO2.GetValue();
									SYS_PRINTF("EtCO2Val %i\n", EtCO2Val);																	

									FiCO2Val = pCRespDData->FiCO2.GetValue();
									SYS_PRINTF("FiCO2Val %i\n", FiCO2Val);

									RRVal = pCRespDData->RR.GetValue();
									SYS_PRINTF("RRVal %i\n\n", RRVal);

									/*Set Result to TRUE to send out ACK*/
									Result = TRUE;									
								}			
							}
							if(!Result)
						    {
									/*Send a NAK message for failure*/
									SEND_TRAP(pSourceID,				 /*Source ID of the Connection*/
												ClassID,				 /*The ClassID of the Message: FmRESPIRATION, GnSTATUS, RESPIRATION_CSS_SpREPORT_BREATH_DETECT*/
												pCRespDData->id(),		 /*The ClassID of the Object */
												CERR_SpERRAPPINTERNAL,	 /*ErrorType: Application Internal Error */
												8993,					 /*ErrorCode: 8993. This is user defined within the CRespDData. 
																			Note: There are no Errors currently defined within the object.
																			This is for demonstration purposes only.*/
												NULL,		 			 /*Serialize CRespDData object: Optional*/  
												0);                      /*Size of the CRespDData object: Optional*/
							}
							else
							{
								/*Send a ACK message for Success*/
								SEND_TRAP(pSourceID,       /*Source ID of the Connection*/
										ClassID,           /*The ClassID of the Message: FmRESPIRATION, GnSTATUS, RESPIRATION_CSS_SpREPORT_BREATH_DETECT*/
										ClassID,           /*The ClassID of the Message*/
										CERR_SpERRNONE,    /*ErrorType: NONE = Ack */
										0,                 /*ErrorCode: 0*/
										NULL,              /*Serialize CErrEStd object: Optional*/  
										0);                /*Size of the CErrEStd object: Optional*/
							}
							break;
						}
                        case RESPIRATION_CSS_SpREPORT_BREATH:
						{
						    /*Received Report Breath Message and the expected payload is:
                                Object Structure: [CRespDData[EtCO2][FiCO2][RR]]
                                1. EtCO2:	A CNumDStd object that has EtCO2 value and Status information.
								2. FiCO2:	A CNumDStd object that has FiCO2 value and Status information. 
								3. Respiration Rate:	A CNumDStd object that has the respiration value and Status information. 
							*/
								
							SYS_PRINTF("Received Report Breath Message\n");

							MsgResult = TRUE;
							
							if(pBuffer != NULL)
							{								
								if(pCRespDData->deserialize_all(pBuffer))
								{
									int16 EtCO2Val = 0;
									int16 FiCO2Val = 0;
									int16 RRVal    = 0;

									/*Retrieve and print value*/
									EtCO2Val = pCRespDData->EtCO2.GetValue();
									SYS_PRINTF("EtCO2Val %i\n", EtCO2Val);																	

									FiCO2Val = pCRespDData->FiCO2.GetValue();
									SYS_PRINTF("FiCO2Val %i\n", FiCO2Val);

									RRVal = pCRespDData->RR.GetValue();
									SYS_PRINTF("RRVal %i\n\n", RRVal);

									/*Set Result to TRUE to send out ACK*/
									Result = TRUE;
								}	
							}
							if(!Result)
							{
								/*Send a NAK message for failure*/
									SEND_TRAP(pSourceID,				/*Source ID of the Connection*/
                                              ClassID,					/*The ClassID of the Message: FmRESPIRATION, GnSTATUS, RESPIRATION_CSS_SpREPORT_BREATH*/
                                              pCRespDData->id(),		/*The ClassID of the Object */
                                              CERR_SpERRAPPINTERNAL,	/*ErrorType: Application Internal Error */
                                              8993,						/*ErrorCode: 8993. This is user defined within the CRespDData. 
																			Note: There are no Errors currently defined within the object.
																			This is for demonstration purposes only.*/
                                              NULL,						/*Serialize CErrEStd object: Optional*/  
                                              0);  
							}
							else
							{
								/*Send a ACK message for Success*/
								SEND_TRAP(pSourceID,       /*Source ID of the Connection*/
										ClassID,           /*The ClassID of the Message: FmRESPIRATION, GnSTATUS, RESPIRATION_CSS_SpREPORT_BREATH*/
										ClassID,           /*The ClassID of the Message*/
										CERR_SpERRNONE,    /*ErrorType: NONE = Ack */
										0,                 /*ErrorCode: 0*/
										NULL,              /*Serialize CErrEStd object: Optional*/  
										0);                /*Size of the CErrEStd object: Optional*/
							}
                            break;
						}
                        case RESPIRATION_CSS_SpREPORT_PI:
						{						
						    /*Received Module response message the expected payload is:
								Object structure: [CRespDIPI[PulmonaryIndex]]	
								1. PulmonaryIndex:	A CNumDStd object that has the respiration value and Status information						
						    */
							SYS_PRINTF("Received Report PI Message\n");

							MsgResult = TRUE;
							if(pBuffer != NULL)
							{							

								/*Deserialize the CRespDIPI*/
								if(pCRespDIPI->deserialize_all(pBuffer))
								{									
									int16 PulmonaryIndex = 0;
									
									PulmonaryIndex = pCRespDIPI->PulmonaryIndex.GetValue();	
									SYS_PRINTF("PulmonaryIndex %i\n\n", PulmonaryIndex);

									/*Set Result to TRUE to send out ACK*/
									Result = TRUE;
								}
							}
							if(!Result)
							{
								/*Send a NAK message for failure*/
									SEND_TRAP(pSourceID,				/*Source ID of the Connection*/
                                              ClassID,					/*The ClassID of the Message: FmRESPIRATION, GnSTATUS, RESPIRATION_CSS_SpREPORT_PI*/
                                              pCRespDData->id(),		/*The ClassID of the Object */
                                              CERR_SpERRAPPINTERNAL,	/*ErrorType: Application Internal Error */
                                              8993,						/*ErrorCode: 8993. This is user defined within the CRespDData. 
																		  Note: There are no Errors currently defined within the object.
																		  This is for demonstration purposes only.*/
                                              NULL,						/*Serialize CRespDData object: Optional*/  
                                              0);						/*Size of the CRespDData object: Optional*/
							}
							else
							{
								/*Send a ACK message for Success*/
									SEND_TRAP(pSourceID,       /*Source ID of the Connection*/
											ClassID,           /*The ClassID of the Message: FmRESPIRATION, GnSTATUS, RESPIRATION_CSS_SpREPORT_PI*/
											ClassID,           /*The ClassID of the Message*/
											CERR_SpERRNONE,    /*ErrorType: NONE = Ack */
											0,                 /*ErrorCode: 0*/
											NULL,              /*Serialize CErrEStd object: Optional*/  
											0);                /*Size of the CErrEStd object: Optional*/
							}
                            break;
						}
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTREAM:
                {
                    switch(Species)
                    {
                        case RESPIRATION_CSTR_SpSTREAM_CO2:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CRSPDDTA_SpDATA:
                            break;
                        case CRSPDIPI_SpPULMONARYINDEX:
                            break;
                        case CRSPDCO2_SpCO2:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        case FmSPO2:
        {
            switch(Genus)
            {
                case GnCOMMAND:
                {
                    switch(Species)
                    {
                        case SPO2_CCMD_SpSTART_ACQUIRE:
                            break;
                        case SPO2_CCMD_SpSTOP_ACQUIRE:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case SPO2_CRST_SpGET_SPO2:
                            break;
                        case SPO2_CRST_SpGET_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case SPO2_CRSP_SpPUT_SPO2:
                            break;
                        case SPO2_CRSP_SpPUT_CONFIG:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTATUS:
                {
                    switch(Species)
                    {
                        case SPO2_CSS_SpREPORT_SPO2:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnSTREAM:
                {
                    switch(Species)
                    {
                        case SPO2_CSTR_SpSTREAM_SPO2:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                case GnDATA:
                {
                    switch(Species)
                    {
                        case CSP2DDTA_SpDATA:
                            break;
                        default:
                            ErrorCode = CERR_PROC_UNSUPPORTED;
                            break;
                    }
                    break;
                }
                default:
                {
                    ErrorCode = CERR_PROC_UNSUPPORTED;
                    break;
                }
            }
            break;
        }
        default:
            ErrorCode = CERR_PROC_UNSUPPORTED;
            break;
    }

    /* Send a TRAP message if we have a unimplemented or unsupported error */
    if (!MsgResult)
    {
        SendMsgProcTrap(pSourceID, ClassID, ErrorCode);
    }

    return(Result);
}

/*================================================================================
RecvError :
================================================================================*/
uint32 CFamilyStub::RecvError(char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer)
{
    uint32 RetVal  = 0;
#if defined(DEBUG_PRINT_STATUS)
    uint16 Family  = GetFamily(ClassID);
    uint8  Genus   = GetGenus(ClassID);
    uint8  Species = GetSpecies(ClassID);

    if(ErrorType == CERR_SpERRNONE)
    {
        /* Handle simple ACK notification */
        SYS_PRINTF("\nRxError: %s Ack #%d F:%d G:%d S:%d\n", pSourceID, ++AckCount, Family, Genus, Species);
    }
    else
    {
        /* Handle simple NAK notification */
        SYS_PRINTF("\nRxError: %s NAK #%d F:%d G:%d S:%d\n", pSourceID, ++ErrCount, Family, Genus, Species);

        if(pBuffer != NULL)
        {
            CErrEStd ErrorObj;

            /* Deserialize the Error object and strip off the Family, Genus, and Species */
            RetVal = ErrorObj.deserialize_all(pBuffer);
            if(RetVal)
            {
                /* Handle NACKs */
                switch(ErrorType)
                {
                    case CERR_SpERRWACPSTACK:
                    case CERR_SpERRMSGPROC:
                    {
                        uint32  XMLSize;
                        char    *Temp;

                        if (ErrorType == CERR_SpERRWACPSTACK)
                            SYS_PRINTF("\n%s RxError: CERR_SpERRWACPSTACK error\n", pSourceID);
                        else
                            SYS_PRINTF("\n%s RxError: CERR_SpERRMSGPROC error\n", pSourceID);

                        XMLSize = ErrorObj.serialize_to_XMLbuf(NULL);
                        Temp = (char*)SYS_MALLOC(XMLSize);
                        if (Temp != NULL)
                        {
                            ErrorObj.serialize_to_XMLbuf(Temp);
                            SYS_PRINTF("%s\n", Temp);
                            SYS_FREE(Temp);
                            break;
                        }
                    }
                    case CERR_SpERRAPPINTERNAL:
                    {
                        uint16 DescBufSize = 0;
                        char *pTempBuf = ErrorObj.GetDescription(&DescBufSize);

                        if (pTempBuf)
                        {
                            uint32  XMLSize;
                            char    *Temp;

                            SYS_PRINTF("\n%s RxError: CERR_SpERRAPPINTERNAL error\n", pSourceID);

                            XMLSize = ErrorObj.serialize_to_XMLbuf(NULL);
                            Temp = (char*)SYS_MALLOC(XMLSize);
                            if (Temp != NULL)
                            {
                                ErrorObj.serialize_to_XMLbuf(Temp);
                                SYS_PRINTF("%s\n", Temp);
                                SYS_FREE(Temp);
                                break;
                            }
                        }
                        break;
                    }
                }/* switch(ErrorType) */
            }/* if(RetVal) */
        }/* if(Buffer != NULL) */
    }
#else
   TOUCH(pSourceID);
   TOUCH(ErrorType);
   TOUCH(ClassID);
   TOUCH(pBuffer);
#endif
    return RetVal;
}

/*================================================================================
RecvConn : Handles connection status message based on the ConnType.
================================================================================*/
uint32 CFamilyStub::RecvConnection(char *pSourceID, uint8 ConnType, uint8 *pBuffer)
{
    uint32 Result  = 0;

    TOUCH(pBuffer);

    switch(ConnType)
    {
        case CONN_CMD_SpSHUTDOWN:
            SYS_PRINTF("\n\nRecvConnection: CONN_CMD_SpSHUTDOWN The session has been terminated\n\n");
            bExitFlag = TRUE;
            if(bServer)
                Terminate();
            break;
        case CONN_CEVNT_SpCONNECTED:
            SYS_PRINTF("RecvConnection: CONN_CEVNT_SpCONNECTED %s Connected\n", pSourceID);
            break;
        default:
            SYS_PRINTF("RecvConnection: Unknown ConnectionType\n", pSourceID);
            break;
    }

    return Result;
}
/* End of File */
