/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// ServerMain.cpp                                                    //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.7.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CWACPStack.h"
#include "CFamilyStub.h"
#include "CServerStub.h"

#if defined(STACK_SERIAL)
#include "CSerialCommSvr.h"
#endif

#if defined(STACK_ETHERNET)
#include "CEthCommSvr.h"
#endif

#if defined(STACK_USB)
#include "CUSBCommSvr.h"
#endif

#include "CFmDEVICEStub.h"
#include "CFmMODULEStub.h"
#include "CFmERRORStub.h"
#include "CFmRESPIRATIONStub.h"
#include "CFmSPO2Stub.h"

void TestServerStub();
void TestClientStub(boolean bByPass);

/* Global Instances */
CWACPStack *pStack = NULL;
CFmDEVICEStub *pFmDEVICE = NULL;
CFmMODULEStub *pFmMODULE = NULL;
CFmERRORStub *pFmERROR = NULL;
CFmRESPIRATIONStub *pFmRESPIRATION = NULL;
CFmSPO2Stub *pFmSPO2 = NULL;

CFamilyStub *pClientStubClass = new CFamilyStub();
CServerStub *pServerStubClass = new CServerStub();

/*********************************************************************************
main:
*********************************************************************************/
#if defined(_WIN32_WCE)
#if defined(_WCE_CONSOLE)

int WinEntry (void *Args)
{
    _newprintf = ((formatter*)Args)[0];
    _newscanf = ((formatter*)Args)[1];

#else
int WINAPI WinMain( HINSTANCE   hInstance,
               HINSTANCE   hPrevInstance,
                LPTSTR  lpCmdLine,
                int nCmdShow)
{
#endif
#else
int main(void)
{
#endif /* _WIN32_WCE */

    int TestID = 1;

    while (TestID != 0)
    {
        SYS_PRINTF("0. Exit\n");
        SYS_PRINTF("1. Start up Server\n");
        SYS_PRINTF("2. Start up Client\n");
        SYS_PRINTF("Enter Selection:");

        scanf("%d", &TestID);
        if (TestID == 0)
        {
            break;
        }
        else if (TestID == 1)
        {
            int  Result = TRUE;
            uint16 i;
            uint32 ConnectionCount;
            uint32 Data;
            ConnArgs **Config = NULL;

            SYS_PRINTF("Enter number of connections: 0..255 (0 to exit)");
            Result = (uint16)scanf("%d", &ConnectionCount);
            ConnectionCount %= 256;
            Config = (ConnArgs**)SYS_MALLOC(ConnectionCount * sizeof(ConnArgs*));
            for (i=0; i < ConnectionCount; i++)
            {
                int StackType;
                SYS_PRINTF("\n0. Serial\n");
                SYS_PRINTF("1. TCP/IP\n");
                SYS_PRINTF("2. USB\n");
                Result = (uint16)scanf("%d", &StackType);
                StackType++;
                if (StackType == SERIAL)
                {
                    Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->pConfig = (SerialAddr*)SYS_MALLOC(sizeof(SerialAddr));
                    Config[i]->StackType = SERIAL;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
                    Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                    Config[i]->bSvrFlag = TRUE;

                    SYS_PRINTF("Enter Comm Port: ");
                    scanf("%d", &Data);
                    sprintf(Config[i]->SourceID, "COM%d", Data);

                    SYS_PRINTF("\n1. 9600\n");
                    SYS_PRINTF("2. 115200\n");
                    SYS_PRINTF("3. 230400\n");
                    SYS_PRINTF("Select Baud Rate:\n");

                    scanf("%d", &Data);
                    switch(Data)
                    {
                        case 1:
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 9600;
                            break;
                        case 2:
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 115200;
                            break;
                        case 3:
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 230400;
                            break;
                        default:
                            SYS_PRINTF("Invalid selection defaulting to %d\n", 9600);
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 9600;
                            break;
                    }
                    ((SerialAddr*)(Config[i]->pConfig))->bAutoDiscovery = TRUE;

                    SYS_PRINTF("\n1. ByPass Disabled\n");
                    SYS_PRINTF("2. ByPass Enabled\n");
                    SYS_PRINTF("Enter ByPass Configuration: ");

                    scanf("%d", &Data);
                    if(Data == 2)
                    {
                        Config[i]->bByPass = TRUE;
                    }
                    else
                    {
                        Config[i]->bByPass = FALSE;
                    }
                }
                else if (StackType == USB)
                {
                    Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->StackType = USB;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->bSvrFlag = TRUE;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
                    Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                }
                else
                {
                    Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->pConfig = (EthAddr*)SYS_MALLOC(sizeof(EthAddr));
                    Config[i]->StackType = ETHERNET;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->bSvrFlag = TRUE;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
                    Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                    SYS_PRINTF("\n1. TCP\n");
                    SYS_PRINTF("2. UDP\n");
                    SYS_PRINTF("Enter Protocol:");
                    scanf("%d", &Data);

                    /* Default to TCP unless UDP is specified */
                    if (Data == 2)
                        ((EthAddr*)(Config[i]->pConfig))->Protocol = IPPROTO_UDP;
                    else
                        ((EthAddr*)(Config[i]->pConfig))->Protocol = IPPROTO_TCP;
                }
            }

            if (pStack == NULL)
                pStack = new CWACPStack(RNDZ_PORT_DEFAULT, WACP_PORT_DEFAULT, THREAD_PRIORITY_HIGHEST, pServerStubClass);

            pStack->StartServer(Config, (uint8)ConnectionCount);

            TestServerStub();

            delete(pStack);
            pStack = NULL;

           /* Free Config Allocations */
           for (i=0; i < ConnectionCount; i++)
           {
               SYS_FREE(Config[i]->pConfig);
               SYS_FREE(Config[i]);
           }
           SYS_FREE(Config);
        }
        else if (TestID == 2)
        {
            if (pStack == NULL)
                pStack = new CWACPStack(RNDZ_PORT_DEFAULT, WACP_PORT_DEFAULT, THREAD_PRIORITY_HIGHEST, pClientStubClass);

            pStack->StartClient();

            TestClientStub(FALSE);

            delete(pStack);
            pStack = NULL;

            pClientStubClass->Reset();
        }
        else
        {
            SYS_PRINTF("\nInvalid Selection\n\n");
        }
    }

    /*Clean up*/
    if (pServerStubClass)
        delete(pServerStubClass);

    if (pClientStubClass)
        delete(pClientStubClass);

    return (0);
}
/*********************************************************************************
TestClientStub:
*********************************************************************************/
void TestClientStub(boolean bByPass)
{
    uint32   TestID = 1;
    uint32   CmdRqst = 0;
    uint32   Result = TRUE;

    while (pClientStubClass->bExitFlag != TRUE)
    {
        /* Wait until AppInit callback is received */
        if(pClientStubClass->bAppInit)
        {
#if defined(PC)
            /* Create new instances of stub classes once we receive AppInit */
            pFmDEVICE = new CFmDEVICEStub(pClientStubClass);
            pFmMODULE = new CFmMODULEStub(pClientStubClass);
            pFmERROR = new CFmERRORStub(pClientStubClass);
            pFmRESPIRATION = new CFmRESPIRATIONStub(pClientStubClass);
            pFmSPO2 = new CFmSPO2Stub(pClientStubClass);

            uint16 i;
            uint32 ConnectionCount;
            uint32 Data;
            char   *pSourceID = NULL; // Fill this in with the current connection SourceID
            ConnArgs **Config = NULL;

            SYS_PRINTF("Enter number of connections: 0..255 (0 to exit)");
            Result = (uint16)scanf("%d", &ConnectionCount);
            ConnectionCount %= 256;
            Config = (ConnArgs**)SYS_MALLOC(ConnectionCount * sizeof(ConnArgs*));
            for (i=0; i < ConnectionCount; i++)
            {
                int StackType;
                SYS_PRINTF("\n0. Serial\n");
                SYS_PRINTF("1. TCP/IP\n");
                Result = (uint16)scanf("%d", &StackType);
                StackType++;
                if (StackType == SERIAL)
                {
                    Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->pConfig = (SerialAddr*)SYS_CALLOC(sizeof(SerialAddr), 1);
                    Config[i]->StackType = SERIAL;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
                    Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                    ((SerialAddr*)(Config[i]->pConfig))->hPort = INVALID_HANDLE_VALUE;

                    SYS_PRINTF("1. Start up Host\n");
                    SYS_PRINTF("2. Start up Client\n");
                    SYS_PRINTF("Enter Selection:");
                    scanf("%d", &Data);

                    if (Data == 1)
                        Config[i]->bSvrFlag = TRUE;
                    else
                        Config[i]->bSvrFlag = FALSE;

                    SYS_PRINTF("Enter Comm Port: ");
                    scanf("%d", &Data);

                    sprintf(Config[i]->SourceID, "COM%d", Data);

                    SYS_PRINTF("\n1. 9600\n");
                    SYS_PRINTF("2. 115200\n");
                    SYS_PRINTF("3. 230400\n");
                    SYS_PRINTF("Select Baud Rate:\n");

                    scanf("%d", &Data);
                    switch(Data)
                    {
                        case 1:
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 9600;
                            break;
                        case 2:
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 115200;
                            break;
                        case 3:
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 230400;
                            break;
                        default:
                            SYS_PRINTF("Invalid selection defaulting to %d\n", 9600);
                            ((SerialAddr*)(Config[i]->pConfig))->BaudRate = 9600;
                            break;
                    }
                    if (Config[i]->bSvrFlag)
                    {
                        SYS_PRINTF("\n0. Disable DTR\n");
                        SYS_PRINTF("1. Enable DTR\n");
                        SYS_PRINTF("Enter Selection:");
                        scanf("%d", &Data);
                        ((SerialAddr*)(Config[i]->pConfig))->bAutoDiscovery = (boolean)Data;
                    }
                }
                else if (StackType == ETHERNET)
                {
                    Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
                    Config[i]->pConfig = (EthAddr*)SYS_MALLOC(sizeof(EthAddr));
                    Config[i]->StackType = ETHERNET;
                    Config[i]->RNDZPort = RNDZ_PORT_DEFAULT;
                    Config[i]->WACPPort = WACP_PORT_DEFAULT;
                    Config[i]->bSvrFlag = FALSE;
                    Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
                    Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
                    SYS_PRINTF("\n1. TCP\n");
                    SYS_PRINTF("2. UDP\n");
                    SYS_PRINTF("Enter Protocol:");
                    scanf("%d", &Data);

                    /* Default to TCP unless UDP is specified */
                    if (Data == 2)
                        ((EthAddr*)(Config[i]->pConfig))->Protocol = IPPROTO_UDP;
                    else
                        ((EthAddr*)(Config[i]->pConfig))->Protocol = IPPROTO_TCP;

                    SYS_PRINTF("ServIP Address:");
                    scanf("%s", Config[i]->SourceID);

                    /* if "0" default to loopback*/
                    if (strcmp(Config[i]->SourceID,"0") == 0)
                    {
                        /* ETHERNET TEST */
                        SYS_PRINTF("Defaulting to Loopback IP 127.0.0.%d\n", i + 1);
                        sprintf(Config[i]->SourceID, "%s%d", "127.0.0.", i + 1);
                    }
                }/* else if (StackType == ETHERNET) */
            }/* for (i=0; i < ConnectionCount; i++) */

            /* Open up the communications channels */
            Result = pClientStubClass->Open(Config, (uint8)ConnectionCount);
            if (Result)
            {
                /* Cycle through the connection list and connect eac client */
                for (i=0; i<ConnectionCount; i++)
                {
                    Result = pClientStubClass->Connect(Config[i]->SourceID);
                    if (!Result)
                    {
                        pClientStubClass->bExitFlag = TRUE;
                        SYS_PRINTF("Unable to connect to %s\n", Config[i]->SourceID);
                    }
                }
            }
            else
                pClientStubClass->bExitFlag = TRUE;

            /* Set the context via the SourceID */
            pSourceID = Config[0]->SourceID;

            while (TestID != 0)
            {
                if(!pClientStubClass->bExitFlag)
                {
                    SYS_PRINTF("0. Disconnect\n");
                    SYS_PRINTF("1. Set Error Configuration\n");
                    SYS_PRINTF("2. Set Connection Configuration\n");
                    SYS_PRINTF("3. FmDEVICE Test\n");
                    SYS_PRINTF("4. FmMODULE Test\n");
                    SYS_PRINTF("5. FmERROR Test\n");
                    SYS_PRINTF("6. FmRESPIRATION Test\n");
                    SYS_PRINTF("7. FmSPO2 Test\n");
                    SYS_PRINTF("8. Get GUIDObj\n");
                    SYS_PRINTF("9. Get GUID\n");
                    SYS_PRINTF("10. Set current connection\n");
                    SYS_PRINTF("Enter selection: ");

                    Result = (uint16)scanf("%d", &TestID);
                    if(pClientStubClass->bExitFlag)
                        TestID = 0;
                }
                else
                {
                    Result = 1;
                    TestID = 0;
                }

                if(Result == 1)
                {
                    if(TestID == 0)
                    {
                        SYS_PRINTF("Disconnecting WACP Connection\n");
                        for (i=0; i<ConnectionCount; i++)
                        {
                            pClientStubClass->Close(Config[i]->SourceID);
                        }
                        
                        SYS_PRINTF("0. Exit\n");
                        SYS_PRINTF("1. Connect\n");
                        Result = (uint16)scanf("%d", &TestID);

                        if(TestID == 1)
                        {
                            pClientStubClass->bExitFlag = FALSE;

                            /* Open up the communications channels */
                            Result = pClientStubClass->Open(Config, (uint8)ConnectionCount);

                            if (Result)
                            {
                                /* Cycle through the connection list and connect eac client */
                                for (i=0; i<ConnectionCount; i++)
                                {
                                    /* Only connect if we are a client. A host device will not get into AppInit until
                                    * the connection is established
                                    */
                                    if(Config[i]->bSvrFlag == FALSE)
                                    {
                                        Result = pClientStubClass->Connect(Config[i]->SourceID);
                                        if (!Result)
                                        {
                                            pClientStubClass->bExitFlag = TRUE;
                                            SYS_PRINTF("Unable to connect to %s\n", Config[i]->SourceID);
                                        }
                                    }
                                }
                            }
                            else
                                break;
                        }
                        else
                        {
                            pClientStubClass->bExitFlag = TRUE;
                            if(bByPass != TRUE)
                            {
                                pClientStubClass->Terminate();
                            }
                            break;
                        }
                    }
                    else if(TestID == 1)
                    {
                        SYS_PRINTF("\t1. Get Last Error\n");
                        SYS_PRINTF("\t2. Send CERR_SpAPPINTERNAL error message\n");
                        SYS_PRINTF("\nEnter Configuration: ");

                        Result = (uint16)scanf("%d", &CmdRqst);
                        if(pClientStubClass->bExitFlag)
                            continue;

                        if(Result == 1)
                        {
                            switch(CmdRqst)
                            {
                                case 1:
                                {
                                    pClientStubClass->SendMsg(pSourceID, FmTRAP, GnREQUEST, TRAP_CRST_SpGET_LASTERROR, NULL, 0, FALSE);
                                    break;
                                }
                                case 2:
                                {
                                    uint32  ClassID = MakeClassID(FmARCH, GnDATA, 0);
                                    char    TempString[] = "Testing AppInternal Error";

                                  //  pClientStubClass->SendTrap(pSourceID, ClassID, ClassID, CERR_SpERRAPPINTERNAL, CERR_APP_NONE = 0, (uint8*)TempString, sizeof(TempString));
                                    break;
                                }
                            }
                        }
                    }
                    else if(TestID == 2)
                    {
                        time_t Interval, TimeOut;
                        CONN_CFG_SPECIES Options;

                        SYS_PRINTF("\n\t0. Exit\n");
                        SYS_PRINTF("\t1. SET options\n");
                        SYS_PRINTF("\t2. GET options\n");
                        SYS_PRINTF("\nEnter Configuration: ");

                        scanf("%d", &Result);
                        switch(Result)
                        {
                            case 0:
                                break;
                            case 1:
                            {
                                SYS_PRINTF("\n\t0. CONN_CFG_SpKEEPALIVEON\n");
                                SYS_PRINTF("\t1. CONN_CFG_SpKEEPALIVEOFF\n");
                                SYS_PRINTF("\nEnter Configuration: ");

                                Result = (uint16)scanf("%d", &CmdRqst);
                                Options = (CONN_CFG_SPECIES)CmdRqst;
                                if(pClientStubClass->bExitFlag)
                                    continue;

                                if(Result == 1)
                                {
                                    if(CmdRqst <= 1)
                                    {
                                        SYS_PRINTF("\nEnter KeepAlive Interval (0 for default): ");
                                        Result = (uint16)scanf("%d", &CmdRqst);
                                        Interval = CmdRqst;
                                        if(pClientStubClass->bExitFlag)
                                            continue;

                                        SYS_PRINTF("\nEnter KeepAlive Timeout (0 for default): ");
                                        Result = (uint16)scanf("%d", &CmdRqst);
                                        TimeOut = CmdRqst;
                                        if(pClientStubClass->bExitFlag)
                                            continue;

                                        if (Interval == 0)
                                            Interval = DEFAULT_KEEPALIVE_INTERVAL;

                                        if (TimeOut == 0)
                                            TimeOut = DEFAULT_TIMEOUT_INTERVAL;

                                        pClientStubClass->SetConnOptions(pSourceID, Options, TRUE, Interval, TimeOut);
                                    }
                                }
                                break;
                            }
                            case 2:
                            {
                                pClientStubClass->GetConnOptions(pSourceID, &Options, &Interval, &TimeOut);
                                SYS_PRINTF("\nOption = %d\nInterval = %d\nTimeOut = %d\n", Options, Interval, TimeOut);
                                break;
                            }
                            default:
                                SYS_PRINTF("Invalid Selection\n");
                                break;
                        }
                    }
                    else if(TestID == 3)
                    {
                        pFmDEVICE->AppInit(pSourceID);
                    }
                    else if(TestID == 4)
                    {
                        pFmMODULE->AppInit(pSourceID);
                    }
                    else if(TestID == 5)
                    {
                        pFmERROR->AppInit(pSourceID);
                    }
                    else if(TestID == 6)
                    {
                        pFmRESPIRATION->AppInit(pSourceID);
                    }
                    else if(TestID == 7)
                    {
                        pFmSPO2->AppInit(pSourceID);
                    }
                    else if(TestID == 8)
                    {
#if((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
                        CRNDZDStd* pGUIDObj = pClientStubClass->GetGUIDObj(pSourceID);
                        if(pGUIDObj != NULL)
                        {
                            uint32 XMLSize = pGUIDObj->size_XML();
                            char *pTemp = (char*)SYS_MALLOC(XMLSize);
                            if (pTemp != NULL)
                            {
                                pGUIDObj->serialize_to_XMLbuf(pTemp);
                                SYS_PRINTF("\n%s\n", pTemp);
                                SYS_FREE(pTemp);
                            }
                        }
#endif
                    }
                    else if(TestID == 9)
                    {
                        uint16 Size = 0;
                        char   GUIDBuf[64];
                        uint8* pGUID = pClientStubClass->GetGUID(pSourceID, &Size);
                        if(pGUID != NULL)
                        {
                            GUIDToStr(pGUID, GUIDBuf, 32);
                            SYS_PRINTF("\n%s\n\n", GUIDBuf);
                        }
                    }
                    else if(TestID == 10)
                    {
                        int CurrentConfig;

                        SYS_PRINTF("\t0. Exit\n");
                        for (i=0; i<ConnectionCount; i++)
                        {
                            SYS_PRINTF("\t%d. %s\n", i+1, Config[i]->SourceID);
                        }
                        SYS_PRINTF("\nEnter Session: ");

                        scanf("%d", &CurrentConfig);

                        if (CurrentConfig-- == 0)
                            break;

                        if ((uint8)CurrentConfig < ConnectionCount)
                        {
                            pSourceID = Config[CurrentConfig]->SourceID;
                            SYS_PRINTF( "Current session set to %s\n", pSourceID);
                        }
                        else
                            SYS_PRINTF("Invalid selection\n");
                    }
                    else
                    {
                        SYS_PRINTF("\n\nInvalid Selection.\n\n");
                    }
                }
                else
                {
                    SYS_PRINTF("\n\nInvalid Selection.\n\n");
                }
            }/* while (TestID != 0) */

            SYS_SLEEP(1);

            for (i=0; i<ConnectionCount; i++)
            {
                SYS_FREE(Config[i]->pConfig);
                SYS_FREE(Config[i]);
            }

            SYS_FREE(Config);

            delete(pFmDEVICE);
            delete(pFmMODULE);
            delete(pFmERROR);
            delete(pFmRESPIRATION);
            delete(pFmSPO2);
#else
            pClientStubClass->bExitFlag = FALSE;

            /* Loop until the connection is closed */
            if(Result)
            {
                while(pClientStubClass->bExitFlag == FALSE)
                    SYS_SLEEP(WAIT_COUNT);

                /* Close the WACP session(s) */
                /* pWACPApp->Close(pSourceID); */
            }
#endif  /*  #if defined(PC) */
        
        }/* if(pClientStubClass->bAppInit) */
        else
        {
            SYS_SLEEP(100);
        }
    }/* while (pClientStubClass->bExitFlag != TRUE) */
}


void TestServerStub()
{
    int  Result = TRUE;
    int  Temp = 0;
    int  TestID = 1;
    uint32   CmdRqst = 0;
    char *pSourceID = NULL;


    while (Result != 0)
    {
        SYS_PRINTF("\n\n========== MENU =============\n");
        SYS_PRINTF("0. Exit Host\n");
        SYS_PRINTF("1. Show Connections\n");
        SYS_PRINTF("2. Terminate Connection\n");
        
        /*This Menu Option allows the Server to use the Console Menu and methods in
        CFmFILESYSStub*/
        SYS_PRINTF("3. Client Connections\n");
        scanf("%d", &Result);
        switch(Result)
        {
            case 0:
                for (uint32 i = pServerStubClass->GetConnCount(); i > 0; i--)
                {
                    pServerStubClass->Close(pServerStubClass->GetConnSourceID(i));
                }
                /* Terminate all the connections */
                pServerStubClass->TerminateAll();
                break;
            case 1:
            case 2:
            {
                SYS_PRINTF("\n\n======== CONNECTIONS ========\n\n");

                if (pServerStubClass->DisplayConnections())
                {
                    /* Check for termination requests */
                    if (Result == 2)
                    {
                        SYS_PRINTF("    0. All connections\n");
                        SYS_PRINTF("\n    Enter Client Number:");
                        scanf("%d", &Temp);
                        if (Temp == 0)
                        {
                            for (uint32 i = pServerStubClass->GetConnCount(); i > 0; i--)
                            {
                                pServerStubClass->Close(pServerStubClass->GetConnSourceID(i));
                            }
                            /* Terminate all the connections */
                            pServerStubClass->TerminateAll();
                        }
                        else
                        {
                            pServerStubClass->Close(pServerStubClass->GetConnSourceID(Temp));
                            boolean bResult = pServerStubClass->TerminateConn(Temp);

                            /* Check to see if the requested connection was terminated */
                            if (!bResult)
                                SYS_PRINTF("Unable to disconnect selection %d\n", Temp);
                        }
                    }
                }
                else
                    SYS_PRINTF("No connections found\n");

                break;
            }
            case 3:

                TestID = 1;
                if(pServerStubClass->GetConnCount() > 0)
                {
                    /*Set to connection at index zero by default*/
                    pSourceID = pServerStubClass->GetConnSourceID(1);

                                
                    pFmDEVICE = new CFmDEVICEStub((CFamilyStub*)pServerStubClass->GetConnFamilyStub(1));
					pFmMODULE = new CFmMODULEStub((CFamilyStub*)pServerStubClass->GetConnFamilyStub(1));
					pFmERROR = new CFmERRORStub((CFamilyStub*)pServerStubClass->GetConnFamilyStub(1));
					pFmRESPIRATION = new CFmRESPIRATIONStub((CFamilyStub*)pServerStubClass->GetConnFamilyStub(1));
					pFmSPO2 = new CFmSPO2Stub((CFamilyStub*)pServerStubClass->GetConnFamilyStub(1));
                    while (TestID != 0)
                    {
                        SYS_PRINTF("0. Disconnect\n");
						SYS_PRINTF("1. Set Error Configuration\n");
						SYS_PRINTF("2. Set Connection Configuration\n");
						SYS_PRINTF("3. FmDEVICE Test\n");
						SYS_PRINTF("4. FmMODULE Test\n");
						SYS_PRINTF("5. FmERROR Test\n");
						SYS_PRINTF("6. FmRESPIRATION Test\n");
						SYS_PRINTF("7. FmSPO2 Test\n");
						SYS_PRINTF("8. Get GUIDObj\n");
						SYS_PRINTF("9. Get GUID\n");
						SYS_PRINTF("10. Set current connection\n");
						SYS_PRINTF("Enter selection: ");

                        Result = (uint16)scanf("%d", &TestID);

                        if(Result == 1)
                        {
                            if(TestID == 0)
                            {
                                break;
                            }
                            else if(TestID == 1)
                            {
                                SYS_PRINTF("\t1. Get Last Error\n");
                                SYS_PRINTF("\t2. Send CERRESTD_SpAPPINTERNAL error message\n");
                                SYS_PRINTF("\nEnter Configuration: ");

                                Result = (uint16)scanf("%d", &CmdRqst);
                                if(pServerStubClass->bExitFlag)
                                    continue;

                                if(Result == 1)
                                {
                                    switch(CmdRqst)
                                    {
                                        case 1:
                                        {
                                            pServerStubClass->SendMsg(pSourceID, FmTRAP, GnREQUEST, TRAP_CRST_SpGET_LASTERROR, NULL, 0, FALSE);
                                            break;
                                        }
                                        case 2:
                                        {
                                            uint32  ClassID = MakeClassID(FmARCH, GnDATA, 0);
                                            char    TempString[] = "Testing AppInternal Error";

                                           // pServerStubClass-> SendTrap(pSourceID, ClassID, ClassID, CERR_SpERRAPPINTERNAL, CERR_APP_TEST1, (uint8*)TempString, sizeof(TempString));
                                            break;
                                        }
                                    }
                                }
                            }
                            else if(TestID == 2)
                            {
                                time_t Interval, TimeOut;
                                CONN_CFG_SPECIES Options;

                                SYS_PRINTF("\n\t0. Exit\n");
                                SYS_PRINTF("\t1. SET options\n");
                                SYS_PRINTF("\t2. GET options\n");
                                SYS_PRINTF("\nEnter Configuration: ");

                                scanf("%d", &Result);
                                switch(Result)
                                {
                                    case 0:
                                        break;
                                    case 1:
                                    {
                                        SYS_PRINTF("\n\t0. CONN_CFG_SpKEEPALIVEON\n");
                                        SYS_PRINTF("\t1. CONN_CFG_SpKEEPALIVEOFF\n");
                                        SYS_PRINTF("\nEnter Configuration: ");

                                        Result = (uint16)scanf("%d", &CmdRqst);
                                        Options = (CONN_CFG_SPECIES)CmdRqst;
                                        if(pServerStubClass->bExitFlag)
                                            continue;

                                        if(Result == 1)
                                        {
                                            if(CmdRqst <= 1)
                                            {
                                                SYS_PRINTF("\nEnter KeepAlive Interval (0 for default): ");
                                                Result = (uint16)scanf("%d", &CmdRqst);
                                                Interval = CmdRqst;
                                                if(pServerStubClass->bExitFlag)
                                                    continue;

                                                SYS_PRINTF("\nEnter KeepAlive Timeout (0 for default): ");
                                                Result = (uint16)scanf("%d", &CmdRqst);
                                                TimeOut = CmdRqst;
                                                if(pServerStubClass->bExitFlag)
                                                    continue;

                                                if (Interval == 0)
                                                    Interval = DEFAULT_KEEPALIVE_INTERVAL;

                                                if (TimeOut == 0)
                                                    TimeOut = DEFAULT_TIMEOUT_INTERVAL;

                                                pServerStubClass->SetConnOptions(pSourceID, Options, TRUE, Interval, TimeOut);
                                            }
                                        }
                                        break;
                                    }
                                    case 2:
                                    {
                                        pServerStubClass->GetConnOptions(pSourceID, &Options, &Interval, &TimeOut);
                                        SYS_PRINTF("\nOption = %d\nInterval = %d\nTimeOut = %d\n", Options, Interval, TimeOut);
                                        break;
                                    }
                                    default:
                                        SYS_PRINTF("Invalid Selection\n");
                                        break;
                                }
                            }
                            else if(TestID == 3)
							{
								pFmDEVICE->AppInit(pSourceID);
							}
							else if(TestID == 4)
							{
								pFmMODULE->AppInit(pSourceID);
							}
							else if(TestID == 5)
							{
								pFmERROR->AppInit(pSourceID);
							}
							else if(TestID == 6)
							{
								pFmRESPIRATION->AppInit(pSourceID);
							}
							else if(TestID == 7)
							{
								pFmSPO2->AppInit(pSourceID);
							}
							else if(TestID == 8)
							{
		#if((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
								CRNDZDStd* pGUIDObj = pClientStubClass->GetGUIDObj(pSourceID);
								if(pGUIDObj != NULL)
								{
									uint32 XMLSize = pGUIDObj->size_XML();
									char *pTemp = (char*)SYS_MALLOC(XMLSize);
									if (pTemp != NULL)
									{
										pGUIDObj->serialize_to_XMLbuf(pTemp);
										SYS_PRINTF("\n%s\n", pTemp);
										SYS_FREE(pTemp);
									}
								}
		#endif
							}
							else if(TestID == 9)
							{
								uint16 Size = 0;
								char   GUIDBuf[64];
								uint8* pGUID = pClientStubClass->GetGUID(pSourceID, &Size);
								if(pGUID != NULL)
								{
									GUIDToStr(pGUID, GUIDBuf, 32);
									SYS_PRINTF("\n%s\n\n", GUIDBuf);
								}
							}
							else if(TestID == 10)
							{
								int CurrentConfig;

                                SYS_PRINTF("    0. Exit\n");
                                pServerStubClass->DisplayConnections();                      
                                SYS_PRINTF("\nEnter Session: ");
                                scanf("%d", &CurrentConfig);
                                
                                if (CurrentConfig == 0)
                                    break;

                                if ((uint8)CurrentConfig <= pServerStubClass->GetConnCount())
                                {
                                    pSourceID = pServerStubClass->GetConnSourceID(CurrentConfig);
                                    SYS_PRINTF( "Current session set to %s\n", pSourceID);
                                }
                                else
                                    SYS_PRINTF("Invalid selection\n");

							}
                            else
                            {
                                SYS_PRINTF("\n\nInvalid Selection.\n\n");
                            }
                        }
                        else
                        {
                            SYS_PRINTF("\n\nInvalid Selection.\n\n");
                        }
                    }/* while (TestID != 0) */

					delete(pFmDEVICE);
					delete(pFmMODULE);
					delete(pFmERROR);
					delete(pFmRESPIRATION);
					delete(pFmSPO2);
                } /* if(pServerStubClass->GetConnCount > 0) */
            }/* switch(Result) */
        }/* while (Result != 0) */
}


/* End of File */

