/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CServerStub.cpp                                                   //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '// AppGen Version :   4.7.0                                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CWACPApp.h"
#include "CServerStub.h"


CServerStub::CServerStub()
{
    ConnCount = 0;
}

CServerStub::~CServerStub()
{
    /* Erase all entries from the list */
    ConnList.clear();
}

/*================================================================================
AppInit : Virtual Function called by CMOMPBaseClass::AppInitInt
================================================================================*/
CMOMPBaseClass* CServerStub::AppInit(CWACPApp *pWACPAppRef, boolean bSvrFlag, char* pSourceID)
{
    CFamilyStub *pBaseClass = NULL;

    if(pWACPAppRef != NULL)
    {
        ConnStruct *pConnection = new ConnStruct;

        /* Create new CFamilyStub instance */
        pBaseClass = new CFamilyStub();

        /* Initialize the new baseclass */
        pBaseClass->AppInitInt(pWACPAppRef, bSvrFlag, pSourceID);

        /* Keep track of all newly created CFamilyStub instances in Connections STL List */
        pConnection->pClass = pBaseClass;
        pConnection->pSourceID = pSourceID;
        pConnection->pWACPAppPtr = pWACPAppRef;

        /* Add the new connection into the STL list */
        ConnList.push_back(*pConnection);

        /* Update the Count */
        ConnCount++;
    }

    return pBaseClass;
}

/*================================================================================
AppEnd :
================================================================================*/
uint32 CServerStub::AppEnd(CWACPApp *pWACPAppRef)
{
    uint32 Result = 0;
    boolean bFound = FALSE;

    /* Find the queue for this message */
    CONNECTION_LIST::iterator pList;

    /* Search for the node that matches the SourceID */
    for (pList = ConnList.begin(); pList != ConnList.end(); pList++)
    {
        if (pList->pWACPAppPtr == pWACPAppRef)
        {
            Result = pList->pClass->AppEndInt(pWACPAppRef);
            delete(pList->pClass);
			pList->pClass = NULL;
            bFound = TRUE;
            break;
        }
    }

    if (bFound)
    {
        /* Erase the entry from the list */
        ConnList.erase(pList);
        ConnCount--;
    }

    return Result;
}

/*================================================================================
RecvMessage : Handles Command, Request, Response, Status, Event, and DataObject 
              messages.
================================================================================*/
uint32 CServerStub::RecvMessage(char *pSourceID, uint32 ClassID, uint8 *pBuffer)
{
    uint32  Result = 0;
    uint16  Family  = GetFamily(ClassID);
    uint8   Genus   = GetGenus(ClassID);
    uint8   Species = GetSpecies(ClassID);

    TOUCH(pBuffer);
    SYS_PRINTF("\nCServerStub::RecvMessage: %s F:%d G:%d S:%d\n", pSourceID, Family, Genus, Species);

    return Result;
}

/*================================================================================
RecvError :
================================================================================*/
uint32 CServerStub::RecvError(char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer)
{
    uint32 RetVal  = 0;
    uint16 Family  = GetFamily(ClassID);
    uint8  Genus   = GetGenus(ClassID);
    uint8  Species = GetSpecies(ClassID);

    TOUCH(pBuffer);

    if(ErrorType == CERR_SpERRNONE)
    {
        /* Handle simple ACK notification */
        SYS_PRINTF("\nCServerStub::RecvError: %s ACK F:%d G:%d S:%d\n", pSourceID, Family, Genus, Species);
    }
    else
    {
        SYS_PRINTF("\nCServerStub::RecvError: %s NACK #%d F:%d G:%d S:%d\n", pSourceID, Family, Genus, Species);
    }

    return RetVal;
}

/*================================================================================
RecvConn : Handles connection status message based on the ConnType.
================================================================================*/
uint32 CServerStub::RecvConnection(char *pSourceID, uint8 ConnType, uint8 *pBuffer)
{
    uint32 Result  = 0;

    TOUCH(pBuffer);

    switch(ConnType)
    {
        case CONN_CMD_SpSHUTDOWN:
            SYS_PRINTF("\n\nCServerStub::RecvConnection: CONN_CMD_SpSHUTDOWN The session has been terminated\n\n");
            bExitFlag = TRUE;
            if(bServer)
                Terminate();
            break;
        case CONN_CEVNT_SpCONNECTED:
            SYS_PRINTF("CServerStub::RecvConnection: CONN_CEVNT_SpCONNECTED %s Connected\n", pSourceID);
            break;
        default:
            SYS_PRINTF("CServerStub::RecvConnection: Unknown ConnectionType\n", pSourceID);
            break;
    }

    return Result;
}

/*================================================================================
GetConnCount : Returns the number of active connections
================================================================================*/
uint32 CServerStub::GetConnCount()
{
    return ConnCount;
}

/*================================================================================
GetConnCount : Displays the active connections and returns the number of active 
    connections
================================================================================*/
uint32 CServerStub::DisplayConnections()
{
    CONNECTION_LIST::iterator pList;
    uint32 i = 0;

    /* Print the list of connections */
    for (pList = ConnList.begin(), i=0; pList != ConnList.end(); pList++, i++)
    {
        SYS_PRINTF("    %d: %s\n", i+1, pList->pSourceID);
    }

    return i;
}

/*================================================================================
GetConnFamilyStub : Return a class based on an index (1 based)
================================================================================*/
CFamilyStub* CServerStub::GetConnFamilyStub(uint32 Index)
{
    CONNECTION_LIST::iterator pList;
    uint32 i = 0;

    /* Print the list of connections */
    for (pList = ConnList.begin(), i=1; pList != ConnList.end(); pList++, i++)
    {
        if (i==Index)
        {
            return pList->pClass;
        }
    }

    return NULL;
}
/*================================================================================
GetConnSourceID : Return a source ID based on an index (1 based)
================================================================================*/
char* CServerStub::GetConnSourceID(uint32 Index)
{
    CONNECTION_LIST::iterator pList;
    uint32 i = 0;

    /* Print the list of connections */
    for (pList = ConnList.begin(), i=1; pList != ConnList.end(); pList++, i++)
    {
        if (i==Index)
        {
            return pList->pSourceID;
        }
    }

    return NULL;
}
/*================================================================================
GetConnCount : Terminates an active client based on an index (1 based)
================================================================================*/
boolean CServerStub::TerminateConn(uint32 Index)
{
    CONNECTION_LIST::iterator pList;
    boolean Result = FALSE;
    uint32  i;

    /* Print the list of connections */
    for (pList = ConnList.begin(), i=1; pList != ConnList.end(); pList++, i++)
    {
        if (i==Index)
        {
            pList->pClass->Terminate();
            Result = TRUE;
            break;
        }
    }

    return Result;
}

/*================================================================================
GetConnCount : Terminates all active connections
================================================================================*/
boolean CServerStub::TerminateAll()
{
    boolean Result = FALSE;
    uint32  Index;
    for (Index=ConnCount; Index > 0; Index--)
    {
        Result = TerminateConn(Index);
    }

    return Result;
}

/*================================================================================
TestByPass : Test Bypass. Returns first serial connection.
================================================================================*/
CFamilyStub* CServerStub::TestByPass()
{
    CONNECTION_LIST::iterator pList;
    uint32 i = 0;

    /* Print the list of connections */
    for (pList = ConnList.begin(), i=1; pList != ConnList.end(); pList++, i++)
    {
        if (pList->pClass->GetStackType(pList->pSourceID) == SERIAL)
        {
            return pList->pClass;
        }
    }

    return NULL;
}

/* End of File */

