/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CCommSvr.cpp   :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CCommSvr.h"

#if defined(STACK_SERIAL)
#include "CSerialCommSvr.h"
#endif

#if defined(STACK_ETHERNET)
#include "CEthCommSvr.h"
#endif

#if defined(STACK_USB)
#include "CUSBCommSvr.h"
#endif

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
*********************************************************************************/
CCommSvr::CCommSvr(boolean (*RecvMsg)(char *pSemName, char *pSourceID, uint8 *pBuf, 
                   uint32 MsgSize, void *pNewConnection),                   
                   boolean (*TerminateClient)(char *pSemName, char *pSourceID, uint16 Port),
                   char *pAppName)
{
    pRecvMsg = RecvMsg;
    pTerminate =  TerminateClient;
    pSemName = pAppName;
}

CCommSvr::~CCommSvr(void)
{
    pRecvMsg = NULL;
    pTerminate =  NULL;
    pSemName = NULL;
}

/*================================================================================
CCommSvr GetSourceID: Returns the SourceID of the object
================================================================================*/
char *CCommSvr::GetSourceID(ConnArgs *pConnArgs)
{
    char *pSourceID = NULL;
    if (pConnArgs)
    {
        if (pConnArgs->pCommSvr)
        {
#if defined(STACK_ETHERNET)
            if (pConnArgs->StackType == ETHERNET)
            {
                pSourceID = ((CEthCommSvr*)(pConnArgs->pCommSvr))->GetSourceID();
            }
#endif

#if defined(STACK_SERIAL)
            if (pConnArgs->StackType == SERIAL)
            {
                pSourceID = ((CSerialCommSvr*)(pConnArgs->pCommSvr))->GetSourceID();
            }
#endif

#if defined(STACK_USB)
            if (pConnArgs->StackType == USB)
            {
                pSourceID = ((CUSBCommSvr*)(pConnArgs->pCommSvr))->GetSourceID();
            }
#endif   
        }
    }
    return(pSourceID);
}

/*================================================================================
CCommSvr SetConfig: Updates the addressing information of the object
================================================================================*/
void CCommSvr::SetConfig(ConnArgs *pConnArgs, void *pAddr)
{
    if (pConnArgs)
    {
        if (pConnArgs->pCommSvr)
        {
#if defined(STACK_ETHERNET)
            if (pConnArgs->StackType == ETHERNET)
            {
                ((CEthCommSvr*)(pConnArgs->pCommSvr))->SetConfig(pAddr);
            }
#endif

#if defined(STACK_SERIAL)
            if (pConnArgs->StackType == SERIAL)
            {
                ((CSerialCommSvr*)(pConnArgs->pCommSvr))->SetConfig(pAddr);
            }
#endif

#if defined(STACK_USB)
            if (pConnArgs->StackType == USB)
            {
                ((CUSBCommSvr*)(pConnArgs->pCommSvr))->SetConfig(pAddr);
            }
#endif   
        }
    }
}


/*================================================================================
CCommSvr GetConfig: Returns the configuration information
================================================================================*/
void *CCommSvr::GetConfig(ConnArgs *pConnArgs)
{
    void *pAddr = NULL;
    
    if (pConnArgs)
    {
        if (pConnArgs->pCommSvr)
        {
#if defined(STACK_ETHERNET)
            if (pConnArgs->StackType == ETHERNET)
            {
                pAddr = ((CEthCommSvr*)(pConnArgs->pCommSvr))->GetConfig();
            }
#endif

#if defined(STACK_SERIAL)
            if (pConnArgs->StackType == SERIAL)
            {
                pAddr = ((CSerialCommSvr*)(pConnArgs->pCommSvr))->GetConfig();
            }
#endif

#if defined(STACK_USB)
            if (pConnArgs->StackType == USB)
            {
                pAddr = ((CUSBCommSvr*)(pConnArgs->pCommSvr))->GetConfig();
            }
#endif    
        }
    }
    return(pAddr);
}

/*================================================================================
CCommSvr GetPortNum: Returns the Port used by the object
================================================================================*/
uint16 CCommSvr::GetPortNum(ConnArgs *pConnArgs)
{
    uint16 Port = 0;
    
    if (pConnArgs)
    {        
        if (pConnArgs->pCommSvr)
        {
#if defined(STACK_ETHERNET)
            if (pConnArgs->StackType == ETHERNET)
            {
                Port = ((CEthCommSvr*)(pConnArgs->pCommSvr))->GetPortNum();
            }
#endif

#if defined(STACK_SERIAL)
            if (pConnArgs->StackType == SERIAL)
            {
                Port = ((CSerialCommSvr*)(pConnArgs->pCommSvr))->GetPortNum();
            }
#endif

#if defined(STACK_USB)
            if (pConnArgs->StackType == USB)
            {
                Port = ((CUSBCommSvr*)(pConnArgs->pCommSvr))->GetPortNum();
            }
#endif    
        }
    }

    return(Port);
}

/*================================================================================
CCommSvr SetPortNum: Changes the current configuration to use the WACPPort. 
    Ethernet - This also requires a new instance of the CEthCommSvr to create
    the WACP data socket.

    Serial - This just sets the port to the WACPPort in ConnArgs.
================================================================================*/
boolean CCommSvr::SetPortNum(ConnArgs *pConnArgs)
{
    boolean Result = FALSE;

#if defined(STACK_ETHERNET)
    if (pConnArgs->StackType == ETHERNET)
    {
        if (pConnArgs->pCommSvr)
        {
            /* Update the port number of the serial comm server */
            Result = ((CEthCommSvr*)(pConnArgs->pCommSvr))->SetPortNum(pConnArgs->WACPPort);
        }
    }
#endif

#if defined(STACK_SERIAL)
    if (pConnArgs->StackType == SERIAL)
    {
        if (pConnArgs->pCommSvr)
        {
            /* Update the port number of the serial comm server */
            Result = ((CSerialCommSvr*)(pConnArgs->pCommSvr))->SetPortNum(pConnArgs->WACPPort);
        }
    }
#endif

#if defined(STACK_USB)
    if (pConnArgs->StackType == USB)
    {
        if (pConnArgs->pCommSvr)
        {
            /* Update the port number of the serial comm server */
            Result = ((CUSBCommSvr*)(pConnArgs->pCommSvr))->SetPortNum(pConnArgs->WACPPort);
        }
    }
#endif    
    return (Result);
}

/*================================================================================
CCommSvr Connect: Creates a socket that connects to the passed in SourceID and Port.
================================================================================*/
boolean CCommSvr::Connect(ConnArgs *pConnArgs)
{
    boolean Result = FALSE;

    if (pConnArgs)
    {
#if defined(STACK_ETHERNET)
        /* Initialize the appropriate CommSvr instance */
        if (pConnArgs->StackType == ETHERNET)
        {
            /* Allocate and initialize a new instance of the CEthCommSvr object */
            pConnArgs->pCommSvr = new CEthCommSvr(pRecvMsg, pTerminate, pSemName);
            if (pConnArgs->pCommSvr)
            {
                /* Update the configuration including SourceID for the CommSvr */
                ((EthAddr*)(pConnArgs->pConfig))->pSourceID = pConnArgs->SourceID;
                ((CEthCommSvr*)(pConnArgs->pCommSvr))->SetConfig(pConnArgs->pConfig);

                Result = TRUE;

                if (pConnArgs->bSvrFlag == FALSE)
                {
                    /* Open up the communications channel */
                    Result = ((CEthCommSvr*)(pConnArgs->pCommSvr))->Connect((uint8*)pConnArgs->SourceID, pConnArgs->RNDZPort);

                    /* Update the Connected status */
                    pConnArgs->bConnected = Result;
                }

                if (Result)
                {
                    /* Start up the receiving thread for the CommSvr. */
                    Result = ClientThread.StartThread(((CEthCommSvr*)(pConnArgs->pCommSvr))->StartClient, pConnArgs->pCommSvr, pConnArgs->ThreadPriority);
                }
            }
        }
#endif

#if defined(STACK_SERIAL)
        if (pConnArgs->StackType == SERIAL)
        {
            /* Allocate and initialize a new instance of the CSerialCommSvr object */
            pConnArgs->pCommSvr = new CSerialCommSvr(pRecvMsg, pTerminate, pSemName);
            if (pConnArgs->pCommSvr)
            {            
                /* Update the configuration including SourceID for the CommSvr */
                ((SerialAddr*)(pConnArgs->pConfig))->pSourceID = pConnArgs->SourceID;
                ((CSerialCommSvr*)(pConnArgs->pCommSvr))->SetConfig(pConnArgs->pConfig);

                /* Open up the communications channel */
                Result = ((CSerialCommSvr*)(pConnArgs->pCommSvr))->Connect((uint8*)pConnArgs->SourceID, pConnArgs->RNDZPort);

                /* Update the Connected status */
                pConnArgs->bConnected = Result;

                if (Result)
                {
                    /* Start up the receiving thread for the CommSvr. */
                    Result = ClientThread.StartThread(((CSerialCommSvr*)(pConnArgs->pCommSvr))->StartClient, pConnArgs->pCommSvr, pConnArgs->ThreadPriority);
                    SYS_SLEEP(WAIT_COUNT);
                }
            }
        }
#endif

#if defined(STACK_USB)
        if (pConnArgs->StackType == USB)
        {
            /* Allocate and initialize a new instance of the CSerialCommSvr object */
            pConnArgs->pCommSvr = new CUSBCommSvr(pRecvMsg, pTerminate, pSemName);
            if (pConnArgs->pCommSvr)
            {
                /* Update the configuration including SourceID for the CommSvr */
                ((CUSBCommSvr*)(pConnArgs->pCommSvr))->SetConfig(pConnArgs->pConfig);

                /* Open up the communications channel */
                Result = ((CUSBCommSvr*)(pConnArgs->pCommSvr))->Connect((uint8*)pConnArgs->SourceID, pConnArgs->RNDZPort);
                
                /* Update the Connected status */
                pConnArgs->bConnected = Result;

                if (Result)
                {
                    /* Start up the receiving thread for the CommSvr. */
                    Result = ClientThread.StartThread(((CUSBCommSvr*)(pConnArgs->pCommSvr))->StartClient, pConnArgs->pCommSvr, pConnArgs->ThreadPriority);
                }
            }
        }
#endif    
    }
    return Result;
}

/*================================================================================
CCommSvr Disconnect: terminates the current connection pointed to pConnArgs.
================================================================================*/
boolean CCommSvr::Disconnect(ConnArgs *pConnArgs)
{
    boolean Result = FALSE;

    if (pConnArgs)
    {
#if defined(STACK_ETHERNET)
        if (pConnArgs->StackType == ETHERNET)
        {
            /* Allow remaining messages to be sent */
            SYS_SLEEP(WAIT_COUNT);
        
            if (pConnArgs->pCommSvr)
            {
                /* Clean up the ethernet commsvr instance */
                delete((CEthCommSvr*)pConnArgs->pCommSvr);
            }
        }
#endif

#if defined(STACK_SERIAL)
        if (pConnArgs->StackType == SERIAL)
        {
            if (pConnArgs->pCommSvr)
            {
                /* Clean up the serial commsvr instance */
                delete((CSerialCommSvr*)pConnArgs->pCommSvr);
            }

            /* Allow the receiving threads a chance to terminate */
            SYS_SLEEP(WAIT_COUNT);
        }
#endif

#if defined(STACK_USB)
        if (pConnArgs->StackType == USB)
        {
            /* Clean up the serial commsvr instance */
            delete((CUSBCommSvr*)pConnArgs->pCommSvr);
            
            /* Allow the receiving threads a chance to terminate */
            SYS_SLEEP(WAIT_COUNT);

        }
#endif
        pConnArgs->pCommSvr = NULL;
        
        Result = TRUE;
    }

    return Result;
}

/*================================================================================
CCommSvr SendMsg: Uses the current socket descriptor to send a message.
================================================================================*/
boolean CCommSvr::SendMsg(ConnArgs *pConnArgs, uint8 *pBuf, uint32 pBufSize)
{
    boolean Result = FALSE;

    if (pConnArgs)
    {
#if defined(STACK_ETHERNET)
        if (pConnArgs->StackType == ETHERNET)
        {
            if (pConnArgs->pCommSvr)
            {
                /* Send the message */
                Result = ((CEthCommSvr*)(pConnArgs->pCommSvr))->SendMsg(pBuf, pBufSize);
            }
        }
#endif

#if defined(STACK_SERIAL)
        if (pConnArgs->StackType == SERIAL)
        {
            if (pConnArgs->pCommSvr)
            {
                /* Send the message */
                Result = ((CSerialCommSvr*)(pConnArgs->pCommSvr))->SendMsg(pBuf, pBufSize);
            }
        }
#endif

#if defined(STACK_USB)
        if (pConnArgs->StackType == USB)
        {
            if (pConnArgs->pCommSvr)
            {
                /* Send the message */
                Result = ((CUSBCommSvr*)(pConnArgs->pCommSvr))->SendMsg(pBuf, pBufSize);
            }
        }
#endif /* #if defined(STACK_USB) */
    }
    return Result;
}   

/*================================================================================
CCommSvr StartServer: Starts up a server to listen for connections.
================================================================================*/
boolean CCommSvr::StartServer(ConnArgs **pConnList, uint8 NumConnections)
{
    boolean Result = FALSE;
    uint16 i;
    CWACPEvent  *pExitEvent;
    char        Buffer[EVENT_BUF_SIZE];

    memset(Buffer, 0, EVENT_BUF_SIZE);
    
    /* Raise an event to signify the end of the client session */
    sprintf(Buffer,END_SERVER,SYS_GETPID());
    pExitEvent = new CWACPEvent(Buffer, TRUE);
    pExitEvent->Reset();
    delete(pExitEvent);

    for (i=0; i < NumConnections; i++)
    {
#if defined(STACK_ETHERNET)
        if (pConnList[i]->StackType == ETHERNET)
        {
            CEthCommSvr *pRNDZServer;

            /* Create a new instance of the CEthCommSvr... This instance is destroyed in the 
             * Server thread that is launched below
             */
            pRNDZServer = new CEthCommSvr(pRecvMsg, pTerminate, pSemName);
            
            /* Update the Port number to use for socket connections */
            pRNDZServer->SetPortNum(pConnList[i]->RNDZPort);

            /* Update the Port used for auto-discovery */
            pRNDZServer->SetAutoPort(pConnList[i]->RNDZPort);
            
            /* Set the Protocol: TCP or UDP */
            pRNDZServer->SetProtocol(((EthAddr*)(pConnList[i]->pConfig))->Protocol);

            /* Launch the listener threads */
            Result = RecvThread.StartThread(pRNDZServer->StartServer, (void*)pRNDZServer, pConnList[i]->ThreadPriority);
            
            /* Exit the for loop if we are unable to start up a listener thread */
            if (!Result)
                break;
        }
#endif /* #if defined(STACK_ETHERNET) */

#if defined(STACK_SERIAL)
        if (pConnList[i]->StackType == SERIAL)
        {
            CSerialCommSvr *pServer;

            /* Create a new instance of the CEthCommSvr... This instance is destroyed in the 
             * Server thread that is launched below
             */
            pServer = new CSerialCommSvr(pRecvMsg, pTerminate, pSemName);

            /* Set the Port to RNDZ */
            pServer->SetPortNum(pConnList[i]->RNDZPort);

            /* Set the SourceID. The SourceID is the friendly name of the com port i.e. "COM1:" */
            pServer->SetSourceID(pConnList[i]->SourceID);

            /* Set the BaudRate */
            pServer->SetBaudRate(((SerialAddr*)(pConnList[i]->pConfig))->BaudRate);

            /* Set the ByPass to allow the app to control the serial connection */
            pServer->SetByPass(pConnList[i]->bByPass);

            /* Launch the listener thread */
            Result = RecvThread.StartThread(pServer->StartServer, (void*)pServer, pConnList[i]->ThreadPriority);
        }
#endif /* #if defined(STACK_SERIAL) */

#if defined(STACK_USB)
        if (pConnList[i]->StackType == USB)
        {
            CUSBCommSvr *pServer;

            /* Create a new instance of the CUSBCommSvr... This instance is destroyed in the 
             * Server thread that is launched below
             */
            pServer = new CUSBCommSvr(pRecvMsg, pTerminate, pSemName);

            /* Set the Port to RNDZ */
            pServer->SetPortNum(pConnList[i]->RNDZPort);

            /* Launch the listener thread */
            Result = RecvThread.StartThread(pServer->StartServer, (void*)pServer, pConnList[i]->ThreadPriority);
        }
#endif /* #if defined(STACK_USB) */
    
    } /* for (i=0; i < NumConnections; i++) */

    return Result;
} 


/*================================================================================
CCommSvr EndServer: Terminates all server listeners.
================================================================================*/
boolean CCommSvr::EndServer(void)
{
    boolean Result = TRUE;
    CWACPEvent  *pExitEvent;
    char        Buffer[EVENT_BUF_SIZE];

    memset(Buffer, 0, EVENT_BUF_SIZE);
    
    /* Raise an event to signify the end of the client session */
    sprintf(Buffer,END_SERVER,SYS_GETPID());
    pExitEvent = new CWACPEvent(Buffer, TRUE);
    pExitEvent->Raise();

    SYS_SLEEP(WAIT_COUNT);

    /* Clean up the Event object */
    delete(pExitEvent);
    return (Result);
}

/* End of File */





