/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CCommSvr.h     :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined (CCOMMSVR_H)
    #define   CCOMMSVR_H

#include "Common.h"
#include "StdVarTyp.h"
#include "CWACPThread.h"

#define CTL_W 0x17
#define CTL_A 0x01
#define CTL_C 0x03
#define CTL_L 0x0C

#define ACK_SIZE    (1)

#define ETH_TIMEOUT 1  /* In seconds */
#define UDP_MUTEX_NAME "UDP_MUTEX_%d"
#define END_SERVER "KILL_%d"

#define PREAMBLE_SIZE (3)

#define MSG_HEADER (sizeof(uint8) * PREAMBLE_SIZE + sizeof(uint32))

typedef struct SessionArgs
{
    STACK_TYPE  StackType;
    uint16      Port;
    char        *pSourceID;
    void        *pAddr;
    boolean     bByPass;
}SesssionArgs;

uint8 *GeneratePacket(uint32 *OutSize, int16 Port, int16 SequenceNum, int32 UUID, uint8 *pBuf, uint32 Size);

class CCommSvr
{
    private:
        SetRecvBufPtr  pRecvMsg;
        TerminatePtr   pTerminate;
        CWACPThread    ClientThread;
        CWACPThread    RecvThread;

        char           *pSemName;

    public:
        CCommSvr(boolean (*RecvMsg)(char *pSemName, char *pSourceID, uint8 *pBuf, 
                           uint32 MsgSize, void *pNewConnection),                   
                           boolean (*TerminateClient)(char *pSemName, char *pSourceID, uint16 Port),
                           char *pAppName);
        CCommSvr();
        ~CCommSvr();

        /* Returns the SourceID of the CCommSvr. The SourceID is the unique
         * identifier for the communications session. In Ethernet the SourceID 
         * would hold the destination IP Address of the connection peer. In 
         * RS232 this would hold the Name of the com Port through which */
        char *GetSourceID(ConnArgs *pConnArgs);

        /* Sets up the configuration of the stack based on the data received 
         * via pAddr.
         */
        void SetConfig(ConnArgs *pConnArgs, void *pAddr);

        /* Gets the configuration of the stack */
        void *GetConfig(ConnArgs *pConnArgs);

        /* Returns the Port which represents either a RNDZ or WACP session */
        uint16 GetPortNum(ConnArgs *pConnArgs);

        /* Sets the Port which represents either a RNDZ or WACP session */
        boolean SetPortNum(ConnArgs *pConnArgs);

        /* Sends a binary Buffer */
        boolean SendMsg(ConnArgs *pConnArgs, uint8 *pBuf, uint32 MsgSize);
        
        /* Creates a connection */
        boolean Connect(ConnArgs *pConnArgs);

        /* Disconnects a connection */
        boolean Disconnect(ConnArgs *pConnArgs);

        /* Starts the listener thread to wait and accept new connections from 
         * client 
         */
        boolean StartServer(ConnArgs **pConnList, uint8 NumConnections);
        
        /* Terminates all listener threads.*/
        boolean EndServer(void);

};

#endif /* #if !defined (CCOMMSVR_H) */


/* End of File */





