/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CConObjSvr.h   :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CCONOBJSVR_H)
    #define  CCONOBJSVR_H

#include "CObjSvr.h"
#include "CMsgSvr.h"
#include "CWACPMutex.h"
#include "CWACPSemaphore.h"
#include "CWACPThread.h"
#include "Common.h"
#include "CTraceSvr.h"
#include "CLinkTable.h"

    typedef enum KEEPALIVESTATUS{
        KEEP_ALIVE_RUNNING = 0,
        KEEP_ALIVE_STOPPED,
        KEEP_ALIVE_TIMEOUT,
        KEEP_ALIVE_END
    }KEEP_ALIVE_STATUS;

    typedef  enum CONNCEVNTtypSPECIES{
        CONN_CEVNT_SpNONE = 0,
        CONN_CEVNT_SpCONNECTED = 1,     /* Must follow CMD_SpSHUTDOWN */
        CONN_CEVNT_SpFAILED,
        CONN_CEVNT_SpMAX
    }CONN_CEVNT_SPECIES;

    typedef  enum CONNCRSTtypSPECIES {
        CONN_CRST_SpKEEPALIVE = 0,
        CONN_CRST_SpMAX
    }CONN_CRST_SPECIES;

    typedef  enum CONNCRSPtypSPECIES {
        CONN_CRSP_SpKEEPALIVE = 0,
        CONN_CRSP_SpMAX
    }CONN_CRSP_SPECIES;

    typedef  enum CONNCMDtypSPECIES {
        CONN_CMD_SpSHUTDOWN = 0,
        CONN_CMD_SpMAX
    }CONN_CMD_SPECIES;

    typedef  enum CONNCFGtypSPECIES {
        CONN_CFG_SpKEEPALIVEON = 0,
        CONN_CFG_SpKEEPALIVEOFF,
        CONN_CFG_SpKEEPALIVECHECK,
        CONN_CFG_SpMAX
    }CONN_CFG_SPECIES;


typedef struct ConnObj
{
    uint8               TxOptions;
    KEEP_ALIVE_STATUS   KeepAliveStatus;
    time_t              TimeoutInterval;
    time_t              KeepAliveInterval;
    time_t              KeepAliveTimer;
    time_t              TimeoutTimer;
    char*               pSourceID;
}ConnObj;

class CConObjSvr : public CObjSvr
{
/*================================================================================
CConObjSvr Members: Public, Private, and Protected
================================================================================*/
public:
    CMsgSvr         MsgSvr;          
    CTraceSvr*      pTrace;
    CWACPSemaphore* pTimeOutSem;
    CWACPThread     ConnObjThread;
    CWACPMutex      ExitMutex;
private:
    /* declaration for callback function */
    ConnFuncPtr         ConnHandler;
    SendPacketFuncPtr   pSendPacket;
    MsgFuncPtr          pMsgFunc;
    void*               pDefaultRtr;

    /* Reference to CLinkTable instance */
    CLinkTable*         pCLinkTable;

    /* Semaphore Name */
    char*               pSemName;

    /* Mutex lock */
    CWACPMutex          ConnMutex;
    volatile boolean    ThreadStatus;

public:
    CConObjSvr();
    ~CConObjSvr();

    void Init(boolean (*SendPacket)(char*, char*, uint8*, uint32),
                          boolean (*MsgFunc)(char*, uint16, void*, uint16),
                          CTraceSvr* pTraceSvr,
                          LinkStruct* pLinkStruct);

    /*================================================================================
    CConObjSvr Set Connection Transmission Options
    ================================================================================*/
    boolean SetTxOptions(char* pSourceID, uint8 Options, boolean bSyncFlag);
    boolean GetTxOptions(char* pSourceID);
    boolean SetTimeoutInterval(char* pSourceID, time_t Count);
    time_t GetTimeoutInterval(char* pSourceID);
    boolean SetKeepAliveInterval(char* pSourceID, time_t Count);
    time_t GetKeepAliveInterval(char* pSourceID);

    /*================================================================================
    CConObjSvr connection operations
    ================================================================================*/
    boolean AddConnection(char* pSourceID);
    boolean RemoveConnection(char* pSourceID);
    ConnObj* GetConnObj(char* pSourceID);

    boolean EvaluateKeepAlive(char* pSourceID);    //Called once per DataThread evaluation
    boolean StartKeepAlive(char* pSourceID);       //Stops Keepalives and initializes the state
    boolean StopKeepAlive(char* pSourceID);        //Stops Keepalives and initializes the state

    boolean Connect(ConnArgs** pConnArgs, uint16 NumConnections);
    boolean Close(char* pSourceID);
    boolean UpdateStatus(char* pSourceID, boolean bStatus);
    boolean Shutdown(char* pSourceID);

    boolean TxMsg(char* pSourceID, CMsgSvr* pMsgSvr, uint8 Genus, uint8 Species);
    /*================================================================================
    CConObjSvr Callback registration
    ================================================================================*/
    boolean RecvMsgHandler(char* pSourceID, uint8* pBuffer, void* pRouter );
    boolean SendMsgHandler(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                            uint8* pBuffer, uint32 Size, boolean EncryptionOn);

    boolean SetCallbacks(va_list vList);
    boolean SetConnFunc(uint32 (*RxConn)(char*, uint8, uint8*, void*), void* pDefaultRouter);

    static uint32 WACPAPI KeepAliveThread(void* pData);
};
#endif /* #if !defined(CCONOBJSVR_H) */

/*---------------------------------------------------*/
/* End of File */

