/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CDataObjSvr.cpp                                                   //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CDataObjSvr.h"

/*================================================================================
CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
Default constructor
    MODULE Start up: All available servers are instantiated here. i.e Command Servers,
    Request Servers, Response Servers, etc.
*********************************************************************************/
CDataObjSvr::CDataObjSvr()
{
    /* Initialize the pointers */
    pMessageHandler = NULL;
    Outbuf = NULL;
}
CDataObjSvr::~CDataObjSvr()
{
    SYS_FREE(Outbuf);
    Outbuf = NULL;
}

void CDataObjSvr::Init(CTrapObjSvr *pTrapObjSvr,
                       CTraceSvr *pTraceSvr,
                       boolean (*SendPacket)(char*, char*, uint8*, uint32),
                       LinkStruct *pLinkStruct)
{
    MsgSvrIn.Init(FmWACP, GnDATA, pLinkStruct);
    MsgSvrOut.Init(FmWACP, GnDATA, pLinkStruct);
    pTrapServer = pTrapObjSvr;
    pTraceServer = pTraceSvr;
    pSendMsg = SendPacket;
    pSemName = pLinkStruct->pSemName;
    pLnkStruct = pLinkStruct;
}
/*================================================================================
CDataObjSvr TxMsg : Helper routine that transmits the contents of a CMsgSvr object
================================================================================*/
boolean CDataObjSvr::TxMsg(char *pSourceID, CMsgSvr *pMsgSvr, uint16 Family, 
                           uint8 Genus, uint8 Species, uint8* Payload, 
                           uint32 Size, boolean bEncryptionOn)
{
    uint32 OutBufSz;
    boolean Result = FALSE;

    /* Set the Family */
    pMsgSvr->set_Family(Family);

    /* Set the Genus */
    pMsgSvr->set_Genus(Genus);

    /* Set the Species */
    pMsgSvr->set_Species(Species);

#if defined(ENCRYPTION_ON)
    /* Set the Encryption status */
    pMsgSvr->SetEncryption(pSourceID, bEncryptionOn);
#else
    TOUCH(bEncryptionOn);
#endif

    /* Set the passed in Payload into the dynamic Data member variable */
    pMsgSvr->SetData(Payload, Size);

    /* Get the Size of the serialized byte Stream */
    OutBufSz = pMsgSvr->size_all();

    /* (Re)Allocate space for the serialized Buffer */
    Outbuf = (uint8 *)SYS_REALLOC(Outbuf, OutBufSz);
    
    /* Serialize the message */
    if(Outbuf != NULL)
    {
        /* Serialize the CErrSvr object */
        pMsgSvr->serialize_all(Outbuf);

        /* Send the message */
        Result = pSendMsg(pSemName, pSourceID, Outbuf, OutBufSz);
    }

    return(Result);
}
/*================================================================================
MODULE SendMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
    the messages with the appropriate object.
================================================================================*/
boolean CDataObjSvr::SendMsgHandler(char *pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                uint8 *pBuffer, uint32 Size, boolean bEncryptionOn)
{
    boolean RetVal = FALSE;

#if defined(TRACE_SERVER_ON_STATUS)
    pTraceServer->tprintf(TRACE_Type_SEND, "CDataObjSvr", "SendMsgHandler",
                          Family, Genus, Species, "");
#endif

    if(Genus != GnDATA)
    {
        RetVal = TxMsg(pSourceID, &MsgSvrOut, Family, Genus, Species, pBuffer, Size, bEncryptionOn);
    }
    else
    {
        RetVal = pSendMsg(pSemName, pSourceID, pBuffer, Size);
    }

    return (RetVal);
}
/*================================================================================
MODULE RecvMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
    the messages with the appropriate object.
================================================================================*/
boolean CDataObjSvr::RecvMsgHandler(char *pSourceID, uint8 *pBuffer, void *pWACPApp)
{
    uint16  Family;
    uint8   Genus;
    uint8   Species;
    uint32  Result = TRUE;
    uint32  Size;
    uint32  ClassID;

    /* Extract the Family */
    Family = GetFamily(pBuffer);

    /* Extract the Genus */
    Genus = GetGenus(pBuffer);

    /* Extract the Species */
    Species = GetSpecies(pBuffer);

    ClassID = MakeClassID(Family, Genus, Species);

    if(Genus != GnDATA)
    {
        /* Set the MsgSvr object to the incoming Genus tag for deserialization */
        MsgSvrIn.set_Family(Family);
        MsgSvrIn.set_Genus(Genus);

#if defined(ENCRYPTION_ON)
        /* Update the CMsgSvr with the proper SourceID. The Encryption flag is ignored
         * for deserialization
         */
        MsgSvrIn.SetEncryption(pSourceID, TRUE);
#endif
        /* Deserialize the data and check for errors */
        Result = MsgSvrIn.deserialize_all(pBuffer);
        if(!Result)
        {
#if defined(TRACE_SERVER_ON)
            pTraceServer->tprintf(TRACE_Type_ERROR, "CDataObjSvr", "RecvMsgHandler",
                                  Family, Genus, Species, "MsgSvr deserialization failed");
#endif

            /* Call the trapserver with the error code stored in Result */
            pTrapServer->SendError(pSourceID, ClassID, CERR_CRC_MISMATCH, pWACPApp);
        }
    }

    if(Result)
    {
        /*Genus Router*/
        switch (Genus)
        {
            case GnDATA:
            {
                Result = pMessageHandler(pSourceID, ClassID, pBuffer, pWACPApp);
                break;
            }
            default:
            {
                Result = pMessageHandler(pSourceID, ClassID, MsgSvrIn.GetData(&Size), pWACPApp);
                break;
            }
        }/* switch (Genus)*/
    }

    return (Result ? TRUE : FALSE);
}
/* Function for setting the callback function for all Messages */
boolean CDataObjSvr::SetMessageFunc(uint32 (*RxMessage)(char*, uint32, uint8*, void*))
{
    boolean Result = FALSE;
    if(RxMessage != NULL)
    {
        pMessageHandler = RxMessage;
        Result = TRUE;
    }
    return(Result);
}
/* Function for setting the callback function for Cmd, Rqst, and Resp Messages */
boolean CDataObjSvr::SetCallbacks(va_list vList)
{
    int ResultCount = 0;

    uint32 (*RxGeneric)(char*, uint32, uint8*, void*);
    uint32 *pRxGeneric = (uint32*)&RxGeneric;

    /* Register Message Callback */
    *pRxGeneric = va_arg(vList, uint32);
    ResultCount += SetMessageFunc(RxGeneric);
   
    return(ResultCount == 1);
}
/* End of File */
