/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CError.cpp                                                        //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CError.h"

const char *CError::TypStackErrorTbl[CERR_MAX]={
    "None",
    "Unimplemented",
    "Invalid Family",
    "Invalid Genus",
    "Invalid Species",
    "Object ID Mismatch",
    "CRC Mismatch",
    "Unable to allocate memory",
    "Unable to find Link Table Entry",
    "Duplicate entry",
    "Invalid Port"
};

/* Add descriptions here for AppInternal Errors */
const char *CError::TypAppErrorTbl[CERR_APP_MAX]={
    "None",
    "Command Failed"
};

const char *CError::TypSpeciesTbl[CERR_SpMAX]={
    "None",
    "WACPStack_Error",
    "AppInternal_Error"
};

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
*********************************************************************************/
CError::CError(uint8 species) : DtData(MakeClassID(FmARCH, GnERROR, species))
{
    memset(m_structData.Description, 0, sizeof(m_structData.Description));
    Data = NULL;
    DataSize = 0;

    /* Initialize DataBitField */
    SetDataBitField(0);
}

/*********************************************************************************
  Default destructor
*********************************************************************************/
CError::~CError()
{
    Reset(TRUE);
}
void CError::Reset(boolean bResetAll)
{
    TOUCH(bResetAll);

    /* Reset dirty and valid flags */
    SetDataBitField(0);

    /* Reset static members */
    memset(&m_structData, 0, sizeof(m_structData));

    /* Free dynamic members */
    SYS_FREE(Data);
    Data = NULL;
}
/*---------------------------------
 Static Members
---------------------------------*/
/*********************************************************************************
*********************************************************************************
 Get function for returning the last error
*********************************************************************************/
uint16 CError::GetLastError(ansichar* pBuf)
{
    pBuf = m_structData.Description;

    TOUCH(pBuf);
    
    return (m_structData.ErrorCode);
}

/*********************************************************************************
 Set function for data member ErrorCode
*********************************************************************************/
void CError::SetErrorCode(uint16 ErrorCode)
{
    setvalid(TRUE);

    m_structData.ErrorCode = ErrorCode;

    /* Update the Description if the species is not SpCUSTOM */
    switch (species())
    {
        case CERR_SpWACPSTACK:
        {
            if (ErrorCode < CERR_MAX)
            {
                SetDescription((ansichar*)CERR_STACK_STRING);
                break;
            }
        }
        case CERR_SpAPPINTERNAL:
        {
            if (ErrorCode < CERR_APP_MAX)
            {
                SetDescription((ansichar*)CERR_APP_STRING);
                break;
            }
        }
            
            break;
        default:
            break;

    }
}

/*********************************************************************************
 Get function for data member ErrorCode
*********************************************************************************/
uint16 CError::GetErrorCode()
{
    return (m_structData.ErrorCode);
}

/*********************************************************************************
 Set function for data member ErrorCode
*********************************************************************************/
void CError::SetClassID(uint32 ClassID)
{
    setvalid(TRUE);

    m_structData.ClassID = ClassID;
}

/*********************************************************************************
 Get function for data member ErrorCode
*********************************************************************************/
uint32 CError::GetClassID()
{
    return (m_structData.ClassID);
}

/*********************************************************************************
 Set function for data member Description
*********************************************************************************/
void CError::SetDescription(ansichar* pDesc)
{   
    size_t Size;

    setvalid(TRUE);

    if (pDesc != NULL)
    {
        Size = strlen(pDesc);
        
        /* Truncate the description to the max Size of the array */
        if (Size > sizeof(m_structData.Description))
        {
            Size = sizeof(m_structData.Description);
        }

        memcpy(m_structData.Description, pDesc, Size);
    }
}

/*********************************************************************************
 Get function for data member Description
*********************************************************************************/
ansichar* CError::GetDescription()
{
    return (m_structData.Description);
}

/*---------------------------------
 Dynamic Members
---------------------------------*/
/*********************************************************************************
 Set function for dynamic data member Data
*********************************************************************************/
void CError::SetData(uint8* NewData, uint32 Size)
{
    /*Allocate memory for the new Buffer*/
    if (NewData != NULL)
    {
        Data = (uint8*)SYS_REALLOC(Data, Size);

        if (Data != NULL)
        {
            /*Make a copy of the data*/
            memcpy(Data, NewData, Size);
    
            /* Set the DataSize Var */
            DataSize = Size;

        }
        setvalid(TRUE);
    }
    else
    {
        SYS_FREE(Data);
        Data = NULL;
    }

}
/*********************************************************************************
 Get function for dynamic data member Data
*********************************************************************************/
uint8* CError::GetData(uint32 *Size)
{
    if (Size)
    {
        *Size = DataSize;
    }
    return (uint8*)Data;
}

/*================================================================================
    INHERITED / OVERIDDEN
================================================================================*/
/*********************************************************************************
Returns the sized state. 
*********************************************************************************/
boolean CError::is_variable_sized ( void )
{
    return TRUE;
}

/*********************************************************************************
Returns the Aggregate state. 
********************************************************************************/
boolean CError::is_Aggregate ( void )
{
    return FALSE; 
}

/*********************************************************************************
Returns the number of Bytes that would be used by the serialize_payload and
deserialize_payload methods.
*********************************************************************************/
uint32 CError::get_payload_size ( void )
{   
    uint32 Size = 0;
    Size = sizeof(uint16)                       /*Size of the struct*/
            + (uint16)CERR_STATIC_STRUCT_SIZE;  /*struct data Size*/

    /* Get the Size of each dynamic member */
    if (Data != NULL)
    {
        Size += sizeof(uint32) + DataSize;
    }
    else
    {
        Size += sizeof(uint32);
    }

    Size += sizeof(uint16);             /* CRC Bytes */

    return Size;            
}

/*********************************************************************************
Serialize the static data defined by m_structData.
*********************************************************************************/
void CError::serialize_static_data(uint8QManager * QMgr)
{
    /* Enqueue only the  members of the Struct.*/
    QMgr->put(m_structData.ClassID);
    QMgr->put(m_structData.ErrorCode);
    QMgr->enqueue_array((byte*)m_structData.Description, sizeof(m_structData.Description));
}

/*********************************************************************************
Deserialize the static data defined by m_structData.
*********************************************************************************/
void CError::deserialize_static_data(uint8QManager *QMgr)
{
    /* Dequeue only the  members of the Struct.*/
    QMgr->get(&m_structData.ClassID);
    QMgr->get(&m_structData.ErrorCode);
    QMgr->dequeue_array((byte*)&m_structData.Description, sizeof(m_structData.Description));
}

/*********************************************************************************
Used to place the data elements of the object into a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CError::serialize_payload ( uint8QManager * QMgr )
{
    uint32      CRCLen;

    /* Save the start of the byte stream for calculating the CRC */
    byte* TailPtr = (byte*)QMgr->GetTailPtr();



    /* Enqueue the Static Structure Size */
    QMgr->put((uint16)CERR_STATIC_STRUCT_SIZE);

    /* Enqueue only the static members of the Struct */
    serialize_static_data(QMgr);

    /* Enqueue variable sized members and there sizes */
    if (Data != NULL)
    {
        QMgr->put(DataSize);
        QMgr->enqueue_array((byte*)Data, DataSize);
    }
    else
    {
        QMgr->put((uint32)(0) );
    }

    /* 5 Enqueue the CRC */
    CRCLen = QMgr->get_num_elements() - sizeof(uint16);
    QMgr->put(uint16( CalculateCRC(TailPtr,CRCLen)));

    return TRUE; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CError::deserialize_payload ( uint8QManager * QMgr, uint32 Sz )
{
    //These data pieces will need to be dequeued
    uint16      Size; 
    uint32      CRCSize;
    uint16      CalcCRC;
    uint16      StoredCRC;
        
    /* Save the start of the byte stream for calculating the CRC */
    byte* HeadPtr = (byte*)QMgr->GetHeadPtr();

    /* Calculate the CRC on all but the 2 Bytes of the CRC */
    CRCSize = QMgr->get_num_elements() - sizeof(uint16);
    CalcCRC = CalculateCRC(HeadPtr, CRCSize);

    /* Dequeue the static member struct Size */
    QMgr->get(&Size);

    /* Dequeue only the static members of the Struct */
    if ( Size <= (uint16)CERR_STATIC_STRUCT_SIZE )
    {   
        deserialize_static_data(QMgr);
#if defined(DEBUG_PRINT)
        if ( Size < (uint16)CERR_STATIC_STRUCT_SIZE )
        {   
            SYS_PRINTF("\nThe source object is smaller than the destination. Some source data will be lost.\n\n");
        }
#endif
    }
    else
    { 
        uint8   *Buffer;

        Buffer = (uint8*)SYS_MALLOC(Sz); 
        if (Buffer != NULL)
        {
            deserialize_static_data(QMgr);
#if defined(DEBUG_PRINT)
            SYS_PRINTF("\nThe Source object is larger than the destination. Some destination data will not be intialized.\n\n");
#endif
            /* Dequeue the remaining extra Bytes of the static struct */
            QMgr->dequeue_array( Buffer, Size - CERR_STATIC_STRUCT_SIZE);

            /* TODO: Create a mechanism for the object to store this unknown data  */
            /*       to prevent data loss and to create a way to forward this data */
            /*       to a host/device that supports the correct version.           */

            SYS_FREE(Buffer);
        }   
    }
    
    /* Dequeue dynamic members */
    QMgr->get(&DataSize);
    
    if (DataSize > 0)
    {
        Data = (uint8*)SYS_REALLOC(Data, DataSize);

        if (Data != NULL)
        {
            QMgr->dequeue_array((byte*)Data, DataSize); 
        }
    }
    else
    {
        SYS_FREE(Data);
        Data = NULL;
    }

    /* Dequeue the CRC */
    QMgr->get(&StoredCRC);
    if ( CalcCRC != StoredCRC)
    {
#if defined(DEBUG_PRINT)
        SYS_PRINTF("CRC Mismatch Error: Calculated=0x%d, Stored=0x%d ", CalcCRC, StoredCRC);
        #endif
return FALSE;
    }

    return TRUE; 
}


#if ((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
/*********************************************************************************
Helper Function for meta data types (not used for low level objects).
*********************************************************************************/
uint32 CError::serialize_to_XMLbuf(char *pBuf)
{
    char Buffer[XML_BUFSIZE];
    char *Output = Buffer;
    uint32 NewSize = 0;
    uint32 FinalSize = 0;
    uint32 ZeroSize = 0;
    uint32 *pSizeCount = &ZeroSize;

    /* If pBuf is NULL, then we don't write anything to it, and just 
       return the potential Size */
    if(pBuf != NULL)
    {
        Output = pBuf;
        pSizeCount = &NewSize;
    }
    NewSize += sprintf(Output,
    //XML formatted Output to file
    "<CERR_OBJECT>\n"
        "<FAMILY>%s</FAMILY>\n"
        "<GENUS>%s</GENUS>\n" 
        "<SPECIES>%s</SPECIES>\n",
        
        //---------------------------------------------------------------
        /*Data for the related types above.*/
        FAMILY_STRING,
        GENUS_STRING, 
        CERR_SPECIES_STRING);

        //---------------------------------------------------------------

        /* -- Pointer to Data Buffer --*/
    
    FinalSize += NewSize;
    Output += *pSizeCount;

    NewSize += sprintf(Output,"<DATA>");
    FinalSize += NewSize;
    Output += *pSizeCount;

    if (Data != NULL)
    {
        NewSize += sprintf(Output, "%d", Data[0]); 
        FinalSize += NewSize;
        Output += *pSizeCount;

        for(uint32 index = 1; index < DataSize; index++)
        {
            NewSize += sprintf(Output, ",%d", Data [index]); 
            FinalSize += NewSize;
            Output += *pSizeCount;
        }
    }

    NewSize += sprintf(Output,"</DATA>\n");
    FinalSize += NewSize;
    Output += *pSizeCount;

    //---------------------------------------------------------------

    /*Write the closing object tag*/
    NewSize += sprintf(Output,"</CCMD_OBJECT>\n");
    FinalSize += NewSize;
    Output += *pSizeCount;

    return FinalSize+1;
}
#endif

#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))

/*********************************************************************************
  The XML Parser makes a callback to this function whenever it encounters a start
  tag for an element. This function checks the string value of that element tag to
  decide what data it holds. It then sets the variable CurrentParsePos to point 
  at the data member corresponding with the tag string value. This CurrentParsePos 
  is used later by the parse_CharacterData() function to determine where the data 
  retrieved by parse_CharacterData() should be placed. 

  It also increments the DepthPtr for the current XML file. This value describes 
  the depth of the currently processed node.

 Ex: DepthPtr = 3:
 1 |->  <TAG1>
 2 |---->   <TAG2>
 3 |------->    <TAG3>This element is 3 deep</TAG3>
            </TAG2>
        </TAG1>
*********************************************************************************/
void CError::parse_StartElement(void *UserData, const XML_Char *Name, const XML_Char **Atts)
{
    //Increment DepthPtr to track node depth
    uint32 *DepthPtr = (uint32*)UserData;
    *DepthPtr += 1;
    if (!validXMLParseFlag) return;

#if DEBUG_PRINT
    //Debug information that will print to terminal screen
    //Prints tabspaces matching the DepthPtr, the Name of 
    //element tag, and then...(onto parse_CharacterData)
    uint32  i;
    fputc('\n', stderr);
    for (i = 0; i < *DepthPtr; i++) fputc('\t',stderr);
    fwprintf(stderr,L"%-16s",Name);
#endif

    TOUCH(Atts);

    // This area checks the current encountered tag. if it matches one that was
    // written in the serialize_to_XMLfile() function, then set CurrentParsePos 
    // to point at the relating data member.
    if(!strcmp(Name, "FAMILY")) CurrentParsePos = (ansichar*)&FAMILY_STRING; 
    else if(!strcmp(Name, "GENUS")) CurrentParsePos =  (ansichar*)&GENUS_STRING; 
    else if(!strcmp(Name, "SPECIES")) CurrentParsePos =  (ansichar*)&CERR_SPECIES_STRING; 
    //Nothing is specifically done with Version yet. Since we are backward and forward
    //compatible, we don't need to check the version. It is also a read-only value, so 
    //we don't write to it.
    
    //---------------------------------------------------------------
    else if(!strcmp(Name, "ClassID")) CurrentParsePos = (ansichar*)&m_structData.ErrorCode;
    else if(!strcmp(Name, "ErrorCode")) CurrentParsePos = (ansichar*)&m_structData.ErrorCode;
    else if(!strcmp(Name, "DESCRIPTION")) CurrentParsePos = (ansichar*)&m_structData.Description;
    else if(!strcmp(Name, "DATA")) CurrentParsePos = (ansichar*)&Data;
    //---------------------------------------------------------------
    //If there is a tag that we don't recognize, set CurrentParsePos to NULL so
    //the tag is not processed by parse_CharacterData()
    else CurrentParsePos = NULL;

}

/*********************************************************************************
XML HELPER FUNCTION
  The XML Parser makes a callback to this function whenever it encounters 
  character data after a start tag. This function uses CurrentParsePos to . This 
  CurrentParsePos value has been set previously by the parse_StartElement 
  function
*********************************************************************************/
void CError::parse_CharacterData(void *UserData, const XML_Char *sTag, int Len)
{
    XML_Char    *BufPtr = (XML_Char*)sTag;
    XML_Char    *BufStart = (XML_Char*)sTag;
    uint32      i = 0;

    TOUCH(UserData);

    //Check to see if CurrentParsePos is NULL or points to something,
    //and if validXMLParseFlag is still true. If CurrentParsePos is
    //NULL, then this means we have encountered an element tag with 
    //no internal character data, and we pass it.
    if (CurrentParsePos && validXMLParseFlag)
    {
        //This command attaches a null on the end of the character for 
        //Wide character fucntionality... remove if using UTF-8
        /*if(Len < XML_BUFSIZE) ((XML_Char*)sTag)[Len] = 0;*/

        //As we parse the XML data, we check to see if the family, 
        //genus and species match the current object. If they do not, 
        //we set validXMLParseFlag to FALSE. This will stop the parser 
        //from handling the additional data, and we clear any data 
        //already in our structure.
        //---------------------------------------------------------------
        if(CurrentParsePos == (ansichar*)&FAMILY_STRING){
            if(strncmp(sTag, (const char*)FAMILY_STRING, Len)) validXMLParseFlag = FALSE;   
        }
        else if(CurrentParsePos == (ansichar*)&GENUS_STRING){
            if(strncmp(sTag, (const char*)GENUS_STRING, Len)) validXMLParseFlag = FALSE;
        }
        else if(CurrentParsePos == (ansichar*)&CERR_SPECIES_STRING){
            if(strncmp(sTag, (const char*)CERR_SPECIES_STRING, Len)) validXMLParseFlag = FALSE;
        }
        else if(CurrentParsePos == (ansichar*)(&SrcVersion))
        {   //specific handling code
            *(uint16*)CurrentParsePos = (uint16)atol(sTag);
        }   
    
        //---------------------------------------------------------------
        else if(CurrentParsePos == (ansichar*)(&m_structData.ErrorCode))
        {
            *(uint16*)CurrentParsePos = (uint16)atol(sTag);
        }
        else if(CurrentParsePos == (ansichar*)(&m_structData.Description))
        {
            memset(CurrentParsePos, 0, sizeof(m_structData.Description));
            memcpy(CurrentParsePos, sTag, Len);
        }
        else if(CurrentParsePos == (ansichar*)(&Data))
        {
            int Count = 0;
            for (BufPtr = (XML_Char*)sTag,BufStart = (XML_Char*)sTag, i = 0; BufPtr <= sTag + Len; BufPtr++)
            {
                if ( *BufPtr == ',')
                    Count++;
            }

            if ( Count > 0 )
                Count++;

            /* Get the Data Size */
            DataSize = Count * sizeof(uint8);

            Data = (uint8*)SYS_REALLOC(Data, DataSize);

            if (Data != NULL)
            {

                for(BufPtr = (XML_Char*)sTag,BufStart = (XML_Char*)sTag, i = 0; BufPtr <= sTag + Len; BufPtr++){
                    if((*BufPtr == ',') || (BufPtr == (sTag + Len)))
                    {
                        *BufPtr = 0;
                        Data[i] = (uint8)atol(BufStart);
                        BufStart = BufPtr+1;
                        i++;
                    }
                }
            }
        }

        //---------------------------------------------------------------
        //otherwise, handle the data generically
        else
        {
            //Copy the CharacterData of sTag to the member pointed at by 
            //CurrentParsePos
            memcpy(CurrentParsePos, sTag, Len) ;
        }
#if DEBUG_PRINT
        //(from parse_StartElement)...picks up from the start
        //element and prints the value following it
        fwprintf(stderr,L" = %s",sTag);
        if(!validXMLParseFlag) fwprintf(stderr,L" - Invalid Data: Stopping parse");
#endif
    }
}

#endif
/* End of File */






