/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CError.h                                                          //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#if !defined(CERROR_H)
    #define  CERROR_H

#include "DtData.h"

/*The following definition is the version.*/
#define CERR_OBJECT_VERSION 100
#define CERR_DESC_SIZE      20

/*---------------------------------
Add Defines for string representation
of enumerations in the next section.
---------------------------------*/

    typedef  enum CERRtypSPECIES {
        CERR_SpNONE = 0,
        CERR_SpWACPSTACK,
        CERR_SpAPPINTERNAL,
        CERR_SpMAX
    } CERR_SPECIES;

    typedef enum CERRtypERROR {
        CERR_NONE = 0,
        CERR_UNIMPL,
        CERR_INVL_FAMILY,
        CERR_INVL_GENUS,
        CERR_INVL_SPECIES,
        CERR_OBJECT_ID,
        CERR_CRC_MISMATCH,  
        CERR_MEMORY,
        CERR_INVL_TBL_ENTRY,
        CERR_DUP_ENTRY,
        CERR_INVL_PORT,
        CERR_MAX
    } CERR_LIST;

    /* Add Code here for AppInternal Errors */
    typedef enum CERRtypAPPERROR {
        CERR_APP_NONE = 0,
        CERR_APP_CMD_FAILED,
        CERR_APP_MAX
    } CERR_APPLIST;


/*---------------------------------
Add Defines for string representation
of enumerations in the next section.
---------------------------------*/
#define CERR_STACK_STRING       TypStackErrorTbl[m_structData.ErrorCode]
#define CERR_APP_STRING         TypAppErrorTbl[m_structData.ErrorCode]
#define CERR_SPECIES_STRING     TypSpeciesTbl[species()]
/*---------------------------------
End...
---------------------------------*/

#ifndef CError_DLLAPI
    #if defined(_MOIB_EXPORT_DLLAPI_)
        #define CError_DLLAPI  __declspec( dllexport )
    #elif defined(_MOIB_IMPORT_DLLAPI_)
        #define CError_DLLAPI  __declspec( dllimport )
    #else
        #define CError_DLLAPI
    #endif
#endif

class CError_DLLAPI CError : public DtData
{
public:

    CError(uint8 species = CERR_SpNONE);
    ~CError();

    /*---------------------------------
    Functions:
    Add function prototypes here
    ---------------------------------*/
    void SetErrorCode(uint16 ErrorCode);
    uint16 GetErrorCode();
    void SetClassID(uint32 ClassID);
    uint32 GetClassID();

    void SetDescription(ansichar* pDesc);
    ansichar* GetDescription();
    uint16 GetLastError(ansichar* pBuf);
    void SetData(uint8* NewData, uint32 Size);
    uint8* GetData(uint32 *Size);

    void Reset(boolean bResetAll);

    //---------------------------------

    /*---------------------------------
        Inherited/Overidden functions:
        Functions derived from DtData
    ---------------------------------*/
#if ((TARGET_XML_SUPPORT == XML_SERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
    uint32 serialize_to_XMLbuf(char* pBuf);
#endif
    //---------------------------------

/*---------------------------------------------------
 Use the following struct for classes that will hold
     internal data.
---------------------------------------------------*/
    typedef struct ClassStruct_s{
        //Static Data
        uint32      ClassID;
        uint16      ErrorCode;
        ansichar    Description[CERR_DESC_SIZE];

    }ClassStruct_t;

#define CERR_STATIC_STRUCT_SIZE \
    (sizeof(uint16) + sizeof(uint32) + (CERR_DESC_SIZE * sizeof(ansichar)))

protected:

    /*---------------------------------
    Enumeration related String tables.
    ---------------------------------*/

static const char   *TypSpeciesTbl[CERR_SpMAX];
static const char   *TypAppErrorTbl[CERR_APP_MAX];
static const char   *TypStackErrorTbl[CERR_MAX];
//---------------------------------------------------
private:
    /*---------------------------------
     Inherited/Overidden functions:
     Functions derived from DtData
    ---------------------------------*/

#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
    void    parse_StartElement(void *UserData, const XML_Char *Name, const XML_Char **Atts);
    void    parse_CharacterData(void *UserData, const XML_Char *sTag, int Len);
#endif

    uint32  get_payload_size ( void );
    boolean serialize_payload ( uint8QManager * QMgr  );
    boolean deserialize_payload ( uint8QManager * QMgr, uint32 Sz );
    void serialize_static_data(uint8QManager * QMgr);
    void deserialize_static_data(uint8QManager *QMgr);

    boolean is_variable_sized ( void );
    boolean is_Aggregate ( void );

    //---------------------------------

    /*--------------------------------------------------
     The following data members relate to the struct
     defined above - if used
    --------------------------------------------------*/

    ClassStruct_t       m_structData;
    ansichar*           CurrentParsePos;
    //Used only to grab the version from incoming source objects
    uint32 SrcVersion;

    uint8   *Data;
    uint32  DataSize;

};
#endif /* #if !defined(CERROR_H) */

/*---------------------------------------------------*/
/* End of File */





