/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CEthCommSvr.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined (CETHCOMMSVR_H)
    #define   CETHCOMMSVR_H

#include "Target.h"
#include "SysUtil.h"
#include "Common.h"
#include "Mem.h"

#include <winsock.h>
#include <process.h>
#include "CWACPEvent.h"
#include "CWACPMutex.h"


typedef struct EthAddr
{
    uint16  Port;
    char*   pSourceID;
    SOCKET  Socket;
    uint16  Protocol;
    struct sockaddr_in  ClntAddr;
}EthAddr;

class CEthCommSvr
{
/*================================================================================
CEthCommSvr Members: Public, Private, and Protected
================================================================================*/

    public:
    /* Constructor - Takes in two function pointers for pushing new connections 
     * and new messages up to the calling application 
     */
    CEthCommSvr(boolean (*RecvMsg)(char* pSemName, char* pSourceID, uint8*, uint32 MsgSize, 
                   void* pNewConnection),                   
                   boolean (*TerminateClient)(char* pSemName, char* pSourceID, uint16 Port),
                   char* pAppName);
    CEthCommSvr();
    ~CEthCommSvr();
    
    /* Returns the SourceID of the CEthCommSvr. The SourceID is the unique
    * identifier for the communications session. In Ethernet the SourceID 
    * would hold the destination IP Address of the connection peer. In 
    * RS232 this would hold the Name of the com Port through which */
    char* GetSourceID();

    /* Returns the Port which represents either a RNDZ or WACP session */
    uint16 GetPortNum();

    /* Returns the type of stack that is being used ( i.e. Ethernet, 
     * RS232, Bluetooth, Zigby, etc
     */
    STACK_TYPE GetStackType();

    /* Sets up the configuration of the stack based on the data received 
     * via pAddr.
     */
    void SetConfig(void* pAddr);
    
    /* Returns the pointer to the configuration */
    void* GetConfig();

    /* Sends a binary Buffer */
    boolean SendMsg(uint8* pBuf, uint32 pBufSize);

    /* Creates a connection */
    boolean Connect(uint8* pLocalID, int PortNum);

    /* Handles the start of a new client connection thread */
    static uint32 WACPAPI StartClient(void* pData);

    /* Handles incoming connection requests */
    static uint32 WACPAPI StartServer(void* pData);

    /* Sets the communication port that distinguishes between WACP and RNDZ messages */
    boolean SetPortNum(uint16 NewPort);

    private:

        SetRecvBufPtr   pSetRecvMsg;
        TerminatePtr    pTerminate;

        char            SourceID[ADDR_BUF_SIZE];
        uint16          Protocol;
        uint16          Port;
        uint16          AutoPort;

        SOCKET          RxSocket;
        SOCKET          TxSocket;
        STACK_TYPE      StackType;
        EthAddr         Addr;

        CWACPMutex*     pUDPMutex;
        CWACPMutex      ClientExitMutex;


        /*** WIN32 API ***/
        boolean         WSAStartFlag;

    public:
        boolean         SetSourceID(char* pSourceID);
        boolean         SetProtocol(uint16 NewProtocol);
        void            SetAutoPort(uint16 NewPort);
        char*           pSemName;
};


#endif /* #if !defined (CETHCOMMSVR_H) */

/*---------------------------------------------------*/
/* End of File */





