/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmRNDZObjSvr.h:                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CFMRNDZOBJSVR_H)
    #define  CFMRNDZOBJSVR_H

#include "CObjSvr.h"
#include "CMsgSvr.h"
#include "CTrapObjSvr.h"
#include "CTraceSvr.h"

    typedef  enum RNDZCCMDtypSPECIES {
        RNDZ_CCMD_SpNONE = 0,
        RNDZ_CCMD_SpTERMINATE,
        RNDZ_CCMD_SpSTART_HOST_PROC,
        RNDZ_CCMD_SpSEND_HOST_READY,
        RNDZ_CCMD_SpMAX
    } RNDZ_CCMD_SPECIES;

    typedef  enum RNDZCRSTtypSPECIES {
        RNDZ_CRST_SpNONE = 0,
        RNDZ_CRST_SpSTART_RNDZ,
        RNDZ_CRST_SpGET_ENCRYPT_KEY,
        RNDZ_CRST_SpMAX
    } RNDZ_CRST_SPECIES;

    typedef  enum RNDZCRSPtypSPECIES {
        RNDZ_CRSP_SpNONE = 0,
        RNDZ_CRSP_SpRNDZ_ACCEPT,
        RNDZ_CRSP_SpSEND_ENCRYPT_KEY,
        RNDZ_CRSP_SpHOST_READY,
        RNDZ_CRSP_SpMAX
    } RNDZ_CRSP_SPECIES;

    typedef  enum RNDZCSTStypSPECIES {
        RNDZ_CSTS_SpNONE = 0,
        RNDZ_CSTS_SpMAX
    } RNDZ_CSTS_SPECIES;

class CFmRNDZObjSvr : public CObjSvr
{
/*================================================================================
RENDEZVOUS Members: Public, Private, and Protected
================================================================================*/
private:
    CMsgSvr MsgSvr;
    CTrapObjSvr* pTrapServer;
    CTraceSvr* pTraceServer;

    /*================================================================================
    RENDEZVOUS Utility - Callback Function pointers
    ================================================================================*/
    DataFuncPtr       pMessageHandler;

    SendPacketFuncPtr   pSendMsg;
    char* pSemName;

public:
    /*================================================================================
    RENDEZVOUS Constructor/Deconstructor
    ================================================================================*/
    CFmRNDZObjSvr();
    ~CFmRNDZObjSvr();

    void Init(CTrapObjSvr* pTrapObjSvr, CTraceSvr* pTraceSvr,
              boolean (*SendPacket)(char*, char*, uint8*, uint32), 
              LinkStruct* pLinkStruct);

    boolean SetCallbacks(va_list vList);

    /*================================================================================
    RENDEZVOUS Message function initialization
    ================================================================================*/
    boolean SetMessageFunc(uint32 (*RxMessage)(char*, uint32, uint8*, void*));

    /*================================================================================
    RENDEZVOUS Message handlers.  
    ================================================================================*/
    boolean TxMsg(char* pSourceID, CMsgSvr* pMsgSvr, uint8 Genus, uint8 Species, uint8* Payload, uint32 Size);
    boolean RecvMsgHandler(char* pSourceID, uint8* pBuffer, void* pRouter );
    boolean SendMsgHandler(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                            uint8* pBuffer, uint32 Size, boolean EncryptionOn);

};
#endif /* #if !defined(CFMRNDZOBJSVR_H) */

/*---------------------------------------------------*/
/* End of File */




