/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmRouter.cpp  :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CFmRouter.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
    CFmRouter Start up: 

*********************************************************************************/
CFmRouter::CFmRouter()
{      
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    /* Initialize the MsgCounts */
    MsgInCount = 0;
    MsgOutCount = 0;
#endif
}

CFmRouter::~CFmRouter()
{
}

void CFmRouter::Init(CTraceSvr *pCTraceSvr,
                     CTrapObjSvr *pCTrapObjSvr,
                     void *pCWACPApp)
{       
    /* Initialize the array list to NULL (0) */
    memset(ObjSvrList, 0, sizeof(ObjSvrList));

    /* Get a reference to the CTraceSvr instance */
    pTraceSvr = pCTraceSvr;

    /* Get a reference to the CTrapObjSvr instance for sending error messages */
    pTrapSvr = pCTrapObjSvr;

    /* Get a reference to the CWACPApp instance */
    pWACPApp = pCWACPApp;
}

/*================================================================================
FmRouter Register object servers - The valid object servers are listed in the
    enumerated type OBJSVR.
================================================================================*/
boolean CFmRouter::registerServer(FAMILY Family, CObjSvr* pObjSvr)
{
    boolean Result=TRUE;
    OBJSVR  Index;

    switch(Family)
    {
        case FmRENDEZVOUS:
            Index = OBJSVR_RNDZ;
            break;
        case FmTRAP:
            Index = OBJSVR_TRAP;
            break;
        case FmCONNECTION:
            Index = OBJSVR_CONN;
            break;
        default:
            Index = OBJSVR_DATA;
            break;
    }

    ObjSvrList[Index] = pObjSvr;

    return Result;
}

/*================================================================================
CFmRouter registerCallback: 
================================================================================*/
boolean CFmRouter::registerCallbacks(FAMILY Family, va_list vList)
{
    boolean Result = FALSE;
    OBJSVR  Index;

    switch(Family)
    {
        case FmRENDEZVOUS:
            Index = OBJSVR_RNDZ;
            break;
        case FmTRAP:
            Index = OBJSVR_TRAP;
            break;
        case FmCONNECTION:
            Index = OBJSVR_CONN;
            break;
        default:
            Index = OBJSVR_DATA;
            break;
    }

    Result = ObjSvrList[Index]->SetCallbacks(vList);

    return Result;
}
/*================================================================================
FmRouter Command, Response, and Request Routing for outgoing messages
================================================================================*/
boolean CFmRouter::SendMsg(char *pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                            uint8 *pBuffer, uint32 Size, boolean EncryptionOn)
{
    boolean RetVal;
    OBJSVR  Index;

    switch(Family)
    {
        case FmRENDEZVOUS:
            Index = OBJSVR_RNDZ;
            break;
        case FmTRAP:
            Index = OBJSVR_TRAP;
            break;
        case FmCONNECTION:
            Index = OBJSVR_CONN;
            break;
        default:
            Index = OBJSVR_DATA;
            break;
    }

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    char tempString[] = {"MsgOutCount %d\nMessage Data:\n%A"};
    pTraceSvr->tprintf(TRACE_Type_SEND,
        "CFmRouter", "SendMsg",
        Family, Genus, Species, 
        tempString, ++MsgOutCount, pBuffer, Size);
#endif

    /* Call the messagehandler for the object server */
    RetVal = (ObjSvrList[Index])->SendMsgHandler(pSourceID, Family, Genus, Species, pBuffer, Size, EncryptionOn);

    return RetVal;
}
/*================================================================================
FmRouter Command, Response, and Request Routing for incoming messages
================================================================================*/
boolean CFmRouter::RecvMsg(char *pSourceID, uint8 *pBuffer, uint32 Size)
{
    uint16  Family;
    uint8   Genus;
    uint8   Species;
    boolean RetVal = TRUE;
    OBJSVR  Index;

    TOUCH(Size);

    /* Extract the Family */
    Family = GetFamily(pBuffer);

    /* Extract the Genus */
    Genus = GetGenus(pBuffer);

    /* Extract the Species */
    Species = GetSpecies(pBuffer);

    switch(Family)
    {
        case FmRENDEZVOUS:
            Index = OBJSVR_RNDZ;
            break;
        case FmTRAP:
            Index = OBJSVR_TRAP;
            break;
        case FmCONNECTION:
            Index = OBJSVR_CONN;
            break;
        default:
            Index = OBJSVR_DATA;
            break;
    }

    if (ObjSvrList[Index])
    {
        /* Call the messagehandler for the object server */
        (ObjSvrList[Index])->RecvMsgHandler(pSourceID, pBuffer, pWACPApp);
    }

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    char tempString[] = {"MsgInCount %d\nMessage Data:\n%A"};
    pTraceSvr->tprintf(TRACE_Type_RECEIVE,
        "CFmRouter", "RecvMsg",
        Family, Genus, Species, 
        tempString, ++MsgInCount, pBuffer, Size);
#endif

    return RetVal;
}

/* End of File */







