/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CMOMPBaseClass.cpp                                                //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CWACPApp.h"
#include "CMOMPBaseClass.h"
#include "CRNDZDStd.h"

CMOMPBaseClass::CMOMPBaseClass()
{
    pWACPApp  = NULL;
    bAppInit  = FALSE;
    bServer   = FALSE;
    bExitFlag = FALSE;
}
CMOMPBaseClass::~CMOMPBaseClass()
{
    
}
/*================================================================================
Reset all CMOMPBaseClass members
================================================================================*/
void CMOMPBaseClass::Reset()
{
    pWACPApp  = NULL;
    bAppInit  = FALSE;
    bServer   = FALSE;
    bExitFlag = FALSE;
}
/*================================================================================
Internal Entry point from CWACPApp to run the process
================================================================================*/
CMOMPBaseClass* CMOMPBaseClass::AppInitInt(CWACPApp* pWACPAppRef, boolean bSvrFlag, char* pServerSourceID)
{
    CMOMPBaseClass* pThis = NULL;

    if(pWACPAppRef != NULL)
    {
        bAppInit = TRUE;
        bServer = bSvrFlag;
        pWACPApp = pWACPAppRef;
        bExitFlag = FALSE;

        pThis = AppInit(pWACPAppRef, bSvrFlag, pServerSourceID);
        if(pThis == NULL)
            pThis = (CMOMPBaseClass*)this;      
    }

    return(pThis);
}

/*================================================================================
Internal Exit point from CWACPApp to exit the process
================================================================================*/
uint32 CMOMPBaseClass::AppEndInt(CWACPApp* pWACPAppRef)
{
    uint32 Result = 0;
        
    if(pWACPAppRef != NULL)
    {   
        /*Reset boolean flags*/
        bAppInit = FALSE;
        bExitFlag = TRUE;

        /*Reset local instance of WACPApp Pointer to NULL*/
        pWACPApp = NULL;

        Result = AppEnd(pWACPAppRef);
    }

    return(Result);
}

/*================================================================================
Open: Opens communication channels 
================================================================================*/
boolean CMOMPBaseClass::Open(ConnArgs* *pConnArgs, uint8 NumConnections)
{
    boolean Result = FALSE;

    if(pWACPApp != NULL)
    {
        Result = pWACPApp->Open(pConnArgs, NumConnections);
    }

    return Result;
}
/*================================================================================
Connect: 
================================================================================*/
boolean CMOMPBaseClass::Connect(char* pSourceID)
{
    boolean Result = FALSE;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        Result = pWACPApp->Connect(pSourceID);
    }

    return Result;
}
/*================================================================================
Close: Closes a WACP Connection
================================================================================*/
boolean CMOMPBaseClass::Close(char* pSourceID)
{
    boolean Result = FALSE;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        Result = pWACPApp->Close(pSourceID);
    }

    return Result;
}
/*================================================================================
SendMsg:
================================================================================*/
boolean CMOMPBaseClass::SendMsg(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, uint8* pBuf, uint32 BufSize, boolean bEncryptionOn)
{
    boolean Result = FALSE;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        Result = pWACPApp->SendMsg(pSourceID, Family, Genus, Species, pBuf, BufSize, bEncryptionOn);
    }

    return Result;
}
/*================================================================================
SendTrap: Sends Trap Messages
================================================================================*/
boolean CMOMPBaseClass::SendTrap(char* pSourceID, uint32 MsgClassID, uint32 ErrClassID, uint8 ErrorSpecies, uint16 ErrorCode,
                           uint8* pData, uint16 DataSize)
{
    boolean Result = FALSE;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        Result = pWACPApp->SendTrap(pSourceID, MsgClassID, ErrClassID, ErrorSpecies, ErrorCode, pData, DataSize);
    }

    return Result;
}
/*================================================================================
SendMsgProcTrap: Sends Message Process Trap 
================================================================================*/
boolean CMOMPBaseClass::SendMsgProcTrap(char* pSourceID, uint32 ClassID, uint16 ErrorCode)
{
    boolean Result = FALSE;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        Result = pWACPApp->SendTrap(pSourceID, ClassID, ClassID, CERR_SpERRMSGPROC, ErrorCode, NULL, 0);
    }

    return (Result);
}
/*================================================================================
SetConnOptions: Sets Connection Options with ConObjSvr
================================================================================*/
boolean CMOMPBaseClass::SetConnOptions(char* pSourceID, CONN_CFG_SPECIES Options, 
                                 boolean bSyncFlag, time_t Interval, time_t TimeOut)
{
    boolean Result = FALSE;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        Result = pWACPApp->SetConnOptions(pSourceID, Options, bSyncFlag, Interval, TimeOut);
    }
    return (Result);
}
/*================================================================================
GetConnOptions: Gets Connection Options from ConObjSvr
================================================================================*/
boolean CMOMPBaseClass::GetConnOptions(char* pSourceID, CONN_CFG_SPECIES* pOptions, time_t* pInterval, time_t* pTimeOut)
{
    boolean Result = FALSE;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        Result = pWACPApp->GetConnOptions(pSourceID, pOptions, pInterval, pTimeOut);
    }

    return (Result);
}
/*================================================================================
Terminate : Terminates a WACP connection
================================================================================*/
void CMOMPBaseClass::Terminate()
{
    if (pWACPApp != NULL)
    {
        pWACPApp->Terminate();
    }
}

/*================================================================================
GetGUID : Retrieves the GUID from the LinkTable.
================================================================================*/
uint8* CMOMPBaseClass::GetGUID(char* pSourceID, uint16* pGUIDSize)
{
    CRNDZDStd* pGUIDStd = NULL;
    uint8*    pGUID = NULL;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        pGUIDStd = pWACPApp->GetGUID(pSourceID);
        if (pGUIDStd)
        {
            pGUID = pGUIDStd->GetDeviceGUID(pGUIDSize);
        }
    }

    return pGUID;
}

/*================================================================================
GetGUIDObj : Retrieves the GUID object from the LinkTable
================================================================================*/
CRNDZDStd* CMOMPBaseClass::GetGUIDObj(char* pSourceID)
{
    CRNDZDStd* pGUIDStd = NULL;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        pGUIDStd = pWACPApp->GetGUID(pSourceID);
    }

    return pGUIDStd;
}

/*================================================================================
GetStackType : Retrieves the stack type from the LinkTable
================================================================================*/
STACK_TYPE CMOMPBaseClass::GetStackType(char* pSourceID)
{
    STACK_TYPE StackType = STACK_UNDEFINED;

    if((pWACPApp != NULL) && (pSourceID != NULL))
    {
        StackType = pWACPApp->GetStackType(pSourceID);
    }

    return StackType;
}

