/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CMOMPBaseClass.h                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CMOMPBASECLASS_H) 
    #define  CMOMPBASECLASS_H

class CWACPApp;
class CRNDZDStd;

class CMOMPBaseClass 
{
public:
    MOMP_DLLAPI CMOMPBaseClass();
    MOMP_DLLAPI ~CMOMPBaseClass();

    MOMP_DLLAPI void Reset();

    /* Internal Application entry point from CWACPApp */
    MOMP_DLLAPI CMOMPBaseClass* AppInitInt(CWACPApp* pWACPAppRef, boolean bSvrFlag, char* pServerSourceID);
    
    /* Internal Application exit point from CWACPApp */
    MOMP_DLLAPI uint32 AppEndInt(CWACPApp* pWACPAppRef);
    
    /*Virtual methods*/
    virtual CMOMPBaseClass* AppInit(CWACPApp* pWACPAppRef, boolean bSvrFlag, char* pServerSourceID) = 0;
    virtual uint32 AppEnd(CWACPApp* pWACPAppRef) = 0;  
    virtual uint32 RecvMessage(char* pSourceID, uint32 ClassID, uint8* pBuffer) = 0;  
    virtual uint32 RecvError(char* pSourceID, uint8 ErrorType, uint32 ClassID, uint8* pBuffer) = 0;
    virtual uint32 RecvConnection(char* pSourceID, uint8 ConnType, uint8* pBuffer) = 0;

    /* CWACPApp wrapper functions */
    MOMP_DLLAPI boolean Open(ConnArgs** pConnArgs, uint8 NumConnections);
    MOMP_DLLAPI boolean Connect(char* pSourceID);
    MOMP_DLLAPI boolean Close(char* pSourceID);
    MOMP_DLLAPI void    Terminate();

    MOMP_DLLAPI boolean SendMsg(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, uint8* pBuf, uint32 BufSize, boolean bEncryptionOn = FALSE);
    MOMP_DLLAPI boolean SendTrap(char* pSourceID, uint32 MsgClassID, uint32 ErrClassID, uint8 ErrorSpecies, uint16 ErrorCode,
                           uint8* pData, uint16 DataSize);
    MOMP_DLLAPI boolean SendMsgProcTrap(char* pSourceID, uint32 ClassID, uint16 ErrorCode);
    MOMP_DLLAPI boolean SetConnOptions(char* pSourceID, CONN_CFG_SPECIES Options, boolean bSyncFlag, time_t Interval, time_t TimeOut);
    MOMP_DLLAPI boolean GetConnOptions(char* pSourceID, CONN_CFG_SPECIES* pOptions, time_t* pInterval, time_t* pTimeOut);

    MOMP_DLLAPI uint8* GetGUID(char* pSourceID, uint16* pGUIDSize);
    MOMP_DLLAPI CRNDZDStd* GetGUIDObj(char* pSourceID);
    
    MOMP_DLLAPI STACK_TYPE GetStackType(char* pSourceID);
    
    CWACPApp* pWACPApp;

    /* Set when AppInit callback received */
    boolean bAppInit;
    boolean bServer;
    boolean bExitFlag;
};

#endif /* #if !defined(CMOMPBASECLASS_H) */

/*---------------------------------------------------*/
/* End of File */
