/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CRNDZStub.h    :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CRNDZSTUB_H)
    #define  CRNDZSTUB_H

#include "Common.h"
#include "CFmRNDZObjSvr.h"
#include "CEncryption.h"
#include "CRNDZDStd.h"

/*State Machine to prevent a client from connecting */
/*without first going through the whole validation process*/
typedef enum RNDZValidationStates{
        RNDZ_VALIDATION_NONE=0,
        RNDZ_VALIDATION_START_RNDZ_RECVD,
        RNDZ_VALIDATION_KEY_EXCHANGE,
        RNDZ_VALIDATION_COMPLETE
    }RNDZ_VALIDATION_TYPE;

class CWACPApp;

class CRNDZStub
{
private:
    CWACPApp*   pWACPApp;
    CRNDZDStd    GUIDDStd;

public:
    CRNDZStub();
    ~CRNDZStub();

    void   Init(CWACPApp* WACPApp);

    uint32 RecvMessage(char* pSourceID, uint32 ClassID, uint8* pBuffer);
    uint32 RecvCommand(char* pSourceID, uint8 CommandID, uint8* pBuffer);
    uint32 RecvRequest(char* pSourceID, uint8 RequestID, uint8* pBuffer);
    uint32 RecvResponse(char* pSourceID, uint8 ResponseID, uint8* pBuffer);
    uint32 RecvConnection(char* pSourceID, uint8 ConnType, uint8* pBuf);

};

#define SEND_RNDZ_COMMAND(pSourceID, CommandID, pDataBuffer, DataSize) pWACPApp->SendMsg(pSourceID, FmRENDEZVOUS, GnCOMMAND, CommandID, pDataBuffer, DataSize, FALSE)
#define SEND_RNDZ_REQUEST(pSourceID, RequestID) pWACPApp->SendMsg(pSourceID, FmRENDEZVOUS, GnREQUEST, RequestID, NULL, 0, FALSE)
#define SEND_RNDZ_RESPONSE(pSourceID, ResponseID, pDataBuffer, DataSize) pWACPApp->SendMsg(pSourceID, FmRENDEZVOUS, GnRESPONSE, ResponseID, pDataBuffer, DataSize, FALSE)
#define SEND_RNDZ_ERROR(pSourceID, ErrorID, pErrorDesc, ErrorSize) pWACPApp->SendMsg(pSourceID, FmTRAP, GnERROR, ErrorID, pErrorDesc, ErrorSize)
#define TERMINATE_RNDZ(pSourceID, Status) pWACPApp->RNDZTerminate(pSourceID, Status)

#endif /* #if !defined(CRNDZSTUB_H) */

/*---------------------------------------------------*/
/* End of File */



