/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CSerialCommSvr.h:                                                 //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined (CSERIALCOMMSVR_H)
    #define   CSERIALCOMMSVR_H

#include "Target.h"
#include "SysUtil.h"
#include "Common.h"
#include "Mem.h"

#include <windows.h>
#include "CWACPEvent.h"
#include "CWACPMutex.h"

typedef struct SerialAddr
{
    /* User configurable */     
    uint32  BaudRate;           /* Rate of connection Ex. 9600, 115200, 230400, etc... */
    boolean bAutoDiscovery;     /* TRUE - Server/Host auto detects client via hardware trigger 
                                 * FALSE - Client must send initial RNDZ connect message.
                                 */ 

    /* Stack modified values */
    uint16  Port;               /* Current message type: RNDZ or WACP Port */
    char*   pSourceID;         /* Friendly name of com port i.e. "COM1:" */
    HANDLE  hPort;              /* Handle to the com port */       
}SerialAddr;

class CSerialCommSvr
{
/*================================================================================
CSerialCommSvr Members: Public, Private, and Protected
================================================================================*/

    public:
    /* Constructor - Takes in two function pointers for pushing new connections 
     * and new messages up to the calling application 
     */
    CSerialCommSvr(boolean (*RecvMsg)(char* pSemName, char* pSourceID, uint8*, uint32 MsgSize, 
                   void* pNewConnection),                   
                   boolean (*TerminateClient)(char* pSemName, char* pSourceID, uint16 Port),
                   char* pAppName);
    CSerialCommSvr();
    ~CSerialCommSvr();
    
    /* Returns the SourceID of the CSerialCommSvr. The SourceID is the unique
    * identifier for the communications session. In Ethernet the SourceID 
    * would hold the destination IP Address of the connection peer. In 
    * RS232 this would hold the Name of the com Port through which */
    char* GetSourceID();

    /* Returns the Port which represents either a RNDZ or WACP session */
    uint16 GetPortNum();

    /* Returns the type of stack that is being used ( i.e. Ethernet, 
     * RS232, Bluetooth, Zigby, etc
     */
    STACK_TYPE GetStackType();

    /* Sets up the configuration of the stack based on the data received 
     * via pAddr.
     */
    void SetConfig(void* pAddr);

    /* Returns the pointer to the configuration */
    void* GetConfig();

    /* Sends a binary Buffer */
    boolean SendMsg(uint8* pBuf, uint32 pBufSize);

    /* Creates a connection */
    boolean Connect(uint8* pLocalID, int PortNum);

    /* Handles the start of a new client connection thread */
    static uint32 WACPAPI StartClient(void* pData);

    /* Handles incoming connection requests */
    static uint32 WACPAPI StartServer(void* pData);

    /* Sets the communication port that distinguishes between WACP and RNDZ messages */
    boolean SetPortNum(uint16 NewPort);

    private:
        SetRecvBufPtr   pSetRecvMsg;
        TerminatePtr    pTerminate;

        char            SourceID[ADDR_BUF_SIZE];
        uint16          Port;
        uint32          BaudRate;
        STACK_TYPE      StackType;

        CWACPMutex      LockMutex;              
        CWACPMutex      ExitMutex;              

        boolean         ReadData(char* pBuf, uint32 BytesToRead, uint32* pBytesRead);
        boolean         CheckDTR();

        HANDLE          hPort;            
        OVERLAPPED      OSWrite, OSRead;
        SerialAddr      Addr;
        boolean         bByPass;            /* TRUE - Server baseline to bypass arrival mechanism */ 
    
    public:
        uint32      GetBaudRate();
        boolean     SetBaudRate(uint32 NewBaudRate);
        boolean     SetSourceID(char* pSourceID);
        boolean     SetByPass(boolean bPass);
        char*       pSemName;
};


#endif /* #if !defined (CSERIALCOMMSVR_H) */

/*---------------------------------------------------*/
/* End of File */





