/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CTrapObjSvr.cpp:                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CTrapObjSvr.h"

#if defined(ATE_ERROR_SUPPORT)
#define ERROR_CLASS_ID_OFFSET 8
#endif

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
    CTrapObjSvr Start up: 

*********************************************************************************/
CTrapObjSvr::CTrapObjSvr()
{
    Outbuf = NULL;
}

CTrapObjSvr::~CTrapObjSvr()
{
    /* Free the memory allocated to Outbuf */
    SYS_FREE(Outbuf);
    Outbuf = NULL;
}

void CTrapObjSvr::Init(boolean (*SendPacket)(char*, char*, uint8*, uint32), 
                         CTraceSvr *pTraceSvr, LinkStruct *pLinkStruct)
{
    ErrObj.set_Species(CERR_SpERRNONE);

    pTraceServer = pTraceSvr;
    pSendPacket = SendPacket;
    pSemName = pLinkStruct->pSemName;

    /* Initialize the general message server */
    MsgSvr.Init(FmTRAP, GnDATA, pLinkStruct);
}
/*================================================================================
CTrapObjSvr TxErrMsg : Helper routine that transmits the contents of a CErrSvr object
================================================================================*/
boolean CTrapObjSvr::TxErrMsg(char *pSourceID, CErrSvr *pErrSvr, uint8 Genus, uint8 Species)
{
    uint32 OutBufSz;
    boolean Result = FALSE;

    /* Set the Genus */
    pErrSvr->set_Genus(Genus);
    
    /* Set the Species */
    pErrSvr->set_Species(Species);

    /* Get the Size of the serialized byte Stream */
    OutBufSz = pErrSvr->size_all();

    /* (Re)Allocate space for the serialized Buffer */
    Outbuf = (uint8 *)SYS_REALLOC(Outbuf, OutBufSz);
    
    /* Serialize the message */
    if (Outbuf != NULL)
    {
        /* Serialize the CErrSvr object */
        pErrSvr->serialize_all(Outbuf);

        /* Send the message */
        Result = pSendPacket(pSemName, pSourceID, Outbuf, OutBufSz);

    }

    return(Result);
}

/*================================================================================
CTrapObjSvr TxMsg : Helper routine that transmits the contents of a CMsgSvr object
================================================================================*/
boolean CTrapObjSvr::TxMsg(char *pSourceID, CMsgSvr *pMsgSvr, uint8 Genus, uint8 Species)
{
    uint32 OutBufSz;
    boolean Result = FALSE;

    /* Set the Genus */
    pMsgSvr->set_Genus(Genus);
    
    /* Set the Species */
    pMsgSvr->set_Species(Species);

    /* Get the Size of the serialized byte Stream */
    OutBufSz = pMsgSvr->size_all();

    /* (Re)Allocate space for the serialized Buffer */
    Outbuf = (uint8 *)SYS_REALLOC(Outbuf, OutBufSz);
    
    /* Serialize the message */
    if (Outbuf != NULL)
    {
        /* Serialize the CErrSvr object */
        pMsgSvr->serialize_all(Outbuf);

        /* Send the message */
        Result = pSendPacket(pSemName, pSourceID, Outbuf, OutBufSz);
    }

    return(Result);
}

/*================================================================================
CTrapObjSvr TxError : Sends an error message based ErrorType and stores the last 
    CErrEStd byte stream into ErrObj for supporting the Get_LASTERROR request.
================================================================================*/
boolean CTrapObjSvr::TxError(char *pSourceID, uint32 ClassID, uint8 ErrorType, uint8* pBuf, 
                             uint32 Size)
{
    boolean Result = FALSE;

    if ((pBuf != NULL) && (Size > 0))
    {
        /* Load the last ErrObj sent out by deserializing the data 
        * The request for last error will return the data held by
        * pErrObj so we must update this each time an error message
        * is sent.
        */
        ErrObj.deserialize_all(pBuf);
    }

    /* Set the ClassID */
    ErrSvr.SetClassID(ClassID);

    /* Check to see if we are trying to update the same pointer 
     * This happens when we are sending a loopback error
     */
    if (pBuf != ErrSvr.Data)
    {
        /* Set the data field */
        ErrSvr.SetData(pBuf, Size);
    }

    /* Send the message */
    Result = TxErrMsg(pSourceID, &ErrSvr, GnERROR, ErrorType);

    return Result;
}

/*================================================================================
TRAP SendError: Internal call for sending a CERRESTD_SpWACPSTACK error to the 
    application. It takes the ClassID and ErrorCode to create a CErrEStd object 
    byte stream and notifies the application of the error.
================================================================================*/
boolean CTrapObjSvr::SendError(char *pSourceID, uint32 ClassID, uint16 ErrorCode, void *pWACPApp)
{
    uint32 Result=0;
    uint32 MsgSize;

    /* Create a new instance of the error object and store the new error */
    ErrObj.Reset(TRUE);
    
    /* Set the Species */
    ErrObj.set_Species(CERR_SpERRWACPSTACK);

    /* Set the ErrorCode */
    ErrObj.SetErrorCode(ErrorCode);

    /* Set the ClassID */
    ErrObj.SetErrClassID(ClassID);

    /* Get the Size of the error object */
    MsgSize = ErrObj.size_all();

    /* Allocate the proper Buffer Size */
    Outbuf = (uint8*) SYS_REALLOC(Outbuf, MsgSize);
    if (Outbuf)
    {
        /* Serialize the error object */
        ErrObj.serialize_all(Outbuf);

        /* Check to see if we should also send this to the application */
        if (pWACPApp)
        {
            /* Report the error to the application */
            Result = errorHandler(pSourceID, GetSpecies(Outbuf), ClassID, Outbuf, pWACPApp);
        }
    }

    return (Result ? TRUE : FALSE);
}

/*================================================================================
CTrapObjSvr SendMsgHandler: Sends a Trap message. 
================================================================================*/
boolean CTrapObjSvr::SendMsgHandler(char *pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                            uint8 *pBuffer, uint32 Size, boolean EncryptionOn)
{   
    boolean Result = FALSE;
    TOUCH(EncryptionOn);

    /* Handle external errors if the Family is FmTrap */
    if (Family == FmTRAP)
    {
        switch (Genus)
        {
            case GnREQUEST:
            case GnRESPONSE:
            {
                MsgSvr.SetData(pBuffer, Size);
                Result = TxMsg(pSourceID, &MsgSvr, Genus, Species);
                break;
            }

#if defined(ATE_ERROR_SUPPORT)
            case GnERROR:
            {
                uint8* pClassID = NULL;
                uint32 ClassID = 0;

                if (pBuffer != NULL)
                {
                    /* Set pClassID to the Relative offset of the ErrClassID */
                    pClassID = pBuffer + ERROR_CLASS_ID_OFFSET;
                    
                    /* Set ClassID to the Error Class ID within the CError Object */
                    ClassID = ReadInt32(pClassID);

                    /* Set the ClassID */
                    ErrSvr.SetClassID(ClassID);
                }
                /* Set the data field */
                ErrSvr.SetData(pBuffer, Size);

                /* Send the message */
                Result = TxErrMsg(pSourceID, &ErrSvr, Genus, Species);
                break;
            }
#endif
            default:
            {
#if defined(TRACE_SERVER_ON)
                pTraceServer->tprintf(TRACE_Type_ERROR, "CTrapObjSvr", "SendMsgHandler",
                                    Family, Genus, Species, "Invalid Genus");
#endif
                break;
            }
        }
    }

    return Result;
}

/*================================================================================
CTrapObjSvr RecvMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
     the messages with the appropriate object.
================================================================================*/
boolean CTrapObjSvr::RecvMsgHandler(char *pSourceID, uint8 *pBuffer, void *pRouter )
{
    uint32  Result = TRUE;;
    uint32  Size;
    uint8   Genus;
    uint8   Species;
#if defined(TRACE_SERVER_ON)
    uint16  Family;

    /* Extract the Family */
    Family = GetFamily(pBuffer);
#endif
    
    /* Extract the Genus */
    Genus = GetGenus(pBuffer);

    /* Extract the Species */
    Species = GetSpecies(pBuffer);

    /* Genus Router */
    switch (Genus)
    {           
        case GnERROR:
        case GnCONFIG:
        {
            /* Set the Genus */
            ErrSvr.set_Genus(Genus);

            /* Deserialize the data and check for errors */
            Result = ErrSvr.deserialize_all(pBuffer);
            break;
        }
        case GnRESPONSE:
        case GnREQUEST:
        {
            /* Set the Genus */
            MsgSvr.set_Genus(Genus);

            /* Deserialize the data and check for errors */
            Result = MsgSvr.deserialize_all(pBuffer);
            break;
        }
    }

    if (Result)
    {
        /* Genus Router */
        switch (Genus)
        {           
            case GnERROR:
            {
                /* Call the Callback function for error messages */
                if (errorHandler != NULL)
                {
                    uint32  ClassID = ErrSvr.GetClassID();
                    uint8   *pErrBuf = ErrSvr.GetData(&Size);
    
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
                    if (Species == CES_SpNONE)
                    {
                        pTraceServer->tprintf(TRACE_Type_STATUS, "CTrapObjSvr", "RecvMsgHandler",
                                          Family, Genus, Species, 
                                          "Received ACK Message");
                    }                    
                    else if (Species == CES_SpWACPSTACK)
                    {
                        pTraceServer->tprintf(TRACE_Type_ERROR, "CTrapObjSvr", "RecvMsgHandler",
                                          Family, Genus, Species, 
                                          "Received Stack NACK Message");
                    }
                    else if (Species == CES_SpERRMSGPROC)
                    {
                        pTraceServer->tprintf(TRACE_Type_ERROR, "CTrapObjSvr", "RecvMsgHandler",
                                          Family, Genus, Species, 
                                          "Received Message Processing NACK Message");
                    }
                    else if (Species == CES_SpERRAPPINTERNAL)
                    {
                        pTraceServer->tprintf(TRACE_Type_ERROR, "CTrapObjSvr", "RecvMsgHandler",
                                          Family, Genus, Species, 
                                          "Received Application NACK Message");
                    }
                    else if (Species == CES_SpLOOPBACK)
                    {
                        pTraceServer->tprintf(TRACE_Type_ERROR, "CTrapObjSvr", "RecvMsgHandler",
                                          Family, Genus, Species,
                                          "Received LOOPBACK ERROR Message");
                    }
                    else
                    {
                        pTraceServer->tprintf(TRACE_Type_ERROR, "CTrapObjSvr", "RecvMsgHandler",
                                          Family, Genus, Species,
                                          "Received Invalid ERROR Species");
                    }
#endif
                    /* If the species is loopback then send the error */
                    if (Species == CES_SpLOOPBACK)
                    {
                        SendError(pSourceID, MakeClassID(FmARCH, GnERROR, CES_SpWACPSTACK), CERR_CRC_MISMATCH, pRouter);
                    }
                    else
                    {
                        /* Report the error to the application */
                        Result = errorHandler(pSourceID, Species, ClassID, pErrBuf, pRouter);
                    }
                }/* if (errorHandler != NULL) */
                
                break;

            }/* case GnERROR: */
            case GnREQUEST:
            {
                if (Species == TRAP_CRST_SpGET_LASTERROR)
                {
                    uint32 MsgSize;
#if defined(TRACE_SERVER_ON)
                    pTraceServer->tprintf(TRACE_Type_STATUS, "CTrapObjSvr", "RecvMsgHandler",
                                          Family, Genus, Species, 
                                          "Received a request for last error");
#endif
                    /* Get the Size of the error object */
                    MsgSize = ErrObj.size_all();

                    /* Allocate the proper Buffer Size */
                    Outbuf = (uint8*) SYS_REALLOC(Outbuf, MsgSize);
                    if (Outbuf)
                    {
#if defined(TRACE_SERVER_ON)
                        uint16 DescSize;
#endif
                        /* Serialize the error object */
                        ErrObj.serialize_all(Outbuf);

#if defined(TRACE_SERVER_ON)
                        pTraceServer->tprintf(TRACE_Type_SEND, "CTrapObjSvr", "SendMsgHandler",
                                        Family, Genus, Species, 
                                        "Sending last error message. Description: %s", ErrObj.GetDescription(&DescSize));
#endif

                        /* Resend the last error message */
                        MsgSvr.SetData(Outbuf, MsgSize);

                        Result = TxMsg(pSourceID, &MsgSvr, GnRESPONSE, TRAP_CRSP_SpPUT_LASTERROR);

                    }/* if (Outbuf) */

                }/* if (Species == TRAP_CRSP_SpGET_LASTERROR) */

                break;
            }/* case GnREQUEST */
            case GnRESPONSE:
            {
                if (Species == TRAP_CRSP_SpPUT_LASTERROR)
                {
                    /* Call the Callback function for error messages */
                    if (errorHandler != NULL)
                    {
                        uint8   *pErrBuf = MsgSvr.GetData(&Size);

#if defined(TRACE_SERVER_ON)
                        pTraceServer->tprintf(TRACE_Type_STATUS, "CTrapObjSvr", 
                                            "RecvMsgHandler",
                                            Family, Genus, Species, 
                                            "Received last error message");
#endif
        
                        Result = errorHandler(pSourceID, GetSpecies(pErrBuf), MakeClassID(FmTRAP, GnRESPONSE, CMSG_SpNONE), pErrBuf, pRouter);
                    }

                }/* if (Species == TRAP_CRSP_SpGET_LASTERROR) */

                break;
            }/* case GnRESPONSE */
            default:
                break;
        }/* switch (Genus)*/

    }/* if(Result) */
#if defined(TRACE_SERVER_ON)
    else
    {
        pTraceServer->tprintf(TRACE_Type_STATUS, "CTrapObjSvr", "RecvMsgHandler",
                                Family, Genus, Species, 
                                "Deserialize Failed");
    }
#endif

    return (Result ? TRUE : FALSE);
}
    
/*================================================================================
CTrapObjSvr SetErrorFunc: Registers the callback function for handling error messages
================================================================================*/
boolean CTrapObjSvr::SetErrorFunc(uint32 (*RxError)(char *, uint8, uint32, uint8*, void *))
{
    boolean Result = FALSE;
    if (RxError != NULL)
    {
        errorHandler = RxError;
        Result = TRUE;
    }
    return(Result);
}
/*================================================================================
CTrapObjSvr SetErrorFunc: Routes the callback function for handling error messages
================================================================================*/
boolean CTrapObjSvr::SetCallbacks(va_list vList)
{
    boolean Result = FALSE;

    uint32 (*RxError)(char *, uint8, uint32, uint8*, void*);
    uint32 *pRxError = (uint32*)&RxError;

    *pRxError = va_arg( vList, uint32);
    Result = SetErrorFunc(RxError);

    return(Result);
}

/* End of File */





