/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CUSBCommSvr.h  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined (CUSBCOMMSVR_H)
    #define   CUSBCOMMSVR_H

#include "Target.h"
#include "SysUtil.h"
#include "Common.h"

#include <process.h>
#include "CWACPMutex.h"
#include "CWACPEvent.h"
#include "CWACPSemaphore.h"

#include "WAUSBDLL.h"

typedef struct USBAddr
{
    DEVICE_DESC* pDevice;
}USBAddr;

typedef struct AsyncRequest
{
    uint32 EndpointID;
    uint8* pDataBuf;
}AsyncRequest;

class uint8QManager;

#define USB_THREAD_START "USB_START"

#define WACP_SEM_NAME "WACP_%d_%p"

class CUSBCommSvr
{
/*================================================================================
CUSBCommSvr Members: Public, Private, and Protected
================================================================================*/

    public:
    /* Constructor - Takes in two function pointers for pushing new connections 
     * and new messages up to the calling application 
     */
    CUSBCommSvr(boolean (*RecvMsg)(char* pSemName, char* pSourceID, uint8*, uint32 MsgSize, 
                   void* pNewConnection),                   
                   boolean (*TerminateClient)(char* pSemName, char* pSourceID, uint16 Port),
                   char* pAppName);
    CUSBCommSvr();
    ~CUSBCommSvr();
    
    /* Returns the SourceID of the CUSBCommSvr */
    char* GetSourceID();

    /* Returns the Port which represents either a RNDZ or WACP session */
    uint16 GetPortNum();

    /* Returns the type of stack that is being used ( i.e. Ethernet, 
     * RS232, USB, etc )
     */
    STACK_TYPE GetStackType();

    /* Sets up the configuration of the stack based on the data received 
     * via pAddr.
     */
    void SetConfig(void* pAddr);
    
    /* Returns the pointer to the configuration */
    void* GetConfig();

    /* Sends a binary Buffer */
    boolean SendMsg(uint8* pBuf, uint32 pBufSize);

    /* Creates a connection */
    boolean Connect(uint8* pLocalID, int PortNum);

    /* Handles the start of a new client connection thread */
    static uint32 WACPAPI StartClient(void* pData);

    /* Handles incoming connection requests */
    static uint32 WACPAPI StartServer(void* pData);

    /* Sets the communication port that distinguishes between WACP and RNDZ messages */
    boolean SetPortNum(uint16 NewPort);

    private:
        SetRecvBufPtr    pSetRecvMsg;
        TerminatePtr     pTerminate;

        boolean          SetSourceID(char* pSourceID);
        char             SourceID[ADDR_BUF_SIZE];
        uint16           Port;
        uint16           Protocol;

        STACK_TYPE       StackType;
        USBAddr          Addr;

        CWACPMutex*      pClientExitMutex;
        CWACPMutex*      pDeviceMutex;
        CWACPMutex*      pDisconnectMutex;
        
        CWACPEvent*      pThreadEvent;

        CWACPSemaphore*  pReadSem;
        volatile boolean bActive;

	    int32	         INEndPointNum;
	    int32	         OUTEndPoint;

        CWACPMutex*      pCacheMutex;
        uint8QManager*   pCacheQMgr;

        uint8            Cache[USB_CACHE_SIZE];
        volatile boolean bConnected;
    public:
        char*            pSemName;

        static uint32 WAUSB_CALLBACK RecvData(uint32 Status, char* pDevName, int32 EndPointID, 
                           void* pRecvData, int32 RecvLength, void*	pUserRecvData);

        static uint32 WAUSB_CALLBACK DeviceArrival(PDEVICE_DESC pDeviceDesc, void* pUserArrivalData);

        static uint32 WAUSB_CALLBACK DeviceRemoval(char* pDevName, void* pUserArrivalData);

        static uint32 WAUSB_CALLBACK DeviceNotify(PDEVICE_DESC pDeviceDesc, void* pUserArrivalData);

        boolean ReadData(char* pBuf, uint32 BytesToRead);
};


#endif /* #if !defined (CUSBCOMMSVR_H) */

/*---------------------------------------------------*/
/* End of File */





