/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPApp.h                                                        //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#if !defined(CWACPAPP_H)
    #define  CWACPAPP_H

#include "SysUtil.h"
#include "Common.h"
#include "CFmRNDZObjSvr.h"
#include "CRNDZStub.h"
#include "CLinkTable.h"
#include "CConObjSvr.h"
#include "CDataObjSvr.h"
#include "CWACPEvent.h"
#include "CWACPSemaphore.h"
#include "CWACPThread.h"
#include "CWACPMutex.h"
#include "CFmRouter.h"
#include "CTraceSvr.h"
#include "CTrapObjSvr.h"

#define TERMINATE_WACP_APP "EXIT_%s"

class CMOMPBaseClass;

class CWACPApp
{
/*================================================================================
CWACPApp Members: Public, Private, and Protected
================================================================================*/
friend class CInterchange;
friend class CRNDZStub;
friend class CMOMPBaseClass;
public:
    /*================================================================================
    CWACPApp Start up:
    ================================================================================*/
    CWACPApp(boolean (*SendPacket)(char*, char*, uint8*, uint32),
             uint32 (*RecvPacket)(char*, char**, uint8**),
             boolean (*MsgFunc)(char*, uint16, void*, uint16),
             CLinkTable *pNewCLinkTable,
             LinkStruct *pLinkStruct);
    ~CWACPApp();

    /* Connection APIs for Stubs and Dlls*/
    boolean Open(ConnArgs **pConnArgs, uint8 NumConnections);
    boolean Connect(char *pSourceID);
    boolean Close(char *pSourceID);

    /* Messaging APIs for Stubs and Dlls */
    boolean SendMsg(char *pSourceID, uint16 Family, uint8 Genus, uint8 Species, uint8 *pBuf, uint32 BufSize, boolean bEncryptionOn);
    boolean SendTrap(char *pSourceID, uint32 MsgClassID, uint32 ErrClassID, uint8 ErrorSpecies, uint16 ErrorCode, 
                     uint8 *pData, uint16 DataSize);
    boolean SendMsgProcTrap(char *pSourceID, uint32 ClassID, uint16 ErrorCode);
    boolean SetConnOptions(char *pSourceID, CONN_CFG_SPECIES Options, boolean bSyncFlag, time_t Interval, time_t TimeOut);
    boolean GetConnOptions(char *pSourceID, CONN_CFG_SPECIES *pOptions, time_t *pInterval, time_t *pTimeOut);

    /* Entry point for CExchange startup call */
    boolean AppStart();

    /* Public function for terminating the CWACPApp's AppStart routine for DLLs */
    void Terminate();

private:
    boolean             bTerminated;
    boolean             bHostFlag;
    boolean				ThreadStatus;
    CWACPEvent          *pStartEvent;
    CWACPEvent          *pExitEvent;
    LinkStruct          *pLnkStruct;
    CLinkTable          *pCLinkTable;
	CTrapObjSvr 		TrapObjSvr;
    CConObjSvr  		ConObjSvr;
    CTraceSvr   		TraceSvr;
    CDataObjSvr 		DataObjSvr;
    CFmRouter   		FmRouter;
    CFmRNDZObjSvr   	RendezvousSvr;
    CWACPMutex			DataThreadMutex;
    CRNDZStub           RNDZStub;
    CMOMPBaseClass      *pCallBackObj;
    CMOMPBaseClass      *pCMOMPBaseClass;
    RecvPacketFuncPtr   pRecvPacket;
    CWACPThread         DThread;
    boolean             bAppInit;

    /* Static Callback Functions that wrap the previous functions */
    static uint32 RxRNDZMessageWrapper(char *pSourceID, uint32 ClassID, uint8* pBuf, void *pWACPApp);
	static uint32 RxMessageWrapper(char *pSourceID, uint32 ClassID, uint8* pBuf, void *pWACPApp);
    static uint32 RxConnWrapper(char *pSourceID, uint8 ConnType, uint8* pBuf, void *pWACPApp);
    static uint32 RxErrorWrapper(char *pSourceID, uint8 ErrorType, uint32 ClassID, 
                                 uint8* pBuf, void *pWACPApp);

    /* Thread Proc and helper methods for handling incoming messages via a separate DataThread */
    static uint32 WACPAPI DataThread(void *pData);
    void StartDataThread();
    boolean ProcessMsg();
    boolean GetThreadStatus();

    /* Cleanup method */
    void DeleteObjects();

    /* Connection/Disconnect routines for RNDZ protocol */
    boolean RNDZConnect(char *pSourceID);
    void RNDZTerminate(char *pSourceID, boolean bStatus);

    boolean RegisterServer(FAMILY Family, CObjSvr* pObjSvr, ...);
    CRNDZDStd* GetGUID(char* pSourceID);
    STACK_TYPE GetStackType(char* pSourceID);
};

#endif /* #if !defined(CWACPAPP_H) */

/*---------------------------------------------------*/
/* End of File */

