/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Dt.cpp                                                            //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'** 
'*-E************************************************************************/


/*********************************************************************************************
Static methods to help manipulate DataClass identifiers and size.These are implemented as 
methods instead of macros because methods are type safe and do not
suffer from many of the side-effects of macros.
*********************************************************************************************/
#include "Dt.h"

/*********************************************************************************
Creates a DataClass identifier from the family, genus and species.
*********************************************************************************/
uint32 MakeClassID(uint16 Family, uint8 Genus, uint8 Species)
{
    return (((uint32)Family << 16) | ((uint16)Genus << 8) | Species);
}

/*********************************************************************************
Returns the family sub-code from an input identifier.
*********************************************************************************/
uint16 GetFamily(const uint32 Id)
{
    return (uint16)(Id >> 16);
}

/*********************************************************************************
Returns the family sub-code from an input identifier.
*********************************************************************************/
uint16 GetFamily (uint8 *pBuf)
{
#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    return (((uint16)pBuf[0]) << 8 | pBuf[1]);
#else
    return (((uint16)pBuf[3]) << 8 | pBuf[2]);
#endif   
}

/*********************************************************************************
Returns the genus sub-code from an input DataClass identifier.
*********************************************************************************/
uint8 GetGenus (const uint32 Id)
{
    return (uint8)(Id >> 8);
}

/*********************************************************************************
Returns the genus sub-code from an input DataClass identifier.
*********************************************************************************/
uint8 GetGenus(uint8 *pBuf)
{
#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    return (pBuf[2]);
#else
    return (pBuf[1]);
#endif   
}

/*********************************************************************************
Returns the species sub-code from the input class identifier.
*********************************************************************************/
uint8 GetSpecies(const uint32 Id)
{
    return (uint8)Id;
}

/*********************************************************************************
Returns the species sub-code from the input class identifier.
*********************************************************************************/
uint8 GetSpecies(uint8 *pBuf)
{
#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    return (pBuf[3]);
#else
    return (pBuf[0]);
#endif   
}

/* End of File */



