/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Dt.h                                                              //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'** 
'*-E************************************************************************/

/*********************************************************************************************
Static methods to help manipulate DataClass identifiers and size.These are implemented as 
methods instead of macros because methods are type safe and do not
suffer from many of the side-effects of macros.
*********************************************************************************************/

#if !defined(DT_H)
    #define  DT_H

#include <stdio.h>
#include <stdlib.h>
#include "Mem.h"
#include "StdVarTyp.h"

#ifndef DT_DLLAPI
    #if defined(_MOIB_EXPORT_DLLAPI_)
        #define DT_DLLAPI  __declspec( dllexport )
    #elif defined(_MOIB_IMPORT_DLLAPI_)
        #define DT_DLLAPI  __declspec( dllimport )
    #else
        #define DT_DLLAPI
    #endif
#endif

/*********************************************************************************
Creates a DataClass identifier from the family, genus and species.
*********************************************************************************/
uint32 DT_DLLAPI MakeClassID(uint16 Family, uint8 Genus, uint8 Species);

/*********************************************************************************
Returns the family sub-code from an input identifier.
*********************************************************************************/
uint16 DT_DLLAPI GetFamily(const uint32 Id);

/*********************************************************************************
Returns the family sub-code from a serialized byte stream.
*********************************************************************************/
uint16 DT_DLLAPI GetFamily (uint8 *pBuf);

/*********************************************************************************
Returns the genus sub-code from an input DataClass identifier.
*********************************************************************************/
uint8 DT_DLLAPI GetGenus (const uint32 Id);

/*********************************************************************************
Returns the genus sub-code from a serialized byte stream.
*********************************************************************************/
uint8 DT_DLLAPI GetGenus(uint8 *pBuf);

/*********************************************************************************
Returns the species sub-code from the input class identifier.
*********************************************************************************/
uint8 DT_DLLAPI GetSpecies(const uint32 Id);

/*********************************************************************************
Returns the species sub-code from a serialized byte stream.
*********************************************************************************/
uint8 DT_DLLAPI GetSpecies(uint8 *pBuf);

#endif  /* #if !defined(DT_H) */

/*---------------------------------------------------*/
/* End of File */




