/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// DtData.h                                                          //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'** 
'*-E************************************************************************/

#if !defined(DTDATA_H)
    #define  DTDATA_H

//Corporate Include files.
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Dt.h"
#include "Mem.h"
#include "StdVarTyp.h"  
#include "DtSupport.h"
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
#include "xmlparse.h"
#endif 
#include "SysUtil.h"
#include "MiscUtil.h"

//Macros for the identification string that appears in XML.
//These string tables are defined in DtData, and indexed by
//the current class family, genus, and species values 
//respectively.
#define FAMILY_STRING   FmStringTable[family()]
#define GENUS_STRING    GnStringTable[genus()]

#define DIRTY_FLAG_MASK 0x80          /*[1000 0000]*/
#define VALID_FLAG_MASK 0x40          /*[0100 0000]*/
#define DIRTY_VALID_FLAG_MASK 0xC0    /*[1100 0000]*/

/* Data Structures */

typedef struct t_MsgEntry {    /* -- Message entry -- */
    uint16 msg_id;             /* Message id */
    ansichar * msg_text;       /* Message text */
} MsgEntry;

typedef struct t_MsgTable {    /* -- Message table -- */
    MsgEntry * msg_list;       /* Message list */
    uint16 msg_entry_count;    /* Number of list entries */
} MsgTable;

typedef struct t_VersionEntry {    /* -- Version entry -- */
    uint16 min_version;            /* Minimum Version */
    uint16 max_version;            /* Maximum Version */
} VersionEntry;

#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
typedef struct XMLParserData{
    uint32 Depth;
    void * Object;
    void * ParentObject;
}XMLParserData;
#endif

#ifndef DTDATA_DLLAPI
    #if defined(_MOIB_EXPORT_DLLAPI_)
        #define DTDATA_DLLAPI  __declspec( dllexport )
    #elif defined(_MOIB_IMPORT_DLLAPI_)
        #define DTDATA_DLLAPI  __declspec( dllimport )
    #else
        #define DTDATA_DLLAPI
    #endif
#endif

/* Function Prototype */
DTDATA_DLLAPI ansichar *GetMsgString(MsgEntry * msg_list, uint16 msg_entry_count, uint16 msg_id);
DTDATA_DLLAPI boolean IsValidEnum(MsgEntry *msg_list, uint16 msg_entry_count, uint16 msg_id);

class DTDATA_DLLAPI DtData
{

public:
    DtData()
    {
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
        CurrentParsePos = NULL;
        returnObject = this;
        returnDepth = 0;
        pClassAttr = NULL;
        validXMLParseFlag = TRUE;
#endif
    };

    DtData(uint32 cid) : classIdentifier ( cid ) 
    {
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
        CurrentParsePos = NULL;
        returnObject = this;
        returnDepth = 0;
        pClassAttr = NULL;
        validXMLParseFlag = TRUE;
#endif
    };
    virtual ~DtData()
    {
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
        CurrentParsePos = NULL;
        returnObject = NULL;
        ResetXMLAttr();
#endif
    };
    
    /*********************************************************************************
    Returns the class identifier. 
    *********************************************************************************/
    uint32 id();
    /*********************************************************************************
    Returns the family sub-code of the class identifier.
    *********************************************************************************/
    uint16 family();
    /*********************************************************************************
    Set function for family
    *********************************************************************************/
    void set_Family(uint16 new_Family);
    /*********************************************************************************
    Returns the genus sub-code of the class identifier.
    *********************************************************************************/
    uint8 genus();
    /*********************************************************************************
    Set function for genus
    *********************************************************************************/
    void set_Genus(uint8 new_Genus);
    /*********************************************************************************
    Returns the species sub-code of the class identifier.
    *********************************************************************************/
    uint8 species();
    /*********************************************************************************
    Set function for species
    *********************************************************************************/
    void set_Species(uint8 new_Species);
    /*********************************************************************************
    Set function for the SrcVersion data member
    *********************************************************************************/
    void SetSrcVersion(uint16 new_SrcVersion);
    /*********************************************************************************
    Returns the SrcVersion 
    *********************************************************************************/
    uint16 GetSrcVersion();
    /*********************************************************************************
    Set function for the DataBitField data member
    *********************************************************************************/
    void SetDataBitField(uint8 new_DataBitField);
    /*********************************************************************************
    Returns the DataBitField
    *********************************************************************************/
    uint8 GetDataBitField();
    /*********************************************************************************
    Returns a boolean value representing the state of the dirtyFlag bit
    *********************************************************************************/
    boolean isdirty();
    /*********************************************************************************
    Sets the dirtyFlag bit 
    *********************************************************************************/
    void setdirty(boolean new_dirtyFlag);
    /*********************************************************************************
    Returns a boolean value representing the state of the validFlag bit
    *********************************************************************************/
    boolean isvalid();
    /*********************************************************************************
    Sets the valid data bit
    *********************************************************************************/
    void setvalid(boolean new_validFlag);
    /*********************************************************************************
    Serializes a MOIB object into a byte stream
    *********************************************************************************/
    uint32 serialize_all ( uint8 *pBuffer );
    /*********************************************************************************
    Deserializes a byte stream into a MOIB object
    *********************************************************************************/
    uint32 deserialize_all ( const uint8 *pBuffer );
    /*********************************************************************************
    Returns the total number of Bytes that would be used by the serialize_all or
    deserialize_all methods.
    *********************************************************************************/
    uint32 size_all ( void ); 
    /*********************************************************************************
    Helper function to enqueue MOIB_TIME struct
    *********************************************************************************/
    void Put_MOIBTIME(uint8QManager *QMgr, MOIB_TIME sDateTime);
    /*********************************************************************************
    Helper function to dequeue MOIB_TIME struct
    *********************************************************************************/
    MOIB_TIME Get_MOIBTIME(uint8QManager *QMgr);
    /*********************************************************************************
    Helper Function:
    Peforms a comparison between the SrcVersion and the version span in the 
    VersionEntry table. Returns TRUE if the SrcVersion falls within the version span
    *********************************************************************************/
    boolean GetVersionStatus(VersionEntry * pEntryList, uint16 EntryCount, uint16 MemberEnum);
    /*********************************************************************************
    Helper Function to return TRUE if a serialized object has a uint32 size identifier
    *********************************************************************************/
    virtual boolean is_Extended(uint8* pBuffer);
    /*********************************************************************************
    Helper Function to return TRUE if a serialized object has a uint32 size identifier
    *********************************************************************************/
    virtual boolean is_Extended(uint32 ClassID);
    /*********************************************************************************
    Helper Function to calculate a serialized object's size
    *********************************************************************************/
    uint32 MOIBSerialSize(uint8* pBuffer, uint8* SizeWidthOut);
    /*********************************************************************************
    Helper Function to calculate the Number of Elements(Serialized Objects) in a 
    byte stream.
    *********************************************************************************/
    uint16 MOIBSerialElements(uint8* pBuffer, uint32 Size);
    /*********************************************************************************
    Virtual Function: Returns the object version
    *********************************************************************************/
    virtual uint16 getVersion();
     /*********************************************************************************
    Virtual Function: Returns number of bytes written to XML Stream
    *********************************************************************************/
    virtual uint32 serialize_to_XMLbuf(char *buf);
    /*********************************************************************************
    Returns the total number of Bytes that would be XML stream
    *********************************************************************************/
    uint32 size_XML(void);
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
    /*********************************************************************************
    Helper Function for meta data types (not used for low level objects).
    *********************************************************************************/
    uint32 deserialize_from_XMLbuf(const char *buf, uint32 Size = 0xFFFF);
    /*********************************************************************************
    XML HELPER FUNCTION
    The SetClassAttr is used to store the value of the class attribute 
    that will be used by the parse_StartElement().
    *********************************************************************************/
    void SetClassAttr(char *pNewClassAttr);
    /*********************************************************************************
    XML HELPER FUNCTION
    The GetClassAttr is used to retrieve a value of a class attribute that will be 
    used by the parse_StartElement().
    *********************************************************************************/
    char *GetClassAttr();
    /*********************************************************************************
    XML HELPER FUNCTION
    The ResetXMLAttr is used free the XML Attribute Member pClassAttr 
    *********************************************************************************/
    void ResetXMLAttr();
    /*********************************************************************************
    XML HELPER FUNCTION
    Used to Set the returnObject and returnDepth
    *********************************************************************************/
    void parse_SetReturnObject(void *retObj, uint32 retDepth)
    {returnObject = (retObj ? retObj : this); returnDepth = retDepth;};
#endif  /*#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))*/

private:
    
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
    /*********************************************************************************
    Virtual Function: Called by Callback function parse_StartElementWrapper when start
    element tag is encountered
    *********************************************************************************/
    virtual void parse_StartElement(void *UserData, const XML_Char *Name, const XML_Char **Atts);
    /*********************************************************************************
    Called by Callback function parse_EndElementWrapper when and end element tag is
    encountered
    *********************************************************************************/
    void parse_EndElement(void *UserData, const XML_Char *Name);
    /*********************************************************************************
    Virtual Function: Called by Callback function parse_CharacterDataWrapper when 
    character data is encountered
    *********************************************************************************/
    virtual void parse_CharacterData(void *UserData, const XML_Char *sTag, int Len);
#endif
    
protected:
    /*********************************************************************************
    Virtual Function: Returns the number of Bytes that would be used by the 
    serialize_payload and deserialize_payload methods.
    *********************************************************************************/
    virtual uint32 get_payload_size ( void );
    /*********************************************************************************
    Virtual Function: Serializes the payload of a MOIB object into a byte stream.
    Excluding the header 
    *********************************************************************************/
    virtual boolean serialize_payload ( uint8QManager * QMgr );
    /*********************************************************************************
    Virtual Function: Deserializes the payload of a MOIB object into a byte stream.
    Excluding the header 
    *********************************************************************************/
    virtual boolean deserialize_payload ( uint8QManager * QMgr, uint32 Sz );
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
public:
    /*********************************************************************************
    CallBack Function: XML parser callback when a start tag is encountered
    *********************************************************************************/
    static void parse_StartElementWrapper(void *UserData, const XML_Char *Name, const XML_Char **Atts)
    {
        ((DtData*)((XMLParserData*)UserData)->Object)->parse_StartElement(((XMLParserData*)UserData), Name, Atts);
    };
    /*********************************************************************************
    CallBack Function: XML parser callback when a end tag is encountered
    *********************************************************************************/
    static void parse_EndElementWrapper(void *UserData, const XML_Char *Name)
    {
        ((DtData*)((XMLParserData*)UserData)->Object)->parse_EndElement(((XMLParserData*)UserData), Name);
    };
    /*********************************************************************************
    CallBack Function: XML parser callback when a character data is encountered
    *********************************************************************************/
    static void parse_CharacterDataWrapper(void *UserData, const XML_Char *sTag, int Len)
    {
        ((DtData*)((XMLParserData*)UserData)->Object)->parse_CharacterData(&((XMLParserData*)UserData)->Depth, sTag, Len);
    };
#endif
protected:
    
    /*---------------------------------
    DtData Members
    ----------------------------------*/  
    byte    DataBitField;
    uint16  SrcVersion;       
    uint32  classIdentifier; 
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE)||(TARGET_XML_SUPPORT == XML_ALL))
    boolean validXMLParseFlag;    
    uint32  returnDepth;
    char    *pClassAttr;
    void    *CurrentParsePos;
    void    *returnObject;
#endif             
    static const char   *FmStringTable[FmMAX];
    static const char   *GnStringTable[GnMAX];   
};

#endif /* #if !defined(DTDATA_H) */

/*---------------------------------------------------*/
/* End of File */





