/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Mem.h                                                             //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOIB Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'** 
'*-E************************************************************************/

#if !defined(MEM_H)
    #define  MEM_H

#include "Target.h"
#include "StdVarTyp.h"

#ifndef MEM_DLLAPI
    #if defined(_MOIB_EXPORT_DLLAPI_)
        #define MEM_DLLAPI  __declspec( dllexport )
    #elif defined(_MOIB_IMPORT_DLLAPI_)
        #define MEM_DLLAPI  __declspec( dllimport )
    #else
        #define MEM_DLLAPI
    #endif
#endif


/***********************************************************************************
This class is based upon the templated QManager and is used specifically for uint8s.
This class is specifically provided to add methods to make it easy to serialize and
deserialize DataClasses, though it may be used for any uint8 buffer that needs to be
manipulated as a circular Q.
***********************************************************************************/
class MEM_DLLAPI uint8QManager
{
public:
    /***************************************************************************
    Constructor. Passes the parameters to the QManager constructor.
    ***************************************************************************/
    uint8QManager(void);
    uint8QManager(uint8* start, uint32 elements, boolean qFull = FALSE);
    boolean enqueue(const uint8 & object);
    boolean dequeue(uint8* object);
    boolean enqueue_array(uint8* array, uint32 elements);
    boolean dequeue_array(uint8* array, uint32 elements);
    boolean dequeue_array(uint32 elements);
    uint32 get_num_free(void);
    uint32 get_num_used(void);
    uint32 get_num_elements(void);
    boolean put(uint8 val);
    boolean get(uint8* val);
    boolean put(int8 val);
    boolean get(int8* val);
    boolean put(uint16 val);
    boolean get(uint16* val);
    boolean put(int16 val);
    boolean get(int16* val);

#if defined(WIDE_CHAR_SUPPORT)
    boolean put (widechar val);
    boolean get (widechar* val);
#endif

    boolean put(uint32 val);
    boolean get(uint32* val);
    boolean put(int32 val);
    boolean get(int32* val);
    boolean put(float val);
    boolean get(float* val);
    boolean put(double val);
    boolean get(double* val);
    void* GetHeadPtr();
    void* GetTailPtr();

private:
    uint8* bufStart;
    uint8* bufEnd;
    uint8* head;
    uint8* tail;
    uint32 numElements;
    uint32 elementsUsed;
};

/* Endian proof helper functions for writing/reading an integer */
uint16  MEM_DLLAPI ReadInt16(uint8* pBuf);
uint32  MEM_DLLAPI ReadInt32(uint8* pBuf);
void    MEM_DLLAPI WriteInt16(uint8* pBuf, uint16 Val);
void    MEM_DLLAPI WriteInt32(uint8* pBuf, uint32 Val);

#endif  /* #if !defined(MEM_H) */

/*---------------------------------------------------*/
/* End of File */
