/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// SysUtil.cpp    :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "SysUtil.h"
#include "target.h"

#if defined (WIN32)
#include <windows.h>
#include <process.h>
#include <time.h>

#if defined(MEMORY_DEBUG)
#include "CWACPMutex.h"

#define MEMORY_TXT  "memory.txt"

static uint32   MallocCount;
FILE*           pMemFile;
CWACPMutex      FileMutex;

void* sys_malloc(MEMORY_POOL Pool, size_t Size, char* pName, int Line )
{   
    void* pTemp;
    TOUCH(Pool);
    
    pTemp = malloc(Size);
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    if (pMemFile)
    {
        fprintf(pMemFile, "sys_malloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, ++MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }
    FileMutex.Release();

    return pTemp;
}

void* sys_calloc(MEMORY_POOL Pool, size_t num, size_t Size, char* pName, int Line  )
{
    void* pTemp;
    TOUCH(Pool);
    
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    pTemp = calloc(num,Size);

    if (pMemFile)
    {
        fprintf(pMemFile, "sys_calloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, ++MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }
    FileMutex.Release();

    return pTemp;
}

void sys_free( MEMORY_POOL Pool, void* pMemBlock , char* pName, int Line )
{
    TOUCH(Pool);
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    if ((pMemFile) && (pMemBlock))
    {
        fprintf(pMemFile, "sys_free\t %p\t %d\t \t %s\t Line %d\n", pMemBlock, --MallocCount, pName, Line);
        fflush(pMemFile);
        free(pMemBlock);
    }
    FileMutex.Release();
}

void* sys_realloc( MEMORY_POOL Pool, void* pMemBlock, size_t Size, char* pName, int Line   )
{
    void* pTemp;

    TOUCH(Pool);

    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    if ((pMemBlock) && (pMemFile))
    {
        fprintf(pMemFile, "clr_free\t %p\t %d\t \t %s\t Line %d\n", pMemBlock, MallocCount, pName, Line);
    }
    else
    {
        MallocCount++;
    }

    pTemp = realloc(pMemBlock, Size);

    if (pMemFile) 
    {
        fprintf(pMemFile, "sys_realloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }
    FileMutex.Release();

    return pTemp;
}
#else
void* sys_malloc( MEMORY_POOL Pool, size_t Size)
{   
    TOUCH(Pool);
    return malloc(Size);
}

void* sys_calloc( MEMORY_POOL Pool, size_t num, size_t Size)
{
    TOUCH(Pool);
    return calloc(num,Size);
}

void sys_free( MEMORY_POOL Pool, void* pMemBlock)
{
    TOUCH(Pool);
    if (pMemBlock)
    {
       free(pMemBlock);
       pMemBlock = NULL;
    }
}

void* sys_realloc(MEMORY_POOL Pool, void* pMemBlock, size_t Size)
{
    TOUCH(Pool);
    return realloc(pMemBlock, Size);
}

#endif

void  sys_sleep(uint32 TimeOut)
{
    Sleep(TimeOut);
}

int sys_getpid(void)
{
    return (_getpid());
}

time_t sys_time(time_t* pTimer)
{
    return (time(pTimer));
}

/*
 * Abstractions for C pseudorandom number functions.
 *
 * void srand( unsigned int seed );
 * int rand( void );
 */

/* Set a random starting point. */
void sys_srand(uint32 Seed)
{
    srand(Seed);
}

/* Generate and return a pseudorandom number. */
int sys_rand(void)
{
    return(rand());
}

/* Abstraction for printf function*/
int sys_printf(const char* pFormat, ...)
{
    va_list(vargs);
    va_start(vargs, pFormat);

    return vprintf(pFormat, vargs);
}

#endif
/* End of File */





