/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// SysUtil.h                                                         //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'** 
'*-E************************************************************************/

#if !defined(SYSUTIL_H)
    #define  SYSUTIL_H

#include "StdVarTyp.h"
#include <time.h>

typedef enum tagMemoryPool {
    GENERAL_POOL,
    OBJECT_POOL,
    BUFFER_POOL
} MEMORY_POOL;

void sys_mempool_init(void);

/*
 * Abstractions for memory allocations functions
 *
 * void *malloc( size_t Size );
 * void *calloc( size_t Num, size_t Size );
 * void free( void *pMemBlock );
 * void *realloc( void *pMemBlock, size_t Size );
 */
#if defined(MEMORY_DEBUG)
    void *sys_malloc(MEMORY_POOL Pool, size_t Size, char *, int);
    void *sys_calloc(MEMORY_POOL Pool, size_t Num, size_t Size, char *, int);
    void sys_free(MEMORY_POOL Pool, void *pMemBlock, char *, int);
    void *sys_realloc(MEMORY_POOL Pool, void *pMemBlock, size_t Size, char *, int);

    #define DFLT_POOL   GENERAL_POOL
    #define SYS_MALLOC(A)       sys_malloc(DFLT_POOL, (A), __FILE__, __LINE__)
    #define SYS_CALLOC(A,B)     sys_calloc(DFLT_POOL, (A), (B), __FILE__, __LINE__)
    #define SYS_FREE(A)         sys_free(DFLT_POOL, (A), __FILE__, __LINE__)
    #define SYS_REALLOC(A,B)    sys_realloc(DFLT_POOL, (A), (B), __FILE__, __LINE__)

#else
    void *sys_malloc(MEMORY_POOL Pool, size_t Size);
    void *sys_calloc(MEMORY_POOL Pool, size_t Num, size_t Size);
    void sys_free(MEMORY_POOL Pool, void *pMemBlock);
    void *sys_realloc(MEMORY_POOL Pool, void *pMemBlock, size_t Size);

    #define DFLT_POOL   GENERAL_POOL
    #define SYS_MALLOC(A)       sys_malloc(DFLT_POOL, (A))
    #define SYS_CALLOC(A,B)     sys_calloc(DFLT_POOL, (A), (B))
    #define SYS_FREE(A)         sys_free(DFLT_POOL, (A))
    #define SYS_REALLOC(A,B)    sys_realloc(DFLT_POOL, (A), (B))
#endif

/*
 * Abstractions for Windows functions
 *
 * Pauses the current thread.
 * void SYS_SLEEP(int);
 */

void sys_sleep(uint32 Msecs);

#define SYS_SLEEP(A)    sys_sleep(A)

/*
 * Abstractions for Windows functions
 *
 * Get the process identification.
 * int sys_getpid(void);
 */
int  sys_getpid(void);

#define SYS_GETPID  sys_getpid

/*
 * Abstraction for time() function.
 *
 * Get system time.
 * time() returns the time in elapsed seconds. There is no error return.
 * time_t time( time_t *pTimer );
 */
time_t sys_time(time_t *pTimer);

#define SYS_TIME(A)     sys_time(A)

/*
 * Abstractions for pseudorandom number functions.
 *
 * void srand(uint32 seed);
 * int rand(void);
 */
/*
 * Set a random starting point.
 */
void sys_srand(uint32 Seed);

#define SYS_SRAND(A)    sys_srand(A)

/*
 * Generates and returns a pseudorandom number.
 * Note that the macro casts the returned value as a uint8.
 */
int sys_rand(void);

#define SYS_RAND        (uint8)sys_rand

/*
 * Abstraction for printf function
 */
int sys_printf(const char *pFormat, ...);

#define SYS_PRINTF       sys_printf


#if defined(_WIN32_WCE) && defined(_WCE_CONSOLE)

typedef int (*formatter)(const char*, va_list vargs);

extern formatter _newprintf;
extern formatter _newscanf;

int scanf(const char* format, ...);

int WinEntry (void *Args);

#endif /* defined(_WIN32_WCE) && defined(_WCE_CONSOLE) */

#endif /* #if !defined(SYSUTIL_H) */

/*---------------------------------------------------*/
/* End of File */






